@@ -1,28 +1,29 @@
 #!/usr/bin/env bash
 #* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 #*                                                                           *
-#*                  This file is part of the program                         *
+#*                  This file is part of the program and library             *
 #*          GCG --- Generic Column Generation                                *
 #*                  a Dantzig-Wolfe decomposition based extension            *
 #*                  of the branch-cut-and-price framework                    *
 #*         SCIP --- Solving Constraint Integer Programs                      *
 #*                                                                           *
-#* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       *
+#* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       *
 #*                         Zuse Institute Berlin (ZIB)                       *
 #*                                                                           *
-#* This program is free software; you can redistribute it and/or             *
-#* modify it under the terms of the GNU Lesser General Public License        *
-#* as published by the Free Software Foundation; either version 3            *
-#* of the License, or (at your option) any later version.                    *
+#*  Licensed under the Apache License, Version 2.0 (the "License");          *
+#*  you may not use this file except in compliance with the License.         *
+#*  You may obtain a copy of the License at                                  *
 #*                                                                           *
-#* This program is distributed in the hope that it will be useful,           *
-#* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
-#* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
-#* GNU Lesser General Public License for more details.                       *
+#*      http://www.apache.org/licenses/LICENSE-2.0                           *
 #*                                                                           *
-#* You should have received a copy of the GNU Lesser General Public License  *
-#* along with this program; if not, write to the Free Software               *
-#* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*
+#*  Unless required by applicable law or agreed to in writing, software      *
+#*  distributed under the License is distributed on an "AS IS" BASIS,        *
+#*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. *
+#*  See the License for the specific language governing permissions and      *
+#*  limitations under the License.                                           *
+#*                                                                           *
+#*  You should have received a copy of the Apache-2.0 license                *
+#*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*
 #*                                                                           *
 #* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 #
@@ -256,7 +257,6 @@ do
             echo set limits nodes $NODELIMIT       >> $TMPFILE
             echo set limits memory $MEMLIMIT       >> $TMPFILE
             echo set lp advanced threads $THREADS  >> $TMPFILE
-            echo set timing clocktype 1            >> $TMPFILE
             echo set display verblevel 4           >> $TMPFILE
             echo set display freq $DISPFREQ        >> $TMPFILE
             if test $STATISTICS = "true"
