@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -48,17 +49,18 @@
 // #define SCIP_DEBUG
 
 #include "scip/scipdefplugins.h"
-#include "gcg.h"
+#include "gcg/gcg.h"
 #include "objscip/objscip.h"
 #include "scip/scip.h"
-#include "class_detprobdata.h"
-#include "struct_detector.h"
-#include "pub_decomp.h"
-#include "struct_decomp.h"
-#include "cons_decomp.h"
-#include "decomp.h"
-#include "miscvisualization.h"
-#include "scip_misc.h"
+#include "gcg/class_detprobdata.h"
+#include "gcg/struct_detector.h"
+#include "gcg/pub_decomp.h"
+#include "gcg/struct_decomp.h"
+#include "gcg/cons_decomp.h"
+#include "gcg/cons_decomp.hpp"
+#include "gcg/decomp.h"
+#include "gcg/miscvisualization.h"
+#include "gcg/scip_misc.h"
 #include "scip/clock.h"
 #include "scip/cons.h"
 #include "scip/scip.h"
@@ -74,7 +76,7 @@
 #include <random> /* needed for exponential distributed random dual variables */
 #include <set>
 
-#include "reader_gp.h"
+#include "gcg/reader_gp.h"
 
 
 #ifdef _OPENMP
@@ -227,16 +229,27 @@ void DETPROBDATA::getTranslatedPartialdecs(
    std::vector<int>& rowthistoother,
    std::vector<int>& colothertothis,
    std::vector<int>& colthistoother,
-   std::vector<PARTIALDECOMP*>& translatedpartialdecs
+   std::vector<PARTIALDECOMP*>& translatedpartialdecs,
+   SCIP_Bool translatesymmetry
    )
 {
-   for(auto otherpartialdec : origpartialdecs)
+   DETPROBDATA* origdetprobdata = NULL;
+
+   if( translatesymmetry )
+   {
+      // even if presolving is disabled some vars might be fixed to 0
+      // @todo: we could check if symmetry is still valid
+      origdetprobdata = GCGconshdlrDecompGetDetprobdataOrig(gcg);
+      if( origdetprobdata->getNConss() != getNConss() || origdetprobdata->getNVars() != getNVars() )
+         translatesymmetry = FALSE;
+   }
+   for( auto otherpartialdec : origpartialdecs )
    {
       PARTIALDECOMP* newpartialdec;
 
       SCIPverbMessage(this->scip, SCIP_VERBLEVEL_FULL, NULL, " transform partialdec %d \n", otherpartialdec->getID());
 
-      newpartialdec = new PARTIALDECOMP(scip, original);
+      newpartialdec = new PARTIALDECOMP(gcg, original);
 
       /* prepare new partialdec */
       newpartialdec->setNBlocks(otherpartialdec->getNBlocks());
@@ -265,6 +278,15 @@ void DETPROBDATA::getTranslatedPartialdecs(
          }
       }
 
+      auto& blockstructures = otherpartialdec->getBlockStructures();
+      for( int b = 0; b < (int)blockstructures.size(); ++ b )
+      {
+         if( blockstructures[b] )
+            newpartialdec->setBlockStructure(b, blockstructures[b]->translateStructure(rowothertothis, colothertothis, translatesymmetry));
+         else
+            newpartialdec->setBlockStructure(b, NULL);
+      }
+
       // we do not assign variables as the previous assignment might be invalid due to presolving
 
       newpartialdec->setDetectorchain(otherpartialdec->getDetectorchain());
@@ -292,12 +314,38 @@ void DETPROBDATA::getTranslatedPartialdecs(
       otherpartialdec->setTranslatedpartialdecid(newpartialdec->getID());
 
       if( otherpartialdec->getFinishedByFinisher() )
-         newpartialdec->setDetectorFinishedOrig(otherpartialdec->getDetectorchain()[otherpartialdec->getNDetectors() - 1]);
+         newpartialdec->setDetectorFinishedOrig();
 
       newpartialdec->setFinishedByFinisher(otherpartialdec->getFinishedByFinisher());
       newpartialdec->prepare();
 
-      newpartialdec->getScore(GCGgetCurrentScore(scip)) ;
+      if( translatesymmetry && otherpartialdec->getNBlocks() == newpartialdec->getNBlocks() && otherpartialdec->aggInfoCalculated() )
+      {
+         newpartialdec->setSymmetryInformation(
+            [otherpartialdec] (int b)
+            {
+               return otherpartialdec->getReprBlockForEqClass(otherpartialdec->getEqClassForBlock(b));
+            },
+            [&colothertothis, &colthistoother, otherpartialdec, newpartialdec] (int b, int vi)
+            {
+               int v = newpartialdec->getVarsForBlock(b)[vi];
+               int eqclass = otherpartialdec->getEqClassForBlock(b);
+               int rb = otherpartialdec->getReprBlockForEqClass(eqclass);
+               auto& eqclassblocks = otherpartialdec->getBlocksForEqClass(eqclass);
+               assert(std::lower_bound(eqclassblocks.begin(), eqclassblocks.end(), b) != eqclassblocks.end());
+               int eqclassblock = (int) (std::lower_bound(eqclassblocks.begin(), eqclassblocks.end(), b) - eqclassblocks.begin());
+               assert(std::find(colthistoother.begin(), colthistoother.end(), v) != colthistoother.end());
+               int othervi = otherpartialdec->getVarsForBlock(b)[vi] == colthistoother[v] ? vi : otherpartialdec->getVarProbindexForBlock(colthistoother[v], b);
+               int blockVarIndex = otherpartialdec->getRepVarmap(eqclass, eqclassblock)[othervi];
+               if( newpartialdec->getVarsForBlock(rb)[blockVarIndex] == colothertothis[otherpartialdec->getVarsForBlock(rb)[blockVarIndex]] )
+                  return blockVarIndex;
+               else
+                  return newpartialdec->getVarProbindexForBlock(colothertothis[otherpartialdec->getVarsForBlock(rb)[blockVarIndex]], rb);
+            }
+         );
+      }
+
+      newpartialdec->getScore(GCGgetCurrentScore(gcg)) ;
 
       translatedpartialdecs.push_back(newpartialdec);
    }
@@ -305,11 +353,11 @@ void DETPROBDATA::getTranslatedPartialdecs(
 
 
 DETPROBDATA::DETPROBDATA(
-   SCIP* givenScip,
+   GCG* gcgstruct,
    SCIP_Bool _originalProblem
    ) :
-      scip(givenScip), openpartialdecs(0), ancestorpartialdecs( 0 ), origfixedtozerovars(0),
-      nvars(SCIPgetNVars(givenScip)), nconss(SCIPgetNConss(givenScip)), nnonzeros(0), original(_originalProblem), candidatesNBlocks(0),
+      gcg(gcgstruct), scip(GCGgetOrigprob(gcg)), openpartialdecs(0), ancestorpartialdecs( 0 ), origfixedtozerovars(0),
+      nvars(SCIPgetNVars(GCGgetOrigprob(gcg))), nconss(SCIPgetNConss(GCGgetOrigprob(gcg))), nnonzeros(0), original(_originalProblem), candidatesNBlocks(0),
       classificationtime(0.), nblockscandidatescalctime(0.), postprocessingtime(0.), translatingtime(0.)
 {
    SCIP_CONS** conss;
@@ -457,7 +505,7 @@ DETPROBDATA::~DETPROBDATA()
    }
 
    // Delete all partialdecs
-   GCGconshdlrDecompDeregisterPartialdecs(scip, original);
+   GCGconshdlrDecompDeregisterPartialdecs(gcg, original);
 
    for( size_t i = 0; i < conspartitioncollection.size(); ++ i )
    {
@@ -782,6 +830,7 @@ int DETPROBDATA::getIndexForVar(
    SCIP_VAR* var
    )
 {
+   assert(var != NULL);
    return vartoindex[var];
 }
 
@@ -888,16 +937,22 @@ SCIP* DETPROBDATA::getScip()
 }
 
 
+GCG* DETPROBDATA::getGcg()
+{
+   return gcg;
+}
+
+
 void DETPROBDATA::getSortedCandidatesNBlocks(
    std::vector<int>& candidates
    )
 {
-   int nusercandidates = GCGconshdlrDecompGetNBlockNumberCandidates(scip);
+   int nusercandidates = GCGconshdlrDecompGetNBlockNumberCandidates(gcg);
    /* get the block number candidates directly given by the user */
    SCIPdebugMessage("number of user block number candidates: %d\n", nusercandidates);
    for( int i = 0; i < nusercandidates; ++i )
    {
-      int candidate = GCGconshdlrDecompGetBlockNumberCandidate(scip, i);
+      int candidate = GCGconshdlrDecompGetBlockNumberCandidate(gcg, i);
       candidates.push_back(candidate);
       SCIPdebugMessage("  %d\n", candidate);
    }
@@ -1082,13 +1137,10 @@ bool DETPROBDATA::isConsSetpp(
 
 
 SCIP_Bool DETPROBDATA::isFiniteNonnegativeIntegral(
-   SCIP*                 givenScip,          /**< SCIP data structure */
    SCIP_Real             x                   /**< value */
    )
 {
-   assert(scip != NULL);
-
-   return (!SCIPisInfinity(givenScip, x) && !SCIPisNegative(givenScip, x) && SCIPisIntegral(givenScip, x));
+   return (!SCIPisInfinity(scip, x) && !SCIPisNegative(scip, x) && SCIPisIntegral(scip, x));
 }
 
 
@@ -1107,15 +1159,14 @@ SCIP_Bool DETPROBDATA::isAssignedToOrigProb()
 
 
 SCIP_Bool DETPROBDATA::isRangedRow(
-   SCIP*                 givenScip,               /**< SCIP data structure */
    SCIP_Real             lhs,
    SCIP_Real             rhs
    )
 {
    assert(scip != NULL);
 
-   return !(SCIPisEQ(givenScip, lhs, rhs)
-      || SCIPisInfinity(givenScip, -lhs) || SCIPisInfinity(givenScip, rhs) );
+   return !(SCIPisEQ(scip, lhs, rhs)
+      || SCIPisInfinity(scip, -lhs) || SCIPisInfinity(scip, rhs) );
 }
 
 
@@ -1141,20 +1192,19 @@ SCIP_Bool DETPROBDATA::partialdecIsNoDuplicateOfPartialdecs(
 
 
 void DETPROBDATA::printBlockcandidateInformation(
- SCIP*                 givenscip,               /**< SCIP data structure */
  FILE*                 file                /**< output file or NULL for standard output */
    )
 {
 
    std::sort( candidatesNBlocks.begin(), candidatesNBlocks.end(), sort_decr() );
-   SCIPmessageFPrintInfo(SCIPgetMessagehdlr(givenscip), file, "NBLOCKCANDIDATES   \n" );
-   SCIPmessageFPrintInfo(SCIPgetMessagehdlr(givenscip), file, "The following %d candidates for the number of blocks are known: (candidate : number of votes)   \n", (int) candidatesNBlocks.size() );
+   SCIPmessageFPrintInfo(SCIPgetMessagehdlr(scip), file, "NBLOCKCANDIDATES   \n" );
+   SCIPmessageFPrintInfo(SCIPgetMessagehdlr(scip), file, "The following %d candidates for the number of blocks are known: (candidate : number of votes)   \n", (int) candidatesNBlocks.size() );
    for( size_t i  = 0; i  < candidatesNBlocks.size(); ++i )
    {
       if( candidatesNBlocks[i].second != INT_MAX )
-         SCIPmessageFPrintInfo(SCIPgetMessagehdlr(givenscip), file, "%d : %d  \n", candidatesNBlocks[i].first, candidatesNBlocks[i].second );
+         SCIPmessageFPrintInfo(SCIPgetMessagehdlr(scip), file, "%d : %d  \n", candidatesNBlocks[i].first, candidatesNBlocks[i].second );
       else
-         SCIPmessageFPrintInfo(SCIPgetMessagehdlr(givenscip), file, "%d : %s  \n", candidatesNBlocks[i].first, "user given" );
+         SCIPmessageFPrintInfo(SCIPgetMessagehdlr(scip), file, "%d : %s  \n", candidatesNBlocks[i].first, "user given" );
    }
 }
 
@@ -1220,7 +1270,7 @@ void DETPROBDATA::printPartitionInformation(
  void DETPROBDATA::sortFinishedForScore()
 {
    /* get scoretype once, no need to call it twice for every comparison */
-   GCG_SCORE* score = GCGgetCurrentScore(scip);
+   GCG_SCORE* score = GCGgetCurrentScore(gcg);
    
    /* sort by score in descending order */
    std::sort(finishedpartialdecs.begin(), finishedpartialdecs.end(), [&](PARTIALDECOMP* a, PARTIALDECOMP* b) {return (a->getScore(score) > b->getScore(score)); });
@@ -1228,8 +1278,9 @@ void DETPROBDATA::printPartitionInformation(
 
 
 std::vector<PARTIALDECOMP*> DETPROBDATA::translatePartialdecs(
-   DETPROBDATA* origdata,
-   std::vector<PARTIALDECOMP*> origpartialdecs
+   DETPROBDATA* otherdata,
+   std::vector<PARTIALDECOMP*> otherpartialdecs,
+   SCIP_Bool translateSymmetry
    )
 {
    std::vector<int> rowothertothis;
@@ -1239,19 +1290,20 @@ std::vector<PARTIALDECOMP*> DETPROBDATA::translatePartialdecs(
    std::vector<int> missingrowinthis;
    std::vector<PARTIALDECOMP*> newpartialdecs;
 
-   calcTranslationMapping( origdata, rowothertothis, rowthistoother, colothertothis, colthistoother, missingrowinthis );
+   calcTranslationMapping(otherdata, rowothertothis, rowthistoother, colothertothis, colthistoother, missingrowinthis);
 
-   SCIPverbMessage( this->scip, SCIP_VERBLEVEL_HIGH, NULL,
+   SCIPverbMessage(this->scip, SCIP_VERBLEVEL_HIGH, NULL,
       " calculated translation; number of missing constraints: %ld; number of other partialdecs: %ld \n", missingrowinthis.size(),
-      origpartialdecs.size() );
+      otherpartialdecs.size());
 
-   getTranslatedPartialdecs( origpartialdecs, rowothertothis, rowthistoother, colothertothis, colthistoother, newpartialdecs );
+   getTranslatedPartialdecs(otherpartialdecs, rowothertothis, rowthistoother, colothertothis, colthistoother, newpartialdecs, translateSymmetry);
    return newpartialdecs;
 }
 
 std::vector<PARTIALDECOMP*> DETPROBDATA::translatePartialdecs(
-   DETPROBDATA* origdata
-)
+   DETPROBDATA* otherdata,
+   SCIP_Bool translateSymmetry
+   )
 {
    std::vector<int> rowothertothis;
    std::vector<int> rowthistoother;
@@ -1260,14 +1312,14 @@ std::vector<PARTIALDECOMP*> DETPROBDATA::translatePartialdecs(
    std::vector<int> missingrowinthis;
    std::vector<PARTIALDECOMP*> newpartialdecs;
 
-   calcTranslationMapping( origdata, rowothertothis, rowthistoother, colothertothis, colthistoother, missingrowinthis );
+   calcTranslationMapping(otherdata, rowothertothis, rowthistoother, colothertothis, colthistoother, missingrowinthis);
 
-   SCIPverbMessage( this->scip, SCIP_VERBLEVEL_HIGH, NULL,
+   SCIPverbMessage(this->scip, SCIP_VERBLEVEL_HIGH, NULL,
       " calculated translation; number of missing constraints: %ld; number of other partialdecs: %ld \n", missingrowinthis.size(),
-      (origdata->getOpenPartialdecs().size() + origdata->getFinishedPartialdecs().size()) );
+      (otherdata->getOpenPartialdecs().size() + otherdata->getFinishedPartialdecs().size()));
 
-   getTranslatedPartialdecs( origdata->getOpenPartialdecs(), rowothertothis, rowthistoother, colothertothis, colthistoother, newpartialdecs );
-   getTranslatedPartialdecs( origdata->getFinishedPartialdecs(), rowothertothis, rowthistoother, colothertothis, colthistoother, newpartialdecs );
+   getTranslatedPartialdecs(otherdata->getOpenPartialdecs(), rowothertothis, rowthistoother, colothertothis, colthistoother, newpartialdecs, translateSymmetry);
+   getTranslatedPartialdecs(otherdata->getFinishedPartialdecs(), rowothertothis, rowthistoother, colothertothis, colthistoother, newpartialdecs, translateSymmetry);
    return newpartialdecs;
 }
 
