@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -32,6 +33,7 @@
  * @author Michael Bastubbe
  * @author Hannah Hechenrieder
  * @author Hanna Franzen
+ * @author Erik Muehmer
  */
 
 /*---+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8----+----9----+----0----+----1----+----2*/
@@ -40,19 +42,21 @@
 #define GCG_CLASS_PARTIALDECOMP_H__
 
 #include "objscip/objscip.h"
-#include "struct_detector.h"
+#include "gcg/struct_detector.h"
 
 #include <vector>
 #include <string>
+#include <functional>
+#include <unordered_map>
 
-#include "def.h"
-#include "class_conspartition.h"
-#include "class_varpartition.h"
+
+#include "gcg/class_conspartition.h"
+#include "gcg/class_varpartition.h"
 #include "graph/graph_gcg.h"
 #include "graph/graph.h"
-#include "type_score.h"
+#include "gcg/type_score.h"
 
-#include "reader_gp.h"
+#include "gcg/reader_gp.h"
 
 namespace gcg {
 
@@ -69,6 +73,39 @@ enum USERGIVEN
 };
 
 class DETPROBDATA;
+class PARTIALDECOMP;
+
+/**
+ * a lightweight struct that stores nested structure information of a block
+ */
+struct BLOCK_STRUCTURE
+{
+   /** destructor */
+   ~BLOCK_STRUCTURE();
+
+   /**
+    * Creates a partialdec using the stored structure information.
+    * The new partialdec is created using the constraint and variable names of a pricing problem created by GCG.
+   */
+   PARTIALDECOMP* createPartialdec(
+      DETPROBDATA* parentdetprobdata,              /**< parent detprobdata (the one that fits to the stored con/var ids) */
+      DETPROBDATA* newdetprobdata,                 /**< detprobdata used to create the new partialdec */
+      int probnr                                   /**< the number of the pricing problem the partialdec is created for */
+      );
+
+   /** translates the structure using provided mappings */
+   BLOCK_STRUCTURE* translateStructure(
+      std::vector<int>& rowmapping,                /**< row mapping used to translate data */
+      std::vector<int>& colmapping,                /**< column mapping used to translate data */
+      SCIP_Bool translatesymmetry                  /**< translate symmetry information? */
+      );
+
+   std::vector<int> masterconss;                   /**< vector containing the master constraints */
+   std::vector<std::vector<int>> blockconss;       /**< vector of vectors (for each block) containing the block constraints */
+   std::vector<BLOCK_STRUCTURE*> blockstructures;  /**< vector containing pointers to block structures for each block (if available) */
+   std::vector<int> symmetricalblocks;             /**< vector containing for each block an id of a block that is symmetrical */
+   std::unordered_map<int, int> symmetryvardata;   /**< symmetry information of variables (mapping) */
+};
 
 
 /*!
@@ -81,6 +118,7 @@ class DETPROBDATA;
 class PARTIALDECOMP
 {
 private:
+   GCG* gcg;                                                   /**< GCG data structure */
    SCIP* scip;                                                 /**< SCIP data structure */
    int id;                                                     /**< unique id of the partialdec, unique */
    int nblocks;                                                /**< number of blocks the partial decomposition currently has */
@@ -93,6 +131,7 @@ private:
    std::vector<std::vector<int>> varsforblocks;                /**< varsforblocks[k] contains a vector of indices of all
                                                                  *< variables assigned to block k */
    std::vector<int> linkingvars;                               /**< vector containing indices of linking variables */
+   std::vector<std::vector<int>> linkingvarsforblocks;         /**< vector of vectors containing indices of linking variables for each block */
    std::vector<std::vector<int>> stairlinkingvars;             /**< vector containing indices of staircase linking variables
                                                                  *< of the blocks (stair-linking variables are registered only
                                                                  *< in their first block) */
@@ -107,7 +146,6 @@ private:
 
    std::vector<int>  ncoeffsforblock;                          /**< number of coeffs per block */
 
-   SCIP_Bool         calculatedncoeffsforblock;                /**< is the number of coeff per block already calculated*/
    int               ncoeffsformaster;                         /**< number of master coefficients */
    std::vector<std::vector<int>> ncoeffsforblockformastercons; /**< number of coeffs a block has in a certain master constraint */
 
@@ -131,10 +169,10 @@ private:
    bool isfinishedbyfinisher;                                   /**< was this partialdec finished by the finishpartialdec() method of a detector */
 
    /* aggregation information */
-   int                  nrepblocks;                                    /**< number of block representatives */
-   std::vector<std::vector<int>> reptoblocks;                          /**< translation of the block representatives to (old) blocks */
-   std::vector<int>     blockstorep;                                   /**< translation of the (old) blocks to the block representatives */
-   std::vector<std::vector<std::vector<int> > > pidtopidvarmaptofirst; /**< [nrepblocks][blockstorep[k].size()][nvarsforprob] collection of varmaps of probindices from k-th subproblem to the zeroth block that is represented */
+   int nequivalenceclasses;                                     /**< number of equivalence classes */
+   std::vector<std::vector<int>> eqclasstoblocks;               /**< translation of the equivalence classes to blocks (first block at index 0 is the representative block of the class) */
+   std::vector<int> blockstoeqclasses;                          /**< translation of the blocks to the equivalence classes */
+   std::vector<std::vector<std::vector<int>>> eqclassesvarmappings; /**< stores for each equivalence class (first index) and for each block of the class (second index) a mapping that maps the indices of the probvars (third index) of the block to the indices of the probvars (actual value) of the representative block */
 
    /* statistic information */
    std::vector<GCG_DETECTOR*> detectorchain;          /**< vector containing detectors that worked on that partialdec */
@@ -179,6 +217,7 @@ private:
    GCG_DETECTOR* finishedorigby;          /**< index of finishing detector of orig ancestor partialdec */
 
    int translatedpartialdecid;
+   std::vector<BLOCK_STRUCTURE*> blockstructures;    /**< structure information for each block (empty iff decomposition is not nested)*/
 
 private:
    /**< id of the translated partialdec */
@@ -198,8 +237,7 @@ private:
       SCIP_Bool*           identical,  /**< pointer to store if the subproblems are identical  */
       unsigned int         searchnodelimit,    /**< bliss search node limit (requires patched bliss version) */
       unsigned int         generatorlimit      /**< bliss generator limit (requires patched bliss version) */
-
-   );
+      );
 
 
    /**
@@ -236,8 +274,8 @@ public:
     */
    GCG_EXPORT
    PARTIALDECOMP(
-      SCIP* scip,                                  /**< scip data structure */
-      bool originalProblem                             /**< true iff partialdec is for presolved problem (else for original problem) */
+      GCG* gcgstruct,                              /**< GCG data structure */
+      bool originalProblem                         /**< true iff partialdec is for presolved problem (else for original problem) */
       );
 
    /**
@@ -382,7 +420,7 @@ public:
    SCIP_RETCODE assignBorderFromConstoblock(
       SCIP_HASHMAP* constoblock, /**< hashmap assigning cons indices (not SCIP_Cons*) to block indices */
       int givenNBlocks           /**< number of blocks the hashmap contains */
-       );
+      );
 
    /**
     * @brief assigns open vars to stairlinking if appropriate
@@ -391,15 +429,13 @@ public:
     * @returns true iff at least one stairlinkingvar was assigned
     */
    GCG_EXPORT
-   bool assignCurrentStairlinking(
-      );
+   bool assignCurrentStairlinking();
 
    /**
     * @brief assigns open conss to master
     */
    GCG_EXPORT
-   void assignOpenConssToMaster(
-      );
+   void assignOpenConssToMaster();
 
    /**
     * @brief assigns conss structure according to given hashmap
@@ -414,7 +450,7 @@ public:
    SCIP_RETCODE assignPartialdecFromConstoblock(
       SCIP_HASHMAP* constoblock, /**< hashmap assigning cons indices (not SCIP_Cons*) to block indices */
       int additionalNBlocks      /**< number of (additional) blocks the hashmap contains */
-       );
+      );
 
    /*!
     * @brief assigns conss structure according to given vector
@@ -443,20 +479,19 @@ public:
     *  hence it cannot be applied in presence of linking variables
     */
    GCG_EXPORT
-   void assignSmallestComponentsButOneConssAdjacency(
-      );
+   void assignSmallestComponentsButOneConssAdjacency();
 
    /**
-    * @brief reassigns linking vars to stairlinkingvars if possible
+    * @brief reorders blocks for making a maximum number of linking vars stairlinking
     *
-    *  potentially reorders blocks for making a maximum number of linking vars stairlinking
+    *  reorders blocks for making a maximum number of linking vars stairlinking
     *  if all vars that connect exactly two blocks have a staircase structure, all of them become stairlinkingvars
     *  otherwise, the stairlinking assignment is done greedily
+    *  call PARTIALDEC::findVarsLinkingToStairlinking afterwards (this is also done by PARTIALDEC::prepare())
     *  @note precondition: partialdec does not have any stairlinking vars
     */
    GCG_EXPORT
-   void calcStairlinkingVars(
-        );
+   void reorderBlocksForStairlinkingVars();
 
    /**
     * @brief checks if all conss are assigned
@@ -484,8 +519,7 @@ public:
     * @returns true iff the partialdec seems to be consistent
     * */
    GCG_EXPORT
-   bool checkConsistency(
-      );
+   bool checkConsistency();
 
    /**
     * @brief assigns all open constraints and open variables trivially
@@ -495,8 +529,7 @@ public:
     *  @note partialdecomps should usually be completed by a detector, only use this function if you know what you are doing.
     */
    GCG_EXPORT
-   void complete(
-      );
+   void complete();
 
    /**
     * @brief assigns all open constraints and open variables
@@ -505,8 +538,7 @@ public:
     *  a cons and a var are adjacent if the var appears in the cons
     */
    GCG_EXPORT
-   void completeByConnected(
-      );
+   void completeByConnected();
 
    /**
     * @brief assigns all open constraints and open variables
@@ -517,8 +549,7 @@ public:
     *  hence it cannot be applied in presence of linking variables
     */
    GCG_EXPORT
-   void completeByConnectedConssAdjacency(
-      );
+   void completeByConnectedConssAdjacency();
 
    /**
     * @brief assigns all open constraints and open variables
@@ -528,8 +559,7 @@ public:
     *  and finally to master, if there does not exist such a block
     */
    GCG_EXPORT
-   void completeGreedily(
-      );
+   void completeGreedily();
 
    /** @brief removes the given cons from master
     */
@@ -549,8 +579,7 @@ public:
     *  - leave the cons/variableopen if nothing from the above holds
     *  */
    GCG_EXPORT
-   void considerImplicits(
-      );
+   void considerImplicits();
 
    /**
     * @brief copies the given partialdec's partition statistics
@@ -573,7 +602,7 @@ public:
    GCG_EXPORT
    void deleteEmptyBlocks(
       bool variables
-   );
+      );
 
    /**
     * @brief deletes a cons from list of open conss
@@ -593,7 +622,7 @@ public:
    GCG_EXPORT
    std::vector<int>::const_iterator deleteOpencons(
       std::vector<int>::const_iterator itr
-   );
+      );
 
    /**
     * @brief deletes a var from the list of open vars
@@ -613,7 +642,7 @@ public:
    GCG_EXPORT
    std::vector<int>::const_iterator deleteOpenvar(
       std::vector<int>::const_iterator itr
-   );
+      );
 
    /**
     * @brief displays the relevant information of the partialdec
@@ -664,8 +693,7 @@ public:
     * Variables are reassigned as master if the variable only hits master conss
     */
    GCG_EXPORT
-   void findVarsLinkingToMaster(
-      );
+   void findVarsLinkingToMaster();
 
    /**
     * @brief reassigns variables classified as linking to stairlinking if appropriate
@@ -674,8 +702,7 @@ public:
     * blocks
     */
    GCG_EXPORT
-   void findVarsLinkingToStairlinking(
-      );
+   void findVarsLinkingToStairlinking();
 
    /**
     * @brief gets partialdec id of given ancestor id
@@ -691,8 +718,7 @@ public:
     * @return vector of ids of all ancestors id
     */
    GCG_EXPORT
-   std::vector<int>& getAncestorList(
-      );
+   std::vector<int>& getAncestorList();
 
    /**
     * set ancestor list directly
@@ -707,7 +733,7 @@ public:
    GCG_EXPORT
    void removeAncestorID(
       int ancestorid    /**< id to remove */
-   );
+      );
 
    /**
     * adds ancestor id to back of list
@@ -719,13 +745,23 @@ public:
       );
 
    /**
-    * @brief get a vector of block ids that are identical to block with id repid
-    * @param repid id of the representative block
-    * @return vector of block ids that are identical to block with id repid
+    * @brief get a vector of block ids that are contained in the equivalence class eqclass
+    * @param eqclass id of the equivalence class
+    * @return vector of block ids
     */
    GCG_EXPORT
-   const std::vector<int> & getBlocksForRep(
-      int  repid
+   const std::vector<int>& getBlocksForEqClass(
+      int  eqclass
+      );
+
+   /**
+    * @brief get the representative block of the given equivalence class
+    * @param eqclass id of the equivalence class
+    * @return id of the representative block
+    */
+   GCG_EXPORT
+   int getReprBlockForEqClass(
+      int  eqclass
       );
 
    /**
@@ -790,6 +826,16 @@ public:
    GCG_EXPORT
    std::vector<int>& getLinkingvars();
 
+   /**
+    * @brief returns vector of all linking var indices contained by the block
+    * @return vector of var indices
+    * @note when accessed the vector is supposed to be sorted
+    */
+   GCG_EXPORT
+   std::vector<int>& getLinkingvarsForBlock(
+      int block
+      );
+
    /**
     * @brief Gets array containing all master conss indices
     * @return array containing all master conss indices
@@ -822,11 +868,10 @@ public:
     * @return the number of nonzero coeffs in master
     */
    GCG_EXPORT
-   int getNCoeffsForMaster(
-      );
+   int getNCoeffsForMaster();
 
    /**
-    * @brief returns the score of the partialdec (depending on enabled score)
+    * @brief returns the score of the partialdec
     * @param score the score
     * @return the score
     */
@@ -835,6 +880,13 @@ public:
       GCG_SCORE* score
       );
 
+   /**
+    * @brief returns the score of the partialdec (depending on enabled score)
+    * @return the score
+    */
+   GCG_EXPORT
+   SCIP_Real getScore();
+
    /**
    * @brief gets an intermediate score value for the blocks of a partialdec
    *
@@ -843,9 +895,7 @@ public:
    * 
    * @returns intermediate score value
    */
-   SCIP_Real calcBlockAreaScore(
-      SCIP* scip                /**< SCIP data structure */
-      );
+   SCIP_Real calcBlockAreaScore();
 
    /**
     * @brief sets the scorevalue of score
@@ -862,8 +912,7 @@ public:
     * @return TRUE iff all master constraints set partitioning, set packing, set cover, or cardinality constraints
     */
    GCG_EXPORT
-   SCIP_Bool hasSetppccardMaster(
-   );
+   SCIP_Bool hasSetppccardMaster();
 
 
    /**
@@ -871,16 +920,14 @@ public:
     * @return TRUE iff all master constraints set partitioning, set packing, or set cover
     */
    GCG_EXPORT
-   SCIP_Bool hasSetppcMaster(
-   );
+   SCIP_Bool hasSetppcMaster();
 
    /**
     * @brief checks iff all master constraints set partitioning, or set packing constraints
     * @return TRUE iff all master constraints set partitioning, or set packing constraints
     */
    GCG_EXPORT
-   SCIP_Bool hasSetppMaster(
-   );
+   SCIP_Bool hasSetppMaster();
 
    /**
     * @brief Gets the USERGIVEN status of this partialdecs
@@ -935,6 +982,18 @@ public:
    GCG_EXPORT
    int getNDetectors();
 
+   /**
+    *  @brief Returns the structure information of a specific block
+    */
+   BLOCK_STRUCTURE* getBlockStructure(
+      int block                          /**< block id */
+      );
+
+   /**
+    *  @brief Returns the structure information of all blocks
+    */
+   std::vector<BLOCK_STRUCTURE*>& getBlockStructures();
+
    /**
     * @brief Gets size of the vector containing linking vars
     * @return size of the vector containing linking vars
@@ -1001,7 +1060,7 @@ public:
     * @return the number of blockrepresentatives
     */
    GCG_EXPORT
-   int getNReps();
+   int getNEquivalenceClasses();
 
    /**
     * @brief Gets size of the vector containing stairlinking vars
@@ -1035,36 +1094,21 @@ public:
     * @return number of vars that are assigned to any block
     */
    GCG_EXPORT
-   int getNVarsForBlocks(
-      );
+   int getNVarsForBlocks();
 
    /**
-    * @brief Gets array containing constraints not assigned yet
-    * @return array containing constraints not assigned yet
+    * @brief Gets vector containing constraints not assigned yet
+    * @return vector containing constraints not assigned yet
     */
    GCG_EXPORT
-   const int* getOpenconss();
+   std::vector<int>& getOpenconss();
 
    /**
-    * @brief Gets a vector containing constraint ids not assigned yet as vector
-    * @return returns a vector containing constraint ids not assigned yet as vector
+    * @brief Gets vector containing variables not assigned yet
+    * @return returns vector containing variables not assigned yet
     */
    GCG_EXPORT
-   std::vector<int>& getOpenconssVec();
-
-   /**
-    * @brief Gets array containing variables not assigned yet
-    * @return returns array containing variables not assigned yet
-    */
-   GCG_EXPORT
-   const int* getOpenvars();
-
-   /**
-    * Gets array containing variables not assigned yet as vector
-    * @return array containing variables not assigned yet as vector
-    */
-   GCG_EXPORT
-   std::vector<int>& getOpenvarsVec();
+   std::vector<int>& getOpenvars();
 
    /**
     * @brief Gets fraction of variables assigned to the border for a detector
@@ -1171,22 +1215,21 @@ public:
     * @return index of the representative block for a block, this might be blockid itself
     */
    GCG_EXPORT
-   int getRepForBlock(
+   int getEqClassForBlock(
       int blockid
       );
 
    /**
-    * @brief Gets the represenation varmap
+    * @brief Returns a vector that maps probvar indices of a block contained in an equivalence class to the probvar indices of the representative block of the class
     *
-    * Var map is vector for represenative repid and the blockrepid-th block that is represented by repid
-    * @param repid id of representative
-    * @param blockrepid id of block
-    * @return the represenation varmap as vector for represenative repid and the blockrepid-th block that is represented by repid
+    * @param eqclass id of the equivalence class
+    * @param eqclassblock index of block (with respect to eqclass)
+    * @return probvar indices mapping
     */
    GCG_EXPORT
    std::vector<int> & getRepVarmap(
-      int repid,
-      int blockrepid
+      int eqclass,
+      int eqclassblock
       );
 
    /**
@@ -1197,13 +1240,13 @@ public:
    DETPROBDATA* getDetprobdata();
 
    /**
-    * @brief Gets array containing stairlinking vars,
+    * @brief Gets vector containing stairlinking vars,
     * @note if a stairlinking variable links block i and i+1 it is only stored in vector of block i
     * @param block id of the block the stairlinking variable varctor is asked for
-    * @return array containing stairlinking vars,
+    * @return vector containing stairlinking vars,
     */
    GCG_EXPORT
-   const int* getStairlinkingvars(
+   std::vector<int>& getStairlinkingvars(
       int block
       );
 
@@ -1218,8 +1261,8 @@ public:
       );
 
    /**
-    * @brief  Gets index in variables array of a block for a variable
-    * @param varid the id of the variable the index
+    * @brief  Gets the index of a variable in the vector returned by PARTIALDECOMP::getVarsForBlock.
+    * @param varid the id of the variable
     * @param block the corresponding block id
     * @return the index of the variable or -1 if var is not in block
     */
@@ -1227,7 +1270,7 @@ public:
    int getVarProbindexForBlock(
       int varid,
       int block
-   );
+      );
 
    /**
     * @brief Gets whether this partialdec is complete,
@@ -1257,12 +1300,19 @@ public:
       int cons
       );
 
+   /**
+    * @brief Gets whether the partialdec is from the original problem
+    * @return true iff the partialdec is from the original problem
+    */
+   GCG_EXPORT
+   bool isAssignedToOrigProb();
+
    /**
     * @brief Gets whether the partialdec is from the presolved problem
     * @return true iff the partialdec is from the presolved problem
     */
    GCG_EXPORT
-   bool isAssignedToOrigProb();
+   bool isAssignedToPresolvedProb();
 
    /**
     * Gets whether the partialdec is currently selected in explore menue
@@ -1293,6 +1343,12 @@ public:
       PARTIALDECOMP* other /**< other partialdec to check equality with */
       );
 
+   /**
+    *  @brief Checks whether this is a nested decomposition
+    *  @returns true iff the decomposition is nested
+    */
+   bool isNested();
+
    /**
     * @brief Gets whether this partialdec was propagated by specified detector
     * @param detector pointer to detector to check for
@@ -1335,6 +1391,18 @@ public:
       int var
       );
 
+   /**
+    * @brief Checks whether the var is a linking var contained by the block
+    * @param var id of var to check
+    * @param block id of block to check
+    * @return true iff the var is a linking var
+    */
+   GCG_EXPORT
+   bool isVarLinkingvarOfBlock(
+      int var,
+      int block
+      );
+
    /**
     * @brief Checks whether the var is a master var
     * @param var id of var to check
@@ -1379,12 +1447,10 @@ public:
 
    /**
     * @brief prints partition information as described in \see cls reader
-    * @param givenscip scip data structure
     * @param file output file
     */
    GCG_EXPORT
    void printPartitionInformation(
-      SCIP*                givenscip,
       FILE*                file
       );
 
@@ -1396,8 +1462,7 @@ public:
     * @note partialdec might be not complete
     */
    GCG_EXPORT
-   void refineToBlocks(
-      );
+   void refineToBlocks();
 
    /**
     * @brief refine partialdec with focus on master
@@ -1406,8 +1471,7 @@ public:
     *  assign other conss and vars to master if possible (@see assignOpenPartialHittingToMaster())
     */
    GCG_EXPORT
-   void refineToMaster(
-      );
+   void refineToMaster();
 
    /**
     * @brief registers statistics for a used conspartition
@@ -1523,12 +1587,10 @@ public:
 
    /**
     * @brief sets detector that finished the partialdec in the original problem
-    * @param detectorID pointer to detector that has finished this partialdecs
     * @note does not add the detector to the detectorchain and does not modify partition statistics
     */
    GCG_EXPORT
    void setDetectorFinishedOrig(
-      GCG_DETECTOR* detectorID
       );
 
    /**
@@ -1560,6 +1622,14 @@ public:
       int nblocks
       );
 
+   /**
+    *  @brief sets structure information of a specific block
+    */
+   void setBlockStructure(
+      int block,                          /**< block id */
+      BLOCK_STRUCTURE* blockstructure     /**< structure information */
+      );
+
    /**
     * @brief set the selection status of this partialdecs
     * @param selected whether the partialdec is selected
@@ -1943,8 +2013,7 @@ public:
     * @returns vector of a vector of indices for each block 
     */
    GCG_EXPORT
-   std::vector<std::vector<int>>& getConssForBlocks(
-   );
+   std::vector<std::vector<int>>& getConssForBlocks();
 
    GCG_EXPORT
    int getTranslatedpartialdecid() const;
@@ -1970,6 +2039,30 @@ public:
       int block /**< block id */
       );
 
+   /**
+   * @brief creates a detector chain short string for this partialdec, is built from detector chain
+   */
+   GCG_EXPORT
+   std::string buildDecChainString();
+
+   /**
+    * @brief sets symmetry information
+    *
+    * Each block b must be mapped to its representative block rb with rb <= b. Furthermore, rb must be mapped to itself.
+    * It is assumed that a variable assigned to a representative block is mapped to itself (varmapping does not have
+    * to cover this case).
+    *
+    * @param blockmapping function that maps block ids to the ids of the representative blocks
+    * @param varmapping function that maps tuples (block, variable index (in the block)) to the indices of the representative variables (in their blocks)
+    *
+    * @returns true iff symmetry information was stored successfully
+    */
+    GCG_EXPORT
+    bool setSymmetryInformation(
+       std::function<int(int)> blockmapping,
+       std::function<int(int, int)> varmapping
+       );
+
 private:
 
    /**
@@ -1988,8 +2081,7 @@ private:
     *  - leave it open if it hits no blocks yet
     *  @return true iff some assignment was made by the method
     */
-   bool assignHittingOpenconss(
-      );
+   bool assignHittingOpenconss();
 
    /** @brief assigns every open var
     *
@@ -1998,8 +2090,7 @@ private:
     *  - to linking if it hits blockconss of more than one different blocks
     *  - leave the var open otherwise
     *  @return true iff there is a var that has been assigned in this call*/
-   bool assignHittingOpenvars(
-      );
+   bool assignHittingOpenvars();
 
    /**
     * @brief assigns every open cons to master that hits
@@ -2009,14 +2100,12 @@ private:
     *  - a master var
     *  - or leave it open elsewise
     */
-   void assignOpenPartialHittingConsToMaster(
-      );
+   void assignOpenPartialHittingConsToMaster();
 
    /**
     * @brief assigns open conss/vars that hit exactly one block and at least one open var/cons to border
     */
-   void assignOpenPartialHittingToMaster(
-      );
+   void assignOpenPartialHittingToMaster();
 
    /**
     * @brief assigns every open var to linking that hits
@@ -2025,14 +2114,12 @@ private:
     *  - exactly one block cons and at least one open cons
     *  - leave it open otherwise
     */
-   void assignOpenPartialHittingVarsToMaster(
-      );
+   void assignOpenPartialHittingVarsToMaster();
 
    /**
     * @brief calculates the number of nonzero coefficients for the blocks
     */
-   void calcNCoeffsForBlocks(
-   );
+   void calcNCoeffsForBlocks();
 
    /**
     * @brief calculates the hash value of the partialdec for comparing
@@ -2045,8 +2132,7 @@ private:
     * counts for each pair of block and master constraint, how many nonzero entries the variables of the blocks
     * have in the master constraint
     */
-   void calcNCoeffsForBlockForMastercons(
-      );
+   void calcNCoeffsForBlockForMastercons();
 
    /**
     *  @brief optimizes block order to max stairlinking vars
@@ -2082,14 +2168,18 @@ private:
     */
    void displayAggregationInformation();
 
+   /**
+    * @brief calculates the linking vars for the blocks
+    */
+   void findLinkingvarBlocks();
+
    /**
     * @brief calculates potential stairlinking variables with their blocks
     *
     * @return a vector of pairs of var index and vector of (two) block indices
     *  the related linking variable hits exactly these two blocks given in the related vector
     */
-   std::vector< std::pair< int, std::vector< int > > > findLinkingVarsPotentiallyStairlinking(
-      );
+   std::vector< std::pair< int, std::vector< int > > > findLinkingVarsPotentiallyStairlinking();
 
    /**
     * @brief returns the data of the conspartition that the given detector made use of
