@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -32,17 +33,17 @@
 
 /*---+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8----+----9----+----0----+----1----+----2*/
 
-#include "clscons_miplibconstypes.h"
-#include "cons_decomp.h"
-#include "cons_decomp.hpp"
+#include "gcg/clscons_miplibconstypes.h"
+#include "gcg/cons_decomp.h"
+#include "gcg/cons_decomp.hpp"
 #include <vector>
 #include <stdio.h>
 #include <sstream>
 
-#include "class_detprobdata.h"
+#include "gcg/class_detprobdata.h"
 
-#include "class_conspartition.h"
-#include "scip_misc.h"
+#include "gcg/class_conspartition.h"
+#include "gcg/scip_misc.h"
 
 /* classifier properties */
 #define CLSCONS_NAME                  "miplibconstype"       /**< name of classifier */
@@ -80,13 +81,14 @@ static
 GCG_DECL_CONSCLASSIFY(classifierClassify)
 {
    gcg::DETPROBDATA* detprobdata;
+   SCIP* origprob = GCGgetOrigprob(gcg);
    if( transformed )
    {
-      detprobdata = GCGconshdlrDecompGetDetprobdataPresolved(scip);
+      detprobdata = GCGconshdlrDecompGetDetprobdataPresolved(gcg);
    }
    else
    {
-      detprobdata = GCGconshdlrDecompGetDetprobdataOrig(scip);
+      detprobdata = GCGconshdlrDecompGetDetprobdataOrig(gcg);
    }
 
    std::vector<int> nfoundconstypesrangedsinglecount( (int) SCIP_CONSTYPE_GENERAL + 1, 0 );
@@ -108,29 +110,29 @@ GCG_DECL_CONSCLASSIFY(classifierClassify)
 
       cons = detprobdata->getCons(c);
 
-      nvars =  GCGconsGetNVars(scip, cons );
+      nvars =  GCGconsGetNVars(origprob, cons );
 
-      lhs = GCGconsGetLhs(scip, cons);
-      rhs = GCGconsGetRhs(scip, cons);
+      lhs = GCGconsGetLhs(origprob, cons);
+      rhs = GCGconsGetRhs(origprob, cons);
       if( nvars != 0 )
       {
-         SCIP_CALL_ABORT( SCIPallocBufferArray(scip, &vals, nvars));
-         SCIP_CALL_ABORT( SCIPallocBufferArray(scip, &vars, nvars));
-         SCIP_CALL_ABORT( GCGconsGetVals(scip, cons, vals, nvars ) );
-         SCIP_CALL_ABORT( GCGconsGetVars(scip, cons, vars, nvars ) );
+         SCIP_CALL_ABORT( SCIPallocBufferArray(origprob, &vals, nvars));
+         SCIP_CALL_ABORT( SCIPallocBufferArray(origprob, &vars, nvars));
+         SCIP_CALL_ABORT( GCGconsGetVals(origprob, cons, vals, nvars ) );
+         SCIP_CALL_ABORT( GCGconsGetVars(origprob, cons, vars, nvars ) );
       }
 
       for( i = 0; i < nvars; i++ )
       {
-         assert(!SCIPisZero(scip, vals[i]) );
+         assert(!SCIPisZero(origprob, vals[i]) );
       }
 
 
       /* is constraint of type SCIP_CONSTYPE_EMPTY? */
       if( nvars == 0 )
       {
-         SCIPdebugMsg(scip, "classified as EMPTY: ");
-         SCIPdebugPrintCons(scip, cons, NULL);
+         SCIPdebugMsg(origprob, "classified as EMPTY: ");
+         SCIPdebugPrintCons(origprob, cons, NULL);
          nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_EMPTY]++;
          nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_EMPTY]++;
          classforcons[c] = SCIP_CONSTYPE_EMPTY;
@@ -138,54 +140,54 @@ GCG_DECL_CONSCLASSIFY(classifierClassify)
       }
 
       /* is constraint of type SCIP_CONSTYPE_FREE? */
-      if( SCIPisInfinity(scip, rhs) && SCIPisInfinity(scip, -lhs) )
+      if( SCIPisInfinity(origprob, rhs) && SCIPisInfinity(origprob, -lhs) )
       {
-         SCIPdebugMsg(scip, "classified as FREE: ");
-         SCIPdebugPrintCons(scip, cons, NULL);
+         SCIPdebugMsg(origprob, "classified as FREE: ");
+         SCIPdebugPrintCons(origprob, cons, NULL);
          nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_FREE]++;
          nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_FREE]++;
          classforcons[c] = SCIP_CONSTYPE_FREE;
-         SCIPfreeBufferArray(scip, &vars);
-         SCIPfreeBufferArray(scip, &vals);
+         SCIPfreeBufferArray(origprob, &vars);
+         SCIPfreeBufferArray(origprob, &vals);
          continue;
       }
 
       /* is constraint of type SCIP_CONSTYPE_SINGLETON? */
       if( nvars == 1 )
       {
-         SCIPdebugMsg(scip, "classified as SINGLETON: ");
-         SCIPdebugPrintCons(scip, cons, NULL);
+         SCIPdebugMsg(origprob, "classified as SINGLETON: ");
+         SCIPdebugPrintCons(origprob, cons, NULL);
          nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_SINGLETON] += 2 ;
          nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_SINGLETON]++;
          classforcons[c] = SCIP_CONSTYPE_SINGLETON;
-         SCIPfreeBufferArray(scip, &vars) ;
-         SCIPfreeBufferArray(scip, &vals) ;
+         SCIPfreeBufferArray(origprob, &vars) ;
+         SCIPfreeBufferArray(origprob, &vals) ;
          continue;
       }
 
       /* is constraint of type SCIP_CONSTYPE_AGGREGATION? */
-      if( nvars == 2 && SCIPisEQ(scip, lhs, rhs) )
+      if( nvars == 2 && SCIPisEQ(origprob, lhs, rhs) )
       {
-         SCIPdebugMsg(scip, "classified as AGGREGATION: ");
-         SCIPdebugPrintCons(scip, cons, NULL);
+         SCIPdebugMsg(origprob, "classified as AGGREGATION: ");
+         SCIPdebugPrintCons(origprob, cons, NULL);
          nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_AGGREGATION]++;
          nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_AGGREGATION]++;
          classforcons[c] = SCIP_CONSTYPE_AGGREGATION;
-         SCIPfreeBufferArray(scip, &vars) ;
-         SCIPfreeBufferArray(scip, &vals) ;
+         SCIPfreeBufferArray(origprob, &vars) ;
+         SCIPfreeBufferArray(origprob, &vals) ;
          continue;
       }
 
       /* is constraint of type SCIP_CONSTYPE_{VARBOUND}? */
       if( nvars == 2 )
       {
-         SCIPdebugMsg(scip, "classified as VARBOUND: ");
-         SCIPdebugPrintCons(scip, cons, NULL);
+         SCIPdebugMsg(origprob, "classified as VARBOUND: ");
+         SCIPdebugPrintCons(origprob, cons, NULL);
          nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_VARBOUND] += 2 ;
          nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_VARBOUND]++;
          classforcons[c] = SCIP_CONSTYPE_VARBOUND;
-         SCIPfreeBufferArray(scip, &vars) ;
-         SCIPfreeBufferArray(scip, &vals) ;
+         SCIPfreeBufferArray(origprob, &vars) ;
+         SCIPfreeBufferArray(origprob, &vals) ;
          continue;
       }
 
@@ -203,9 +205,9 @@ GCG_DECL_CONSCLASSIFY(classifierClassify)
          for( i = 0; i < nvars && !unmatched; i++ )
          {
             unmatched = unmatched || SCIPvarGetType(vars[i]) == SCIP_VARTYPE_CONTINUOUS;
-            unmatched = unmatched || SCIPisLE(scip, SCIPvarGetLbGlobal(vars[i]), -1.0);
-            unmatched = unmatched || SCIPisGE(scip, SCIPvarGetUbGlobal(vars[i]), 2.0);
-            unmatched = unmatched || !SCIPisEQ(scip, REALABS(vals[i]), scale);
+            unmatched = unmatched || SCIPisLE(origprob, SCIPvarGetLbGlobal(vars[i]), -1.0);
+            unmatched = unmatched || SCIPisGE(origprob, SCIPvarGetUbGlobal(vars[i]), 2.0);
+            unmatched = unmatched || !SCIPisEQ(origprob, REALABS(vals[i]), scale);
 
             if( vals[i] < 0.0 )
                nnegbinvars++;
@@ -213,68 +215,68 @@ GCG_DECL_CONSCLASSIFY(classifierClassify)
 
          if( !unmatched )
          {
-            if( SCIPisEQ(scip, lhs, rhs) )
+            if( SCIPisEQ(origprob, lhs, rhs) )
             {
                b = rhs/scale + nnegbinvars;
-               if( SCIPisEQ(scip, 1.0, b) )
+               if( SCIPisEQ(origprob, 1.0, b) )
                {
-                  SCIPdebugMsg(scip, "classified as SETPARTITION: ");
-                  SCIPdebugPrintCons(scip, cons, NULL);
+                  SCIPdebugMsg(origprob, "classified as SETPARTITION: ");
+                  SCIPdebugPrintCons(origprob, cons, NULL);
                   nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_SETPARTITION] += 1 ;
                   nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_SETPARTITION]++;
                   classforcons[c] = SCIP_CONSTYPE_SETPARTITION;
-                  SCIPfreeBufferArray(scip, &vars) ;
-                  SCIPfreeBufferArray(scip, &vals) ;
+                  SCIPfreeBufferArray(origprob, &vars) ;
+                  SCIPfreeBufferArray(origprob, &vals) ;
                   continue;
                }
-               else if( SCIPisIntegral(scip, b) && !SCIPisNegative(scip, b) )
+               else if( SCIPisIntegral(origprob, b) && !SCIPisNegative(origprob, b) )
                {
-                  SCIPdebugMsg(scip, "classified as CARDINALITY: ");
-                  SCIPdebugPrintCons(scip, cons, NULL);
+                  SCIPdebugMsg(origprob, "classified as CARDINALITY: ");
+                  SCIPdebugPrintCons(origprob, cons, NULL);
                   nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_CARDINALITY] += 1 ;
                   nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_CARDINALITY]++;
                   classforcons[c] = SCIP_CONSTYPE_CARDINALITY;
-                  SCIPfreeBufferArray(scip, &vars);
-                  SCIPfreeBufferArray(scip, &vals);
+                  SCIPfreeBufferArray(origprob, &vars);
+                  SCIPfreeBufferArray(origprob, &vals);
                   continue;
                }
             }
 
             b = rhs/scale + nnegbinvars;
-            if( SCIPisEQ(scip, 1.0, b) )
+            if( SCIPisEQ(origprob, 1.0, b) )
             {
-               SCIPdebugMsg(scip, "classified as SETPACKING: ");
-               SCIPdebugPrintCons(scip, cons, NULL);
+               SCIPdebugMsg(origprob, "classified as SETPACKING: ");
+               SCIPdebugPrintCons(origprob, cons, NULL);
                nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_SETPACKING] += 1 ;
                nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_SETPACKING]++;
                classforcons[c] = SCIP_CONSTYPE_SETPACKING;
-               rhs = SCIPinfinity(scip);
+               rhs = SCIPinfinity(origprob);
             }
-            else if( SCIPisIntegral(scip, b) && !SCIPisNegative(scip, b) )
+            else if( SCIPisIntegral(origprob, b) && !SCIPisNegative(origprob, b) )
             {
-               SCIPdebugMsg(scip, "classified as INVKNAPSACK: ");
-               SCIPdebugPrintCons(scip, cons, NULL);
+               SCIPdebugMsg(origprob, "classified as INVKNAPSACK: ");
+               SCIPdebugPrintCons(origprob, cons, NULL);
                nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_INVKNAPSACK] += 1 ;
                 nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_INVKNAPSACK]++;
                 classforcons[c] = SCIP_CONSTYPE_INVKNAPSACK;
-               rhs = SCIPinfinity(scip);
+               rhs = SCIPinfinity(origprob);
             }
 
             b = lhs/scale + nnegbinvars;
-            if( SCIPisEQ(scip, 1.0, b) )
+            if( SCIPisEQ(origprob, 1.0, b) )
             {
-               SCIPdebugMsg(scip, "classified as SETCOVERING: ");
-               SCIPdebugPrintCons(scip, cons, NULL);
+               SCIPdebugMsg(origprob, "classified as SETCOVERING: ");
+               SCIPdebugPrintCons(origprob, cons, NULL);
                nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_SETCOVERING] += 1 ;
                nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_SETCOVERING]++;
                classforcons[c] = SCIP_CONSTYPE_SETCOVERING;
-               lhs = -SCIPinfinity(scip);
+               lhs = -SCIPinfinity(origprob);
             }
 
-            if( SCIPisInfinity(scip, -lhs) && SCIPisInfinity(scip, rhs) )
+            if( SCIPisInfinity(origprob, -lhs) && SCIPisInfinity(origprob, rhs) )
             {
-               SCIPfreeBufferArray(scip, &vars);
-               SCIPfreeBufferArray(scip, &vals);
+               SCIPfreeBufferArray(origprob, &vars);
+               SCIPfreeBufferArray(origprob, &vals);
                continue;
             }
          }
@@ -293,26 +295,26 @@ GCG_DECL_CONSCLASSIFY(classifierClassify)
          for( i = 0; i < nvars && !unmatched; i++ )
          {
             unmatched = unmatched || SCIPvarGetType(vars[i]) == SCIP_VARTYPE_CONTINUOUS;
-            unmatched = unmatched || SCIPisLE(scip, SCIPvarGetLbGlobal(vars[i]), -1.0);
-            unmatched = unmatched || SCIPisGE(scip, SCIPvarGetUbGlobal(vars[i]), 2.0);
-            unmatched = unmatched || !SCIPisIntegral(scip, vals[i]);
+            unmatched = unmatched || SCIPisLE(origprob, SCIPvarGetLbGlobal(vars[i]), -1.0);
+            unmatched = unmatched || SCIPisGE(origprob, SCIPvarGetUbGlobal(vars[i]), 2.0);
+            unmatched = unmatched || !SCIPisIntegral(origprob, vals[i]);
 
-            if( SCIPisNegative(scip, vals[i]) )
+            if( SCIPisNegative(origprob, vals[i]) )
                b -= vals[i];
          }
-         unmatched = unmatched || !detprobdata->isFiniteNonnegativeIntegral(scip, b);
+         unmatched = unmatched || !detprobdata->isFiniteNonnegativeIntegral(b);
 
          if( !unmatched )
          {
-            if( SCIPisEQ(scip, lhs, rhs) )
+            if( SCIPisEQ(origprob, lhs, rhs) )
             {
-               SCIPdebugMsg(scip, "classified as EQKNAPSACK: ");
-               SCIPdebugPrintCons(scip, cons, NULL);
+               SCIPdebugMsg(origprob, "classified as EQKNAPSACK: ");
+               SCIPdebugPrintCons(origprob, cons, NULL);
                nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_EQKNAPSACK] += 1 ;
                nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_EQKNAPSACK]++;
                classforcons[c] = SCIP_CONSTYPE_EQKNAPSACK;
-               SCIPfreeBufferArray(scip, &vars);
-               SCIPfreeBufferArray(scip, &vals);
+               SCIPfreeBufferArray(origprob, &vars);
+               SCIPfreeBufferArray(origprob, &vals);
                continue;
             }
             else
@@ -322,25 +324,25 @@ GCG_DECL_CONSCLASSIFY(classifierClassify)
                matched = FALSE;
                for( i = 0; i < nvars && !matched; i++ )
                {
-                  matched = matched || SCIPisEQ(scip, b, REALABS(vals[i]));
+                  matched = matched || SCIPisEQ(origprob, b, REALABS(vals[i]));
                }
 
-               SCIPdebugMsg(scip, "classified as %s: ", matched ? "BINPACKING" : "KNAPSACK");
-               SCIPdebugPrintCons(scip, cons, NULL);
+               SCIPdebugMsg(origprob, "classified as %s: ", matched ? "BINPACKING" : "KNAPSACK");
+               SCIPdebugPrintCons(origprob, cons, NULL);
                nfoundconstypesrangeddoublecount[matched ? SCIP_CONSTYPE_BINPACKING : SCIP_CONSTYPE_KNAPSACK] += 1 ;
                nfoundconstypesrangedsinglecount[matched ? SCIP_CONSTYPE_BINPACKING : SCIP_CONSTYPE_KNAPSACK]++;
                classforcons[c] = matched ? SCIP_CONSTYPE_BINPACKING : SCIP_CONSTYPE_KNAPSACK;
 
             }
 
-            if( SCIPisInfinity(scip, -lhs) )
+            if( SCIPisInfinity(origprob, -lhs) )
             {
-               SCIPfreeBufferArray(scip, &vars);
-               SCIPfreeBufferArray(scip, &vals);
+               SCIPfreeBufferArray(origprob, &vars);
+               SCIPfreeBufferArray(origprob, &vals);
                continue;
             }
             else
-               rhs = SCIPinfinity(scip);
+               rhs = SCIPinfinity(origprob);
          }
       }
 
@@ -352,32 +354,32 @@ GCG_DECL_CONSCLASSIFY(classifierClassify)
          unmatched = FALSE;
 
          b = rhs;
-         unmatched = unmatched || !detprobdata->isFiniteNonnegativeIntegral(scip, b);
+         unmatched = unmatched || !detprobdata->isFiniteNonnegativeIntegral(b);
 
          for( i = 0; i < nvars && !unmatched; i++ )
          {
             unmatched = unmatched || SCIPvarGetType(vars[i]) == SCIP_VARTYPE_CONTINUOUS;
-            unmatched = unmatched || SCIPisNegative(scip, SCIPvarGetLbGlobal(vars[i]));
-            unmatched = unmatched || !SCIPisIntegral(scip, vals[i]);
-            unmatched = unmatched || SCIPisNegative(scip, vals[i]);
+            unmatched = unmatched || SCIPisNegative(origprob, SCIPvarGetLbGlobal(vars[i]));
+            unmatched = unmatched || !SCIPisIntegral(origprob, vals[i]);
+            unmatched = unmatched || SCIPisNegative(origprob, vals[i]);
          }
 
          if( !unmatched )
          {
-            SCIPdebugMsg(scip, "classified as INTKNAPSACK: ");
-            SCIPdebugPrintCons(scip, cons, NULL);
+            SCIPdebugMsg(origprob, "classified as INTKNAPSACK: ");
+            SCIPdebugPrintCons(origprob, cons, NULL);
             nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_INTKNAPSACK] += 1 ;
             nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_INTKNAPSACK]++;
             classforcons[c] = SCIP_CONSTYPE_INTKNAPSACK;
 
-            if( SCIPisInfinity(scip, -lhs) )
+            if( SCIPisInfinity(origprob, -lhs) )
             {
-               SCIPfreeBufferArray(scip, &vars);
-               SCIPfreeBufferArray(scip, &vals);
+               SCIPfreeBufferArray(origprob, &vars);
+               SCIPfreeBufferArray(origprob, &vals);
                continue;
             }
             else
-               rhs = SCIPinfinity(scip);
+               rhs = SCIPinfinity(origprob);
          }
       }
 
@@ -389,36 +391,36 @@ GCG_DECL_CONSCLASSIFY(classifierClassify)
          for( i = 0; i < nvars && !unmatched; i++ )
          {
             if( SCIPvarGetType(vars[i]) != SCIP_VARTYPE_CONTINUOUS
-               && (SCIPisLE(scip, SCIPvarGetLbGlobal(vars[i]), -1.0)
-                  || SCIPisGE(scip, SCIPvarGetUbGlobal(vars[i]), 2.0)) )
+               && (SCIPisLE(origprob, SCIPvarGetLbGlobal(vars[i]), -1.0)
+                  || SCIPisGE(origprob, SCIPvarGetUbGlobal(vars[i]), 2.0)) )
                unmatched = TRUE;
          }
 
          if( !unmatched )
          {
-            SCIPdebugMsg(scip, "classified as MIXEDBINARY (%d): ", detprobdata->isRangedRow(scip, lhs, rhs) ? 2 : 1);
-            SCIPdebugPrintCons(scip, cons, NULL);
+            SCIPdebugMsg(origprob, "classified as MIXEDBINARY (%d): ", detprobdata->isRangedRow(lhs, rhs) ? 2 : 1);
+            SCIPdebugPrintCons(origprob, cons, NULL);
             nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_MIXEDBINARY] += 1 ;
             nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_MIXEDBINARY]++;
             classforcons[c] = SCIP_CONSTYPE_MIXEDBINARY;
-            SCIPfreeBufferArray(scip, &vars) ;
-            SCIPfreeBufferArray(scip, &vals) ;
+            SCIPfreeBufferArray(origprob, &vars) ;
+            SCIPfreeBufferArray(origprob, &vals) ;
             continue;
 
          }
       }
 
       /* no special structure detected */
-      SCIPdebugMsg(scip, "classified as GENERAL: ");
-      SCIPdebugPrintCons(scip, cons, NULL);
+      SCIPdebugMsg(origprob, "classified as GENERAL: ");
+      SCIPdebugPrintCons(origprob, cons, NULL);
       nfoundconstypesrangeddoublecount[SCIP_CONSTYPE_GENERAL] += 1 ;
       nfoundconstypesrangedsinglecount[SCIP_CONSTYPE_GENERAL]++;
       classforcons[c] = SCIP_CONSTYPE_GENERAL;
-      SCIPfreeBufferArray(scip, &vars);
-      SCIPfreeBufferArray(scip, &vals);
+      SCIPfreeBufferArray(origprob, &vars);
+      SCIPfreeBufferArray(origprob, &vals);
    }
 
-   classifier = new gcg::ConsPartition(scip, "constypes according to miplib", (int) SCIP_CONSTYPE_GENERAL + 1, detprobdata->getNConss() );
+   classifier = new gcg::ConsPartition(gcg, "constypes according to miplib", (int) SCIP_CONSTYPE_GENERAL + 1, detprobdata->getNConss() );
 
    /* set class names and descriptions of every class */
    for( int c = 0; c < classifier->getNClasses(); ++ c )
@@ -505,7 +507,7 @@ GCG_DECL_CONSCLASSIFY(classifierClassify)
 
 
    classifier->removeEmptyClasses();
-   SCIPverbMessage(scip, SCIP_VERBLEVEL_HIGH, NULL, " Consclassifier \"%s\" yields a classification with %d different constraint classes \n", classifier->getName(), classifier->getNClasses() );
+   SCIPverbMessage(origprob, SCIP_VERBLEVEL_HIGH, NULL, " Consclassifier \"%s\" yields a classification with %d different constraint classes \n", classifier->getName(), classifier->getNClasses() );
 
    detprobdata->addConsPartition(classifier);
    return SCIP_OKAY;
@@ -515,14 +517,14 @@ GCG_DECL_CONSCLASSIFY(classifierClassify)
  * classifier specific interface methods
  */
 
-SCIP_RETCODE SCIPincludeConsClassifierMiplibConstypes(
-   SCIP*                 scip                /**< SCIP data structure */
+SCIP_RETCODE GCGincludeConsClassifierMiplibConstypes(
+   GCG*                  gcg                 /**< GCG data structure */
    )
 {
    GCG_CLASSIFIERDATA* classifierdata = NULL;
 
    SCIP_CALL(
-      GCGincludeConsClassifier(scip, CLSCONS_NAME, CLSCONS_DESC, CLSCONS_PRIORITY, CLSCONS_ENABLED, classifierdata,
+      GCGincludeConsClassifier(gcg, CLSCONS_NAME, CLSCONS_DESC, CLSCONS_PRIORITY, CLSCONS_ENABLED, classifierdata,
                                classifierFree, classifierClassify));
 
    return SCIP_OKAY;
