@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -38,10 +39,10 @@
 #include <assert.h>
 #include <string.h>
 
-#include "gcg.h"
-#include "branch_generic.h"
-#include "cons_masterbranch.h"
-#include "cons_origbranch.h"
+#include "gcg/gcg.h"
+#include "gcg/branch_generic.h"
+#include "gcg/cons_masterbranch.h"
+#include "gcg/cons_origbranch.h"
 
 #include "scip/nodesel_bfs.h"
 #include "scip/nodesel_dfs.h"
@@ -81,13 +82,14 @@ struct SCIP_ConsData
                                               *   in the master program */
    GCG_BRANCHDATA*       branchdata;         /**< branching data stored by the branching rule containing information
                                               *   about the branching restrictions */
-   SCIP_BRANCHRULE*      branchrule;         /**< branching rule that created the corresponding node and imposed
+   GCG_BRANCHRULE*       branchrule;         /**< branching rule that created the corresponding node and imposed
                                               *   branching restrictions */
 };
 
 /** constraint handler data */
 struct SCIP_ConshdlrData
 {
+   GCG*                  gcg;                /**< GCG data structure */
    SCIP_CONS**           stack;              /**< stack for storing active constraints */
    int                   nstack;             /**< number of elements on the stack */
    int                   maxstacksize;       /**< maximum size of the stack */
@@ -152,7 +154,7 @@ SCIP_DECL_CONSINITSOL(consInitsolOrigbranch)
       --(conshdlrData->nstack);
    }
 
-   GCGconsOrigbranchCheckConsistency(scip);
+   GCGconsOrigbranchCheckConsistency(conshdlrData->gcg);
 
    return SCIP_OKAY;
 }
@@ -268,15 +270,17 @@ SCIP_DECL_CONSDELETE(consDeleteOrigbranch)
    for( i = 0; i < (*consdata)->nchildconss; ++i )
       assert((*consdata)->childconss[i] == NULL);
 
-   /* delete branchdata if no mastercons is linked which would still need the branchdata;
-    * otherwise, the mastercons deletes the branchdata when it is deleted itself
-    */
-   if( (*consdata)->mastercons == NULL && (*consdata)->branchdata != NULL && (*consdata)->branchrule != NULL )
+   /* allow the correspondig branchrule to delete the branch data */
+   if( (*consdata)->branchdata != NULL && (*consdata)->branchrule != NULL )
    {
-      SCIP_CALL( GCGrelaxBranchDataDelete(scip, (*consdata)->branchrule, &(*consdata)->branchdata) );
-      (*consdata)->branchdata = NULL;
+      SCIP_Bool force = ((*consdata)->mastercons == NULL);
+      SCIP_CALL( GCGrelaxBranchDataDelete(GCGorigGetGcg(scip), (*consdata)->branchrule, &(*consdata)->branchdata, TRUE, force) );
+      if( (*consdata)->mastercons != NULL && (*consdata)->branchdata == NULL )
+         GCGconsMasterbranchSetBranchdata((*consdata)->mastercons, NULL);
    }
 
+   (*consdata)->branchdata = NULL;
+
    SCIPfreeBlockMemoryArrayNull(scip, &(*consdata)->childconss, (*consdata)->maxchildconss);
    (*consdata)->childconss = NULL;
    (*consdata)->nchildconss = 0;
@@ -409,34 +413,37 @@ SCIP_DECL_CONSLOCK(consLockOrigbranch)
 
 
 /** creates the handler for origbranch constraints and includes it in SCIP */
-SCIP_RETCODE SCIPincludeConshdlrOrigbranch(
-   SCIP*                 scip                /**< SCIP data structure */
+SCIP_RETCODE GCGincludeConshdlrOrigbranch(
+   GCG*                  gcg                 /**< GCG data structure */
    )
 {
    SCIP_CONSHDLRDATA* conshdlrData;
    SCIP_CONSHDLR* conshdlr;
+   SCIP* origprob = GCGgetOrigprob(gcg);
+   assert(origprob != NULL);
 
-   SCIP_CALL( SCIPallocMemory(scip, &conshdlrData) );
+   SCIP_CALL( SCIPallocMemory(origprob, &conshdlrData) );
+   conshdlrData->gcg = gcg;
    conshdlrData->stack = NULL;
    conshdlrData->nstack = 0;
    conshdlrData->maxstacksize = 25;
    conshdlrData->rootcons = NULL;
 
    /* include constraint handler */
-   SCIP_CALL( SCIPincludeConshdlrBasic(scip, &conshdlr, CONSHDLR_NAME, CONSHDLR_DESC,
+   SCIP_CALL( SCIPincludeConshdlrBasic(origprob, &conshdlr, CONSHDLR_NAME, CONSHDLR_DESC,
          CONSHDLR_ENFOPRIORITY, CONSHDLR_CHECKPRIORITY, CONSHDLR_EAGERFREQ, CONSHDLR_NEEDSCONS,
          consEnfolpOrigbranch, consEnfopsOrigbranch, consCheckOrigbranch,
          consLockOrigbranch, conshdlrData) );
    assert(conshdlr != NULL);
 
-   SCIP_CALL( SCIPsetConshdlrEnforelax(scip, conshdlr, consEnforeOrigbranch) );
-   SCIP_CALL( SCIPsetConshdlrFree(scip, conshdlr, consFreeOrigbranch) );
-   SCIP_CALL( SCIPsetConshdlrExit(scip, conshdlr, consExitOrigbranch) );
-   SCIP_CALL( SCIPsetConshdlrInitsol(scip, conshdlr, consInitsolOrigbranch) );
-   SCIP_CALL( SCIPsetConshdlrExitsol(scip, conshdlr, consExitsolOrigbranch) );
-   SCIP_CALL( SCIPsetConshdlrDelete(scip, conshdlr, consDeleteOrigbranch) );
-   SCIP_CALL( SCIPsetConshdlrActive(scip, conshdlr, consActiveOrigbranch) );
-   SCIP_CALL( SCIPsetConshdlrDeactive(scip, conshdlr, consDeactiveOrigbranch) );
+   SCIP_CALL( SCIPsetConshdlrEnforelax(origprob, conshdlr, consEnforeOrigbranch) );
+   SCIP_CALL( SCIPsetConshdlrFree(origprob, conshdlr, consFreeOrigbranch) );
+   SCIP_CALL( SCIPsetConshdlrExit(origprob, conshdlr, consExitOrigbranch) );
+   SCIP_CALL( SCIPsetConshdlrInitsol(origprob, conshdlr, consInitsolOrigbranch) );
+   SCIP_CALL( SCIPsetConshdlrExitsol(origprob, conshdlr, consExitsolOrigbranch) );
+   SCIP_CALL( SCIPsetConshdlrDelete(origprob, conshdlr, consDeleteOrigbranch) );
+   SCIP_CALL( SCIPsetConshdlrActive(origprob, conshdlr, consActiveOrigbranch) );
+   SCIP_CALL( SCIPsetConshdlrDeactive(origprob, conshdlr, consDeactiveOrigbranch) );
 
    return SCIP_OKAY;
 }
@@ -444,22 +451,24 @@ SCIP_RETCODE SCIPincludeConshdlrOrigbranch(
 
 /** creates and captures a origbranch constraint */
 SCIP_RETCODE GCGcreateConsOrigbranch(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP_CONS**           cons,               /**< pointer to hold the created constraint */
    const char*           name,               /**< name of constraint */
    SCIP_NODE*            node,               /**< the node to which this origbranch constraint belongs */
    SCIP_CONS*            parentcons,         /**< origbranch constraint associated with the father node */
-   SCIP_BRANCHRULE*      branchrule,         /**< the branching rule that created the b&b node the constraint belongs to */
+   GCG_BRANCHRULE*       branchrule,         /**< the branching rule that created the b&b node the constraint belongs to */
    GCG_BRANCHDATA*       branchdata          /**< branching data storing information about the branching restrictions at the
                                               *   corresponding node */
    )
 {
+   SCIP* scip;
    SCIP_CONSHDLR* conshdlr;
    SCIP_CONSDATA* consdata;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert((parentcons == NULL) == (node == NULL));
 
+   scip = GCGgetOrigprob(gcg);
    /* find the origbranch constraint handler */
    conshdlr = SCIPfindConshdlr(scip, CONSHDLR_NAME);
    assert(conshdlr != NULL);
@@ -467,14 +476,6 @@ SCIP_RETCODE GCGcreateConsOrigbranch(
    /* create constraint data */
    SCIP_CALL( SCIPallocBlockMemory(scip, &consdata) );
 
-   if( branchdata == NULL && branchrule != NULL )
-   {
-      if( strcmp(SCIPbranchruleGetName(branchrule), "generic") == 0 )
-      {
-         SCIP_CALL( GCGbranchGenericCreateBranchdata(scip, &branchdata) );
-      }
-   }
-
    /* initialize the fields in the constraint data */
    consdata->node = node;
    consdata->parentcons = parentcons;
@@ -529,13 +530,15 @@ SCIP_RETCODE GCGcreateConsOrigbranch(
 
 /** returns the branch orig constraint of the current node, only needs the pointer to scip */
 SCIP_CONS* GCGconsOrigbranchGetActiveCons(
-   SCIP*                 scip                /**< SCIP data structure */
+   GCG*                  gcg                 /**< GCG data structure */
    )
 {
+   SCIP* scip;
    SCIP_CONSHDLR*     conshdlr;
    SCIP_CONSHDLRDATA* conshdlrData;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
+   scip = GCGgetOrigprob(gcg);
    conshdlr = SCIPfindConshdlr(scip, CONSHDLR_NAME);
    assert(conshdlr != NULL);
 
@@ -549,15 +552,17 @@ SCIP_CONS* GCGconsOrigbranchGetActiveCons(
 
 /** returns the stack and the number of elements on it */
 void GCGconsOrigbranchGetStack(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP_CONS***          stack,              /**< return value: pointer to the stack */
    int*                  nstackelements      /**< return value: pointer to int, for number of elements on the stack */
    )
 {
+   SCIP* scip;
    SCIP_CONSHDLR*     conshdlr;
    SCIP_CONSHDLRDATA* conshdlrData;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
+   scip = GCGgetOrigprob(gcg);
    conshdlr = SCIPfindConshdlr(scip, CONSHDLR_NAME);
    assert(conshdlr != NULL);
 
@@ -569,7 +574,7 @@ void GCGconsOrigbranchGetStack(
    *nstackelements = conshdlrData->nstack;
 }
 
-/** returns the branching data for a given origbranch constraint */
+/** sets the branching data for a given origbranch constraint */
 void GCGconsOrigbranchSetBranchdata(
    SCIP_CONS*            cons,               /**< origbranch constraint for which the branching data is requested */
    GCG_BRANCHDATA*       branchdata          /**< branching data */
@@ -597,7 +602,7 @@ GCG_BRANCHDATA* GCGconsOrigbranchGetBranchdata(
 }
 
 /** returns the branching rule for a given origbranch constraint */
-SCIP_BRANCHRULE* GCGconsOrigbranchGetBranchrule(
+GCG_BRANCHRULE* GCGconsOrigbranchGetBranchrule(
    SCIP_CONS*            cons                /**< origbranch constraint for which the branchrule is requested */
    )
 {
@@ -710,9 +715,10 @@ SCIP_CONS* GCGconsOrigbranchGetMastercons(
 
 /** adds initial constraint to root node */
 SCIP_RETCODE GCGconsOrigbranchAddRootCons(
-   SCIP*                 scip                /**< SCIP data structure */
+   GCG*                  gcg                 /**< GCG data structure */
    )
 {
+   SCIP* scip;
    SCIP_CONSHDLR* conshdlr;
    SCIP_CONSHDLRDATA* conshdlrdata;
    SCIP_CONS* cons;
@@ -720,8 +726,9 @@ SCIP_RETCODE GCGconsOrigbranchAddRootCons(
    int nconss;
    int i;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
 
+   scip = GCGgetOrigprob(gcg);
    conshdlr = SCIPfindConshdlr(scip, CONSHDLR_NAME);
    assert(conshdlr != NULL);
 
@@ -738,24 +745,24 @@ SCIP_RETCODE GCGconsOrigbranchAddRootCons(
    assert(SCIPconshdlrGetNConss(conshdlr) == 0);
    if( conshdlrdata->rootcons == NULL )
    {
-      SCIP_CALL( GCGcreateConsOrigbranch(scip, &cons, "root-origbranch", NULL, NULL, NULL, NULL) );
+      SCIP_CALL( GCGcreateConsOrigbranch(gcg, &cons, "root-origbranch", NULL, NULL, NULL, NULL) );
       SCIP_CALL( SCIPaddConsNode(scip, SCIPgetRootNode(scip), cons, SCIPgetRootNode(scip)) );
       conshdlrdata->rootcons = cons;
    }
 
    /* check consistency */
-   GCGconsOrigbranchCheckConsistency(scip);
+   GCGconsOrigbranchCheckConsistency(gcg);
 
    return SCIP_OKAY;
 }
 
 /** checks the consistency of the origbranch constraints in the problem */
 void GCGconsOrigbranchCheckConsistency(
-   SCIP*                 scip                /**< SCIP data structure */
+   GCG*                  gcg                 /**< GCG data structure */
    )
 {
 #ifdef CHECKCONSISTENCY
-
+   SCIP* scip;
    SCIP_CONSHDLR*     conshdlr;
 
 #ifndef NDEBUG
@@ -764,7 +771,8 @@ void GCGconsOrigbranchCheckConsistency(
    int i;
 #endif
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
+   scip = GCGgetOrigprob(gcg);
    conshdlr = SCIPfindConshdlr(scip, CONSHDLR_NAME);
    if( conshdlr == NULL )
    {
