@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -35,11 +36,10 @@
 
 /*---+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8----+----9----+----0----+----1----+----2*/
 
-#include "gcg.h"
-#include "pub_gcgvar.h"
-#include "struct_vardata.h"
-#include "relax_gcg.h"
-#include "scip_misc.h"
+#include "gcg/gcg.h"
+#include "gcg/pub_gcgvar.h"
+#include "gcg/struct_vardata.h"
+#include "gcg/relax_gcg.h"
 #include "scip/cons_linear.h"
 
 #define STARTMAXMASTERVARS 8
@@ -49,6 +49,15 @@
  * Vardata methods
  */
 
+/** callback method called when a GCG variable is copied */
+static
+SCIP_DECL_VARCOPY(gcgvarcopy)
+{
+   /* do not copy the vardata (since SCIP 10 the vardata pointer will be copied if no callback is registered) */
+   *targetdata = NULL;
+   return SCIP_OKAY;
+}
+
 /** callback method called when an original GCG variable is deleted */
 static
 SCIP_DECL_VARDELORIG(GCGvarDelOrig)
@@ -63,7 +72,7 @@ SCIP_DECL_VARDELORIG(GCGvarDelOrig)
       {
          int nblocks;
 
-         nblocks = GCGgetNPricingprobs(scip);
+         nblocks = GCGgetNPricingprobs(GCGorigGetGcg(scip));
          assert(nblocks > 0);
 
          assert((*vardata)->data.origvardata.linkingvardata != NULL);
@@ -129,7 +138,21 @@ SCIP_DECL_VARDELTRANS(gcgvardeltrans)
 }
 
 
+/** callback method called when an original GCG variable is deleted */
+static
+SCIP_DECL_VARDELORIG(GCGvarDelInferredPricing)
+{
+   if( *vardata == NULL )
+      return SCIP_OKAY;
 
+   assert((*vardata)->vartype == GCG_VARTYPE_INFERREDPRICING);
+
+   SCIPfreeBlockMemory(scip, vardata);
+
+   return SCIP_OKAY;
+}
+
+#ifndef NDEBUG
 /** returns TRUE or FALSE whether variable is a pricing variable or not */
 SCIP_Bool GCGvarIsPricing(
    SCIP_VAR*             var                 /**< SCIP variable structure */
@@ -144,6 +167,7 @@ SCIP_Bool GCGvarIsPricing(
 
    return vardata->vartype == GCG_VARTYPE_PRICING;
 }
+#endif
 
 #ifndef NDEBUG
 /** returns TRUE or FALSE whether variable is a master variable or not */
@@ -161,6 +185,45 @@ SCIP_Bool GCGvarIsMaster(
 }
 #endif
 
+#ifndef NDEBUG
+/** returns TRUE or FALSE whether variable is a inferred pricing variable or not
+  *
+  * inferred pricing variables are auxilary variables that are required by specific extended master cons */
+GCG_EXPORT
+SCIP_Bool GCGvarIsInferredPricing(
+   SCIP_VAR*             var                 /**< SCIP variable structure */
+   )
+{
+   SCIP_VARDATA* vardata;
+   assert(var != NULL);
+
+   vardata = SCIPvarGetData(var);
+   assert(vardata != NULL);
+
+   return vardata->vartype == GCG_VARTYPE_INFERREDPRICING;
+}
+#endif
+
+/** count the number of inferred (coefficient) pricing variables in a array of variables */
+int GCGcountInferredCoefPricingVars(
+   SCIP_VAR**             vars,               /**< array of variables */
+   int                    nvars               /**< number of variables */
+   )
+{
+   int i;
+   int count = 0;
+
+   assert(vars != NULL);
+
+   for( i = 0; i < nvars; i++ )
+   {
+      if( GCGvarIsInferredPricing(vars[i]) && GCGinferredPricingVarIsCoefVar(vars[i]) )
+         count++;
+   }
+
+   return count;
+}
+
 #ifndef NDEBUG
 /** returns TRUE or FALSE whether variable is a original variable or not */
 SCIP_Bool GCGvarIsOriginal(
@@ -231,7 +294,7 @@ SCIP_VAR* GCGoriginalVarGetPricingVar(
 }
 #endif
 
-/** returns the pricing var of an original variable */
+/** sets the pricing var of an original variable */
 void GCGoriginalVarSetPricingVar(
    SCIP_VAR*             var,                /**< SCIP variable structure */
    SCIP_VAR*             pricingvar          /**< SCIP variable structure */
@@ -253,15 +316,17 @@ void GCGoriginalVarSetPricingVar(
 
 /** creates the data for all variables of the original program */
 SCIP_RETCODE GCGcreateOrigVarsData(
-   SCIP*                 scip                /**< SCIP data structure */
+   GCG*                  gcg                 /**< GCG data structure */
    )
 {
+   SCIP* scip;
    SCIP_VAR** vars;
    int nvars;
    int i;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
 
+   scip = GCGgetOrigprob(gcg);
    vars = SCIPgetVars(scip);
    nvars = SCIPgetNVars(scip);
 
@@ -269,7 +334,7 @@ SCIP_RETCODE GCGcreateOrigVarsData(
    for( i = 0; i < nvars; i++ )
    {
       assert(vars[i] != NULL);
-      SCIP_CALL( GCGorigVarCreateData(scip, vars[i]) );
+      SCIP_CALL( GCGorigVarCreateData(gcg, vars[i]) );
    }
 
    return SCIP_OKAY;
@@ -277,15 +342,17 @@ SCIP_RETCODE GCGcreateOrigVarsData(
 
 /** frees the data for all variables of the original program */
 SCIP_RETCODE GCGfreeOrigVarsData(
-   SCIP*                 scip                /**< SCIP data structure */
+   GCG*                  gcg                 /**< GCG data structure */
    )
 {
+   SCIP* scip;
    SCIP_VAR** vars;
    int nvars;
    int i;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
 
+   scip = GCGgetOrigprob(gcg);
    vars = SCIPgetVars(scip);
    nvars = SCIPgetNVars(scip);
 
@@ -311,15 +378,19 @@ SCIP_RETCODE GCGfreeOrigVarsData(
 
 /** creates the data for a variable of the original program */
 SCIP_RETCODE GCGorigVarCreateData(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP_VAR*             var                 /**< pointer to variable object */
    )
 {
+   SCIP* scip;
    SCIP_VARDATA* vardata;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(var != NULL);
    assert(SCIPvarIsOriginal(var) || SCIPvarGetStatus(var) == SCIP_VARSTATUS_LOOSE);
+   
+   scip = GCGgetOrigprob(gcg);
+   
    /* create the vardata and initialize its values */
    SCIP_CALL( SCIPallocBlockMemory(scip, &vardata) );
    vardata->vartype = GCG_VARTYPE_ORIGINAL;
@@ -344,6 +415,9 @@ SCIP_RETCODE GCGorigVarCreateData(
       SCIP_CALL( GCGvarDelOrig(scip, var, &oldvardata) );
    }
    SCIPvarSetData(var, vardata);
+
+   SCIPvarSetCopyData(var, gcgvarcopy);
+
    if( SCIPvarIsOriginal(var) )
    {
       SCIPvarSetDelorigData(var, GCGvarDelOrig);
@@ -365,11 +439,12 @@ SCIP_RETCODE GCGorigVarCreateData(
  * subproblems are merged into the master problem.
  */
 SCIP_RETCODE GCGcopyPricingvarDataToMastervar(
-   SCIP*                 scip,               /**< master SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP_VAR*             pricingvar,         /**< the pricing problem variable is copied from */
    SCIP_VAR*             mastervar           /**< the master variable that the vardata is copied to */
    )
 {
+   SCIP* scip;
    SCIP_VARDATA* targetvardata;
    SCIP_VAR* origvar;
    assert(pricingvar != NULL);
@@ -379,6 +454,8 @@ SCIP_RETCODE GCGcopyPricingvarDataToMastervar(
 
    assert(GCGpricingVarGetNOrigvars(pricingvar) == 1);
 
+   scip = GCGgetMasterprob(gcg);
+
    /* create vardata */
    SCIP_CALL( SCIPallocBlockMemory(scip, &targetvardata) );
    targetvardata->vartype = GCG_VARTYPE_MASTER;
@@ -529,7 +606,7 @@ SCIP_VAR* GCGpricingVarGetOriginalVar(
 
 /** adds the original var to the pricing variable */
 SCIP_RETCODE GCGpricingVarAddOrigVar(
-   SCIP*                 scip,               /**< SCIP variable structure */
+   SCIP*                 pricingprob,        /**< pricingprob SCIP data structure */
    SCIP_VAR*             pricingvar,         /**< pricing variable */
    SCIP_VAR*             origvar             /**< original pricing variable */
    )
@@ -550,8 +627,8 @@ SCIP_RETCODE GCGpricingVarAddOrigVar(
    /* realloc origvars array of the pricing variable, if needed */
    if( vardata->data.pricingvardata.maxorigvars == vardata->data.pricingvardata.norigvars )
    {
-      int newsize = SCIPcalcMemGrowSize(scip, vardata->data.pricingvardata.norigvars+1);
-      SCIP_CALL( SCIPreallocBlockMemoryArray(scip, &(vardata->data.pricingvardata.origvars), vardata->data.pricingvardata.maxorigvars,
+      int newsize = SCIPcalcMemGrowSize(pricingprob, vardata->data.pricingvardata.norigvars+1);
+      SCIP_CALL( SCIPreallocBlockMemoryArray(pricingprob, &(vardata->data.pricingvardata.origvars), vardata->data.pricingvardata.maxorigvars,
             newsize) );
       SCIPdebugMessage("origvars array of var %s resized from %d to %d\n", SCIPvarGetName(origvar),
          vardata->data.pricingvardata.maxorigvars, newsize);
@@ -677,21 +754,24 @@ void GCGoriginalVarSetNCoefs(
 
 /** adds a coefficient of the master variable to the coefs array for the resp. constraint */
 SCIP_RETCODE GCGoriginalVarAddCoef(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP_VAR*             var,                /**< variable to add coef */
    SCIP_Real             val,                /**< coefficent to set */
    SCIP_CONS*            cons                /**< constraint the variable is in */
    )
 {
+   SCIP* scip;
    SCIP_VARDATA* vardata;
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(var != NULL);
-   assert(!SCIPisZero(scip, val));
    assert(cons != NULL);
    assert(GCGvarIsOriginal(var));
    vardata = SCIPvarGetData(var);
    assert(vardata != NULL);
 
+   scip = GCGgetOrigprob(gcg);
+   assert(!SCIPisZero(scip, val));
+
    SCIP_CALL( SCIPreallocBlockMemoryArray(scip, &(vardata->data.origvardata.coefs), (size_t)vardata->data.origvardata.ncoefs, (size_t)vardata->data.origvardata.ncoefs+1) );
    SCIP_CALL( SCIPreallocBlockMemoryArray(scip, &(vardata->data.origvardata.masterconss), (size_t)vardata->data.origvardata.ncoefs, (size_t)vardata->data.origvardata.ncoefs+1) );
 
@@ -724,7 +804,7 @@ SCIP_CONS** GCGoriginalVarGetMasterconss(
 
 /** adds variable to a new block, making a linkingvariable out of it, if necessary */
 SCIP_RETCODE GCGoriginalVarAddBlock(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< SCIP data structure */
    SCIP_VAR*             var,                /**< var that is added to a block */
    int                   newblock,           /**< the new block the variable will be in */
    int                   nblocks,            /**< total number of pricing problems */
@@ -733,28 +813,31 @@ SCIP_RETCODE GCGoriginalVarAddBlock(
 {
    SCIP_VARDATA* vardata;
    int blocknr;
-   assert(scip != NULL);
+   SCIP* origprob;
+   assert(gcg != NULL);
    assert(var != NULL);
    assert(GCGvarIsOriginal(var));
 
    vardata = SCIPvarGetData(var);
    assert(vardata != NULL);
 
+   origprob = GCGgetOrigprob(gcg);
+
    assert(nblocks >= 0);
    assert((newblock >= 0 && newblock < nblocks)
-      || (GCGgetDecompositionMode(scip) == GCG_DECMODE_BENDERS && newblock == -2));
+      || (GCGgetDecompositionMode(gcg) == GCG_DECMODE_BENDERS && newblock == -2));
    blocknr = GCGvarGetBlock(var);
    assert(newblock >= 0 || (newblock == -2 && blocknr > -1));
    /* the variable was only in one block so far, so set up the linking variable data */
    if( blocknr > -1 )
    {
-      SCIP_CALL( SCIPallocBlockMemory(scip, &vardata->data.origvardata.linkingvardata) );
-      SCIP_CALL( SCIPallocBlockMemoryArray(scip, &vardata->data.origvardata.linkingvardata->pricingvars, nblocks) );
+      SCIP_CALL( SCIPallocBlockMemory(origprob, &vardata->data.origvardata.linkingvardata) );
+      SCIP_CALL( SCIPallocBlockMemoryArray(origprob, &vardata->data.origvardata.linkingvardata->pricingvars, nblocks) );
       BMSclearMemoryArray(vardata->data.origvardata.linkingvardata->pricingvars, nblocks);
 
       if( mode != GCG_DECMODE_BENDERS )
       {
-         SCIP_CALL( SCIPallocBlockMemoryArray(scip, &vardata->data.origvardata.linkingvardata->linkconss, nblocks) );
+         SCIP_CALL( SCIPallocBlockMemoryArray(origprob, &vardata->data.origvardata.linkingvardata->linkconss, nblocks) );
          BMSclearMemoryArray(vardata->data.origvardata.linkingvardata->linkconss, nblocks);
       }
       else
@@ -1119,17 +1202,21 @@ SCIP_Bool GCGisMasterVarInBlock(
  * @todo this method needs a little love
  */
 SCIP_RETCODE GCGoriginalVarAddMasterVar(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP_VAR*             origvar,            /**< original variable */
    SCIP_VAR*             var,                /**< master variable */
    SCIP_Real             val                 /**< fraction of the original variable */
    )
 {
+   SCIP* scip;
    SCIP_VARDATA* vardata;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(origvar != NULL);
    assert(var != NULL);
+
+   scip = GCGgetOrigprob(gcg);
+
    assert(GCGisOriginal(scip));
    vardata = SCIPvarGetData(origvar);
 
@@ -1166,7 +1253,7 @@ SCIP_RETCODE GCGoriginalVarAddMasterVar(
  * @todo this method needs a little love
  */
 SCIP_RETCODE GCGoriginalVarRemoveMasterVar(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP_VAR*             origvar,            /**< original variable */
    SCIP_VAR*             var                 /**< master variable */
    )
@@ -1174,7 +1261,7 @@ SCIP_RETCODE GCGoriginalVarRemoveMasterVar(
    SCIP_VARDATA* vardata;
    int i;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(origvar != NULL);
    assert(var != NULL);
 
@@ -1211,7 +1298,7 @@ SCIP_RETCODE GCGoriginalVarRemoveMasterVar(
 
 /** creates the corresponding pricing variable for the given original variable */
 SCIP_RETCODE GCGoriginalVarCreatePricingVar(
-   SCIP*                 scip,               /**< SCIP data structure */
+   SCIP*                 pricingprob,        /**< pricingprob SCIP data structure */
    SCIP_VAR*             origvar,            /**< original variable */
    SCIP_VAR**            var                 /**< pricing variable */
    )
@@ -1219,7 +1306,7 @@ SCIP_RETCODE GCGoriginalVarCreatePricingVar(
    SCIP_VARDATA* vardata;
    char name[SCIP_MAXSTRLEN];
    int pricingprobnr;
-   assert(scip != NULL);
+   assert(pricingprob != NULL);
    assert(origvar != NULL);
    assert(var != NULL);
    assert(GCGvarIsOriginal(origvar));
@@ -1230,18 +1317,18 @@ SCIP_RETCODE GCGoriginalVarCreatePricingVar(
    pricingprobnr = GCGvarGetBlock(origvar);
 
    /* create variable data */
-   SCIP_CALL( SCIPallocBlockMemory(scip, &vardata) );
+   SCIP_CALL( SCIPallocBlockMemory(pricingprob, &vardata) );
    vardata->vartype = GCG_VARTYPE_PRICING;
    vardata->blocknr = pricingprobnr;
    vardata->data.pricingvardata.maxorigvars = STARTMAXORIGVARS;
-   SCIP_CALL( SCIPallocBlockMemoryArray(scip, &(vardata->data.pricingvardata.origvars), vardata->data.pricingvardata.maxorigvars) ); /*lint !e506*/
+   SCIP_CALL( SCIPallocBlockMemoryArray(pricingprob, &(vardata->data.pricingvardata.origvars), vardata->data.pricingvardata.maxorigvars) ); /*lint !e506*/
    vardata->data.pricingvardata.origvars[0] = origvar;
    vardata->data.pricingvardata.norigvars = 1;
 
    (void) SCIPsnprintf(name, SCIP_MAXSTRLEN, "pr%d_%s", pricingprobnr, SCIPvarGetName(origvar));
-   SCIP_CALL( SCIPcreateVar(scip, var, name, SCIPvarGetLbGlobal(origvar),
+   SCIP_CALL( SCIPcreateVar(pricingprob, var, name, SCIPvarGetLbGlobal(origvar),
          SCIPvarGetUbGlobal(origvar), 0.0, SCIPvarGetType(origvar),
-         TRUE, FALSE, GCGvarDelOrig, NULL, NULL, NULL, vardata) );
+         TRUE, FALSE, GCGvarDelOrig, NULL, NULL, gcgvarcopy, vardata) );
 
    return SCIP_OKAY;
 }
@@ -1276,14 +1363,14 @@ SCIP_RETCODE GCGlinkingVarCreatePricingVar(
    (void) SCIPsnprintf(name, SCIP_MAXSTRLEN, "pr%d_%s", pricingprobnr, SCIPvarGetName(origvar));
    SCIP_CALL( SCIPcreateVar(pricingscip, var, name, SCIPvarGetLbGlobal(origvar),
          SCIPvarGetUbGlobal(origvar), 0.0, SCIPvarGetType(origvar),
-         TRUE, FALSE, GCGvarDelOrig, NULL, NULL, NULL, vardata) );
+         TRUE, FALSE, GCGvarDelOrig, NULL, NULL, gcgvarcopy, vardata) );
 
    return SCIP_OKAY;
 }
 
 /** creates the corresponding constraint in the master problem for the linking variable */
 SCIP_RETCODE GCGlinkingVarCreateMasterCons(
-   SCIP*                 masterscip,         /**< msater problem SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    int                   pricingprobnr,      /**< number of the pricing problem */
    SCIP_VAR*             origvar,            /**< original variable */
    SCIP_CONS**           linkcons            /**< constraint linking pricing variables */
@@ -1291,7 +1378,7 @@ SCIP_RETCODE GCGlinkingVarCreateMasterCons(
 {
    char name[SCIP_MAXSTRLEN];
 
-   assert(masterscip != NULL);
+   assert(gcg != NULL);
    assert(pricingprobnr >= 0);
    assert(origvar != NULL);
    assert(GCGoriginalVarIsLinking(origvar));
@@ -1299,7 +1386,7 @@ SCIP_RETCODE GCGlinkingVarCreateMasterCons(
 
    /* add corresponding linking constraint to the master problem */
    (void) SCIPsnprintf(name, SCIP_MAXSTRLEN, "l_%s_%d", SCIPvarGetName(origvar), pricingprobnr);
-   SCIP_CALL( SCIPcreateConsLinear(masterscip, linkcons, name, 0, NULL, NULL, 0.0, 0.0,
+   SCIP_CALL( SCIPcreateConsLinear(GCGgetMasterprob(gcg), linkcons, name, 0, NULL, NULL, 0.0, 0.0,
          TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, TRUE, FALSE, FALSE, FALSE) );
 
    return SCIP_OKAY;
@@ -1307,8 +1394,7 @@ SCIP_RETCODE GCGlinkingVarCreateMasterCons(
 
 /** creates the master var and initializes the vardata */
 SCIP_RETCODE GCGcreateMasterVar(
-   SCIP*                 scip,               /**< master SCIP data structure */
-   SCIP*                 origscip,           /**< original SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP*                 pricingscip,        /**< pricing problem SCIP data structure */
    SCIP_VAR**            newvar,             /**< pointer to store new master variable */
    const char*           varname,            /**< new variable name */
@@ -1322,13 +1408,16 @@ SCIP_RETCODE GCGcreateMasterVar(
    SCIP_Bool             auxiliaryvar        /**< is new variable an Benders' auxiliary variables? */
    )
 {
+   SCIP* scip;
    SCIP_VARDATA* newvardata;
+   SCIP_VAR** pricingvars;
+   int npricingvars;
    SCIP_Real lb;
    int i;
    int j;
    SCIP_Bool trivialsol;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(pricingscip != NULL);
    assert(newvar != NULL);
    assert(varname != NULL);
@@ -1339,7 +1428,9 @@ SCIP_RETCODE GCGcreateMasterVar(
    assert(solvals != NULL || nsolvars == 0);
    assert(solvars != NULL || nsolvars == 0);
 
+   scip = GCGgetMasterprob(gcg);
    trivialsol = FALSE;
+   npricingvars = 0;
 
    lb = 0.0;
    if( auxiliaryvar )
@@ -1368,6 +1459,11 @@ SCIP_RETCODE GCGcreateMasterVar(
       assert(solvars != NULL);
       assert(solvals != NULL);
 
+      assert(GCGvarIsPricing(solvars[i]) || GCGvarIsInferredPricing(solvars[i]));
+
+      if( GCGvarIsInferredPricing(solvars[i]) )
+         continue;
+
       assert(!SCIPisInfinity(scip, solvals[i]));
       if( !SCIPisZero(scip, solvals[i]) )
       {
@@ -1381,7 +1477,9 @@ SCIP_RETCODE GCGcreateMasterVar(
     */
    if( newvardata->data.mastervardata.norigvars == 0 && !auxiliaryvar )
    {
-      newvardata->data.mastervardata.norigvars = SCIPgetNOrigVars(pricingscip);
+      npricingvars = SCIPgetNOrigVars(pricingscip);
+      pricingvars = SCIPgetOrigVars(pricingscip);
+      newvardata->data.mastervardata.norigvars = npricingvars - GCGcountInferredCoefPricingVars(pricingvars, npricingvars);
       trivialsol = TRUE;
    }
 
@@ -1410,12 +1508,16 @@ SCIP_RETCODE GCGcreateMasterVar(
       assert(solvars != NULL);
       assert(solvals != NULL);
 
+      assert(GCGvarIsPricing(solvars[i]) || GCGvarIsInferredPricing(solvars[i]));
+
+      if( GCGvarIsInferredPricing(solvars[i]) )
+         continue;
+
       solval = solvals[i];
 
       if( !SCIPisZero(scip, solval) )
       {
          SCIP_VAR* origvar;
-         assert(GCGvarIsPricing(solvars[i]));
 
          origvar = GCGpricingVarGetOrigvars(solvars[i])[0];
          assert(origvar != NULL);
@@ -1434,54 +1536,60 @@ SCIP_RETCODE GCGcreateMasterVar(
          newvardata->data.mastervardata.origvals[j] = solval;
          SCIPhashmapInsertReal(newvardata->data.mastervardata.origvar2val, origvar, solval);
          /* save the quota in the original variable's data */
-         SCIP_CALL( GCGoriginalVarAddMasterVar(origscip, origvar, *newvar, solval) );
+         SCIP_CALL( GCGoriginalVarAddMasterVar(gcg, origvar, *newvar, solval) );
          j++;
       }
    }
    if( trivialsol )
    {
-      SCIP_VAR** pricingvars;
-      int npricingvars;
-
       pricingvars = SCIPgetOrigVars(pricingscip);
       npricingvars = SCIPgetNOrigVars(pricingscip);
-      for( j = 0; j < npricingvars; ++j )
+      assert(j == 0);
+
+      for( i = 0; i < npricingvars; ++i )
       {
          SCIP_VAR* origvar;
-         assert(GCGvarIsPricing(pricingvars[j]));
+         assert(GCGvarIsPricing(pricingvars[i]) || GCGvarIsInferredPricing(pricingvars[i]));
 
-         origvar = GCGpricingVarGetOrigvars(pricingvars[j])[0];
+         if( GCGvarIsInferredPricing(pricingvars[i]) )
+            continue;
+
+         origvar = GCGpricingVarGetOrigvars(pricingvars[i])[0];
          assert(origvar != NULL);
 
          assert(newvardata->data.mastervardata.origvars != NULL);
          assert(newvardata->data.mastervardata.origvals != NULL);
          assert(GCGvarIsOriginal(origvar));
          /* save in the master problem variable's data the quota of the corresponding original variable */
-         newvardata->data.mastervardata.origvars[j] = origvar;
-         newvardata->data.mastervardata.origvals[j] = 0.0;
+         newvardata->data.mastervardata.origvars[i] = origvar;
+         newvardata->data.mastervardata.origvals[i] = 0.0;
          SCIPhashmapInsertReal(newvardata->data.mastervardata.origvar2val, origvar, 0.0);
          /* save the quota in the original variable's data */
-         SCIP_CALL( GCGoriginalVarAddMasterVar(origscip, origvar, *newvar, 0.0) );
+         SCIP_CALL( GCGoriginalVarAddMasterVar(gcg, origvar, *newvar, 0.0) );
       }
+      j = i;
    }
    assert(j == newvardata->data.mastervardata.norigvars);
-return SCIP_OKAY;
+
+   return SCIP_OKAY;
 }
 
 /** creates initial master variables and the vardata */
 SCIP_RETCODE GCGcreateInitialMasterVar(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP_VAR*             var,                /**< original variable */
    SCIP_VAR**            newvar              /**< pointer to store new variable */
 
    )
 {
+   SCIP* masterprob;
    SCIP_VARDATA* newvardata;
    int blocknr;
 
+   masterprob = GCGgetMasterprob(gcg);
    blocknr = GCGvarGetBlock(var);
    assert( blocknr == -1 || blocknr == -2
-           || GCGgetMasterDecompMode(scip) == GCG_DECMODE_BENDERS || GCGgetMasterDecompMode(scip) == GCG_DECMODE_ORIGINAL);
+           || GCGgetDecompositionMode(gcg) == GCG_DECMODE_BENDERS || GCGgetDecompositionMode(gcg) == GCG_DECMODE_ORIGINAL);
 
    if( blocknr == -1 )
    {
@@ -1493,7 +1601,7 @@ SCIP_RETCODE GCGcreateInitialMasterVar(
    }
 
    /* create vardata */
-   SCIP_CALL( SCIPallocBlockMemory(scip, &newvardata) );
+   SCIP_CALL( SCIPallocBlockMemory(masterprob, &newvardata) );
    newvardata->vartype = GCG_VARTYPE_MASTER;
    newvardata->blocknr = -1;
    newvardata->data.mastervardata.isray = FALSE;
@@ -1503,15 +1611,15 @@ SCIP_RETCODE GCGcreateInitialMasterVar(
    newvardata->data.mastervardata.index = -1;
 
    /* save corresoponding origvar */
-   SCIP_CALL( SCIPallocBlockMemoryArray(scip, &(newvardata->data.mastervardata.origvars), 1) ); /*lint !e506*/
-   SCIP_CALL( SCIPallocBlockMemoryArray(scip, &(newvardata->data.mastervardata.origvals), 1) ); /*lint !e506*/
-   SCIP_CALL( SCIPhashmapCreate(&(newvardata->data.mastervardata.origvar2val), SCIPblkmem(scip), 1) );
+   SCIP_CALL( SCIPallocBlockMemoryArray(masterprob, &(newvardata->data.mastervardata.origvars), 1) ); /*lint !e506*/
+   SCIP_CALL( SCIPallocBlockMemoryArray(masterprob, &(newvardata->data.mastervardata.origvals), 1) ); /*lint !e506*/
+   SCIP_CALL( SCIPhashmapCreate(&(newvardata->data.mastervardata.origvar2val), SCIPblkmem(masterprob), 1) );
    newvardata->data.mastervardata.origvars[0] = var;
    newvardata->data.mastervardata.origvals[0] = 1.0;
    SCIPhashmapInsertReal(newvardata->data.mastervardata.origvar2val, var, 1.0);
 
    /* create variable in the master problem */
-   SCIP_CALL( SCIPcreateVar(scip, newvar, SCIPvarGetName(var),
+   SCIP_CALL( SCIPcreateVar(masterprob, newvar, SCIPvarGetName(var),
          SCIPvarGetLbGlobal(var), SCIPvarGetUbGlobal(var), SCIPvarGetObj(var), SCIPvarGetType(var),
          TRUE, TRUE, NULL, NULL, gcgvardeltrans, NULL, newvardata) );
 
@@ -1520,14 +1628,17 @@ SCIP_RETCODE GCGcreateInitialMasterVar(
 
 /** creates artificial variable and the vardata */
 SCIP_RETCODE GCGcreateArtificialVar(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP_VAR**            newvar,             /**< pointer to store new variable */
    const char*           name,               /**< name of variable, or NULL for automatic name creation */
    SCIP_Real             objcoef             /**< objective coefficient of artificial variable */
    )
 {
+   SCIP* scip;
    SCIP_VARDATA* newvardata;
 
+   scip = GCGgetMasterprob(gcg);
+
    /* create vardata */
    SCIP_CALL( SCIPallocBlockMemory(scip, &newvardata) );
    newvardata->vartype = GCG_VARTYPE_MASTER;
@@ -1543,25 +1654,64 @@ SCIP_RETCODE GCGcreateArtificialVar(
    newvardata->data.mastervardata.index = -1;
 
    /* create variable in the master problem */
-   SCIP_CALL( SCIPcreateVar(scip, newvar, name,
-         0.0, SCIPinfinity(scip), objcoef, SCIP_VARTYPE_IMPLINT,
+   SCIP_CALL( SCIPcreateVarImpl(scip, newvar, name,
+         0.0, SCIPinfinity(scip), objcoef, SCIP_VARTYPE_CONTINUOUS, SCIP_IMPLINTTYPE_WEAK,
          TRUE, TRUE, NULL, NULL, gcgvardeltrans, NULL, newvardata) );
 
    return SCIP_OKAY;
 }
 
-/* adds the vardata to the auxiliary variable */
-SCIP_RETCODE GCGaddDataAuxiliaryVar(
-   SCIP*                 scip,               /**< SCIP data structure */
-   SCIP_VAR*             auxiliaryvar,       /**< the auxiliary variable */
-   int                   probnumber          /**< the subproblem number */
+/** creates a pricing variable inferred from an extended master cons
+ * that does not correspond to any original variable and its vardata */
+SCIP_RETCODE GCGcreateInferredPricingVar(
+   SCIP*                 pricingscip,        /**< pricing problem SCIP data structure */
+   SCIP_VAR**            newvar,             /**< pointer to store new master variable */
+   const char*           varname,            /**< new variable name */
+   SCIP_Real             lb,                 /**< new variable lower bound */
+   SCIP_Real             ub,                 /**< new variable lower bound */
+   SCIP_Bool             iscoefvar,          /**< is this a coefficient variable? (objcoeff can be 0 if TRUE but not != 0 if FALSE) */
+   SCIP_Real             objcoeff,           /**< new objective coefficient */
+   SCIP_VARTYPE          vartype,            /**< new variable type */
+   int                   prob                /**< number of pricing problem that created this variable */
    )
 {
    SCIP_VARDATA* newvardata;
 
-   assert(scip != NULL);
+   assert(pricingscip != NULL);
+   assert(newvar != NULL);
+   assert(varname != NULL);
+   assert(iscoefvar || SCIPisZero(pricingscip, objcoeff));
+
+   SCIP_CALL( SCIPallocBlockMemory(pricingscip, &newvardata) );
+   newvardata->vartype = GCG_VARTYPE_INFERREDPRICING;
+   newvardata->blocknr = prob;
+   newvardata->data.inferredpricingvardata.extendedmasterconsdata = NULL; // will be set in GCGextendedmasterconsCreateFrom*
+   newvardata->data.inferredpricingvardata.iscoefvar = iscoefvar;
+
+   /* create variable in the master problem */
+   SCIP_CALL( SCIPcreateVar(pricingscip, newvar, varname, lb, ub,
+         objcoeff, vartype, TRUE, TRUE, GCGvarDelInferredPricing,
+         NULL, NULL, NULL, newvardata) );
+
+   return SCIP_OKAY;
+}
+
+
+/* adds the vardata to the auxiliary variable */
+SCIP_RETCODE GCGaddDataAuxiliaryVar(
+   GCG*                  gcg,                /**< GCG data structure */
+   SCIP_VAR*             auxiliaryvar,       /**< the auxiliary variable */
+   int                   probnumber          /**< the subproblem number */
+   )
+{
+   SCIP* scip;
+   SCIP_VARDATA* newvardata;
+
+   assert(gcg != NULL);
    assert(auxiliaryvar != NULL);
 
+   scip = GCGgetMasterprob(gcg);
+
    /* create data for the new variable in the master problem */
    SCIP_CALL( SCIPallocBlockMemory(scip, &newvardata) );
    newvardata->vartype = GCG_VARTYPE_MASTER;
@@ -1729,15 +1879,15 @@ SCIP_Real GCGgetVarGap(
 
 /** store reduced cost */
 void GCGsetRedcost(
-   SCIP*                 scip,               /**< master SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP_VAR*             var,                /**< variable data structure */
    SCIP_Real             redcost             /**< reduced cost of the variable at creation */
    )
 {
    SCIP_VARDATA* vardata;
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(var != NULL);
-   assert(SCIPisLE(scip, redcost, 0.0));
+   assert(SCIPisLE(GCGgetOrigprob(gcg), redcost, 0.0));
 
    vardata = SCIPvarGetData(var);
    vardata->redcost = redcost;
@@ -1759,14 +1909,19 @@ SCIP_Real GCGgetRedcost(
 
 /** updates the statistics part of the variable */
 void GCGupdateVarStatistics(
-    SCIP*                scip,               /**< master SCIP data structure */
-    SCIP*                origprob,           /**< original SCIP data structure */
+    GCG*                 gcg,                /**< GCG data structure */
     SCIP_VAR*            newvar,             /**< new variable for statistic update */
     SCIP_Real            redcost             /**< reduced cost of the variable */
     )
 {
+   SCIP* scip;
+   SCIP* origprob;
    SCIP_Longint redcostcall;
-   assert(scip != NULL);
+   assert(gcg != NULL);
+
+   scip = GCGgetMasterprob(gcg);
+   origprob = GCGgetOrigprob(gcg);
+
    assert(GCGisMaster(scip));
    assert(origprob != NULL);
    assert(GCGisOriginal(origprob));
@@ -1779,7 +1934,7 @@ void GCGupdateVarStatistics(
    GCGsetRootRedcostCall(newvar, redcostcall);
    GCGsetIteration(newvar, SCIPgetNLPIterations(scip));
    GCGsetVarGap(newvar, MIN(SCIPgetGap(origprob), SCIPgetGap(scip))); /*lint !e666*/
-   GCGsetRedcost(origprob, newvar, redcost);
+   GCGsetRedcost(gcg, newvar, redcost);
 
 }
 
@@ -1787,15 +1942,17 @@ void GCGupdateVarStatistics(
  * and the list of all variables related to the given variable
  */
 void GCGprintVar(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    FILE*                 file,               /**< File to write information to, or NULL for stdout */
    SCIP_VAR*             var                 /**< variable that should be printed */
    )
 {
+   SCIP* scip;
    int i;
    int blocknr;
    assert(GCGvarIsOriginal(var) || GCGvarIsMaster(var) || GCGvarIsPricing(var));
 
+   scip = GCGgetOrigprob(gcg);
    blocknr = GCGvarGetBlock(var);
 
    if( GCGvarIsOriginal(var) )
@@ -1899,3 +2056,33 @@ void GCGmasterVarSetIndex(
    vardata = SCIPvarGetData(var);
    vardata->data.mastervardata.index = index;
 }
+
+#ifndef NDEBUG
+/* returns the extended master cons data of the inferred pricing var */
+GCG_EXTENDEDMASTERCONSDATA* GCGinferredPricingVarGetExtendedmasterconsdata(
+   SCIP_VAR*             var                 /**< SCIP variable structure */
+   )
+{
+   SCIP_VARDATA* vardata;
+   assert(var != NULL);
+   assert(GCGvarIsInferredPricing(var));
+
+   vardata = SCIPvarGetData(var);
+   return vardata->data.inferredpricingvardata.extendedmasterconsdata;
+}
+#endif
+
+#ifndef NDEBUG
+/* returns whether the inferred pricing variable is a coefficient variable */
+SCIP_Bool GCGinferredPricingVarIsCoefVar(
+   SCIP_VAR*             var                 /**< SCIP variable structure */
+   )
+{
+   SCIP_VARDATA* vardata;
+   assert(var != NULL);
+   assert(GCGvarIsInferredPricing(var));
+
+   vardata = SCIPvarGetData(var);
+   return vardata->data.inferredpricingvardata.iscoefvar;
+}
+#endif
