@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -36,9 +37,9 @@
 #include <assert.h>
 #include <string.h>
 
-#include "heur_gcgveclendiving.h"
-#include "heur_origdiving.h"
-#include "gcg.h"
+#include "gcg/heur_gcgveclendiving.h"
+#include "gcg/heur_origdiving.h"
+#include "gcg/gcg.h"
 
 
 #define HEUR_NAME             "gcgveclendiving"
@@ -161,7 +162,7 @@ SCIP_Bool areVarsInSameBlock(
  */
 static
 SCIP_RETCODE getMasterDownScore(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_DIVINGDATA*       divingdata,         /**< diving heuristic data */
    SCIP_VAR*             var,                /**< original variable to get fractionality for */
    SCIP_Real*            score               /**< pointer to store fractionality */
@@ -173,11 +174,11 @@ SCIP_RETCODE getMasterDownScore(
    int nmastervars;
    int norigmastervars;
    SCIP_Real roundval;
-
    int i;
+   SCIP* origprob = GCGgetOrigprob(gcg);
 
    /* get master problem */
-   masterprob = GCGgetMasterprob(scip);
+   masterprob = GCGgetMasterprob(gcg);
    assert(masterprob != NULL);
 
    /* get master variable information */
@@ -187,7 +188,7 @@ SCIP_RETCODE getMasterDownScore(
    origmastervals = GCGoriginalVarGetMastervals(var);
    norigmastervars = GCGoriginalVarGetNMastervars(var);
 
-   roundval = SCIPfeasFloor(scip, SCIPgetRelaxSolVal(scip, var));
+   roundval = SCIPfeasFloor(origprob, SCIPgetRelaxSolVal(origprob, var));
    *score = 0.0;
 
    /* calculate sum of scores over all master variables
@@ -218,7 +219,7 @@ SCIP_RETCODE getMasterDownScore(
  */
 static
 SCIP_RETCODE getMasterUpScore(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_DIVINGDATA*       divingdata,         /**< diving heuristic data */
    SCIP_VAR*             var,                /**< original variable to get fractionality for */
    SCIP_Real*            score               /**< pointer to store fractionality */
@@ -230,11 +231,11 @@ SCIP_RETCODE getMasterUpScore(
    int nmastervars;
    int norigmastervars;
    SCIP_Real roundval;
-
    int i;
+   SCIP* origprob = GCGgetOrigprob(gcg);
 
    /* get master problem */
-   masterprob = GCGgetMasterprob(scip);
+   masterprob = GCGgetMasterprob(gcg);
    assert(masterprob != NULL);
 
    /* get master variable information */
@@ -244,7 +245,7 @@ SCIP_RETCODE getMasterUpScore(
    origmastervals = GCGoriginalVarGetMastervals(var);
    norigmastervars = GCGoriginalVarGetNMastervars(var);
 
-   roundval = SCIPfeasCeil(scip, SCIPgetRelaxSolVal(scip, var));
+   roundval = SCIPfeasCeil(origprob, SCIPgetRelaxSolVal(origprob, var));
    *score = 0.0;
 
    /* calculate sum of scores over all master variables
@@ -272,7 +273,7 @@ SCIP_RETCODE getMasterUpScore(
 /** for a variable, calculate the vector length score w.r.t. the master problem */
 static
 SCIP_RETCODE calculateScoreMaster(
-   SCIP*                 scip,               /**< SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_DIVINGDATA*       divingdata,         /**< diving heuristic data */
    SCIP_VAR*             var,                /**< variable to calculate score for */
    SCIP_Real*            score,              /**< pointer to return the score */
@@ -283,8 +284,8 @@ SCIP_RETCODE calculateScoreMaster(
    SCIP_Real upscore;
 
    /* calculate scores for rounding down or up */
-   SCIP_CALL( getMasterDownScore(scip, divingdata, var, &downscore) );
-   SCIP_CALL( getMasterUpScore(scip, divingdata, var, &upscore) );
+   SCIP_CALL( getMasterDownScore(gcg, divingdata, var, &downscore) );
+   SCIP_CALL( getMasterUpScore(gcg, divingdata, var, &upscore) );
 
    *score = MIN(downscore, upscore);
    *roundup = upscore <= downscore;
@@ -309,12 +310,11 @@ GCG_DECL_DIVINGFREE(heurFreeGcgveclendiving) /*lint --e{715}*/
    GCG_DIVINGDATA* divingdata;
 
    assert(heur != NULL);
-   assert(scip != NULL);
 
    /* free diving rule specific data */
    divingdata = GCGheurGetDivingDataOrig(heur);
    assert(divingdata != NULL);
-   SCIPfreeMemory(scip, &divingdata);
+   SCIPfreeMemory(GCGgetOrigprob(gcg), &divingdata);
    GCGheurSetDivingDataOrig(heur, NULL);
 
    return SCIP_OKAY;
@@ -330,8 +330,8 @@ GCG_DECL_DIVINGINITEXEC(heurInitexecGcgveclendiving) /*lint --e{715}*/
    SCIP_SOL* masterlpsol;
    SCIP_VAR** mastervars;
    int nmastervars;
-
    int i;
+   SCIP* origprob = GCGgetOrigprob(gcg);
 
    assert(heur != NULL);
 
@@ -344,14 +344,14 @@ GCG_DECL_DIVINGINITEXEC(heurInitexecGcgveclendiving) /*lint --e{715}*/
       return SCIP_OKAY;
 
    /* get master problem */
-   masterprob = GCGgetMasterprob(scip);
+   masterprob = GCGgetMasterprob(gcg);
    assert(masterprob != NULL);
 
    /* get master variables */
    SCIP_CALL( SCIPgetVarsData(masterprob, &mastervars, &nmastervars, NULL, NULL, NULL, NULL) );
 
    /* allocate memory */
-   SCIP_CALL( SCIPallocBufferArray(scip, &divingdata->masterscores, nmastervars) );
+   SCIP_CALL( SCIPallocBufferArray(origprob, &divingdata->masterscores, nmastervars) );
    SCIP_CALL( SCIPcreateSol(masterprob, &masterlpsol, NULL) );
 
    /* get master LP solution */
@@ -368,7 +368,7 @@ GCG_DECL_DIVINGINITEXEC(heurInitexecGcgveclendiving) /*lint --e{715}*/
       objdelta = SCIPfeasFrac(masterprob, SCIPgetSolVal(masterprob, masterlpsol, mastervar)) * SCIPvarGetObj(mastervar);
       objdelta = ABS(objdelta);
       colveclen = (SCIPvarGetStatus(mastervar) == SCIP_VARSTATUS_COLUMN ? SCIPcolGetNNonz(SCIPvarGetCol(mastervar)) : 0);
-      divingdata->masterscores[i] = (objdelta + SCIPsumepsilon(scip))/((SCIP_Real)colveclen+1.0);
+      divingdata->masterscores[i] = (objdelta + SCIPsumepsilon(origprob))/((SCIP_Real)colveclen+1.0);
    }
 
    /* free memory */
@@ -383,6 +383,7 @@ static
 GCG_DECL_DIVINGEXITEXEC(heurExitexecGcgveclendiving) /*lint --e{715}*/
 {  /*lint --e{715}*/
    GCG_DIVINGDATA* divingdata;
+   SCIP* origprob = GCGgetOrigprob(gcg);
 
    assert(heur != NULL);
 
@@ -395,7 +396,7 @@ GCG_DECL_DIVINGEXITEXEC(heurExitexecGcgveclendiving) /*lint --e{715}*/
       return SCIP_OKAY;
 
    /* free memory */
-   SCIPfreeBufferArray(scip, &divingdata->masterscores);
+   SCIPfreeBufferArray(origprob, &divingdata->masterscores);
 
    return SCIP_OKAY;
 }
@@ -416,9 +417,10 @@ GCG_DECL_DIVINGSELECTVAR(heurSelectVarGcgveclendiving) /*lint --e{715}*/
    int nlpcands;
    SCIP_Real bestscore;
    int c;
+   SCIP* origprob = GCGgetOrigprob(gcg);
 
    /* check preconditions */
-   assert(scip != NULL);
+   assert(origprob != NULL);
    assert(heur != NULL);
    assert(bestcand != NULL);
    assert(bestcandmayround != NULL);
@@ -429,7 +431,7 @@ GCG_DECL_DIVINGSELECTVAR(heurSelectVarGcgveclendiving) /*lint --e{715}*/
    assert(divingdata != NULL);
 
    /* get fractional variables that should be integral */
-   SCIP_CALL( SCIPgetExternBranchCands(scip, &lpcands, &lpcandssol, NULL, &nlpcands, NULL, NULL, NULL, NULL) );
+   SCIP_CALL( SCIPgetExternBranchCands(origprob, &lpcands, &lpcandssol, NULL, &nlpcands, NULL, NULL, NULL, NULL) );
    assert(lpcands != NULL);
    assert(lpcandssol != NULL);
 
@@ -453,11 +455,11 @@ GCG_DECL_DIVINGSELECTVAR(heurSelectVarGcgveclendiving) /*lint --e{715}*/
       /* calculate score */
       if( divingdata->usemasterscores )
       {
-         SCIP_CALL( calculateScoreMaster(scip, divingdata, lpcands[c], &score, &roundup) );
+         SCIP_CALL( calculateScoreMaster(gcg, divingdata, lpcands[c], &score, &roundup) );
       }
       else
       {
-         SCIP_CALL( calculateScoreOrig(scip, lpcands[c], lpcandssol[c] - SCIPfloor(scip, lpcandssol[c]), &score, &roundup) );
+         SCIP_CALL( calculateScoreOrig(origprob, lpcands[c], lpcandssol[c] - SCIPfloor(origprob, lpcandssol[c]), &score, &roundup) );
       }
 
       /* check whether the variable is roundable */
@@ -482,17 +484,18 @@ GCG_DECL_DIVINGSELECTVAR(heurSelectVarGcgveclendiving) /*lint --e{715}*/
 
 /** creates the gcgveclendiving heuristic and includes it in GCG */
 SCIP_RETCODE GCGincludeHeurGcgveclendiving(
-   SCIP*                 scip                /**< SCIP data structure */
+   GCG*                  gcg                 /**< GCG data structure */
    )
 {
    SCIP_HEUR* heur;
    GCG_DIVINGDATA* divingdata;
+   SCIP* origprob = GCGgetOrigprob(gcg);
 
    /* create gcgguideddiving primal heuristic data */
-   SCIP_CALL( SCIPallocMemory(scip, &divingdata) );
+   SCIP_CALL( SCIPallocMemory(origprob, &divingdata) );
 
    /* include diving heuristic */
-   SCIP_CALL( GCGincludeDivingHeurOrig(scip, &heur,
+   SCIP_CALL( GCGincludeDivingHeurOrig(gcg, &heur,
          HEUR_NAME, HEUR_DESC, HEUR_DISPCHAR, HEUR_PRIORITY, HEUR_FREQ, HEUR_FREQOFS,
          HEUR_MAXDEPTH, heurFreeGcgveclendiving, NULL, NULL, NULL, NULL, heurInitexecGcgveclendiving,
          heurExitexecGcgveclendiving, heurSelectVarGcgveclendiving, divingdata) );
@@ -500,7 +503,7 @@ SCIP_RETCODE GCGincludeHeurGcgveclendiving(
    assert(heur != NULL);
 
    /* add gcgveclendiving specific parameters */
-   SCIP_CALL( SCIPaddBoolParam(scip, "heuristics/"HEUR_NAME"/usemasterscores",
+   SCIP_CALL( SCIPaddBoolParam(origprob, "heuristics/"HEUR_NAME"/usemasterscores",
          "calculate vector length scores w.r.t. the master LP?",
          &divingdata->usemasterscores, TRUE, DEFAULT_USEMASTERSCORES, NULL, NULL) );
 
