@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -35,9 +36,9 @@
 
 #include <string.h>
 
-#include "heur_gcgzirounding.h"
-#include "gcg.h"
-#include "relax_gcg.h"
+#include "gcg/heur_gcgzirounding.h"
+#include "gcg/gcg.h"
+#include "gcg/relax_gcg.h"
 
 #define HEUR_NAME             "gcgzirounding"
 #define HEUR_DESC             "rounding heuristic on original variables as suggested by C. Wallace taking row slacks and bounds into account"
@@ -64,6 +65,7 @@
 /** primal heuristic data */
 struct SCIP_HeurData
 {
+   GCG*                  gcg;                /**< GCG data structure */
    SCIP_SOL*             sol;                /**< working solution */
    SCIP_Longint          lastlp;             /**< the number of the last LP for which ZIRounding was called */
    int                   maxroundingloops;   /**< limits rounding loops in execution */
@@ -468,8 +470,12 @@ SCIP_DECL_HEUREXEC(heurExecGcgzirounding)
    assert(strcmp(SCIPheurGetName(heur), HEUR_NAME) == 0);
    assert(result != NULL);
 
+   /* get heuristic data */
+   heurdata = SCIPheurGetData(heur);
+   assert(heurdata != NULL);
+
    /* get master problem */
-   masterprob = GCGgetMasterprob(scip);
+   masterprob = GCGgetMasterprob(heurdata->gcg);
    assert(masterprob != NULL);
 
    *result = SCIP_DIDNOTRUN;
@@ -491,10 +497,6 @@ SCIP_DECL_HEUREXEC(heurExecGcgzirounding)
    if( SCIPgetLPSolstat(masterprob) != SCIP_LPSOLSTAT_OPTIMAL )
       return SCIP_OKAY;
 
-   /* get heuristic data */
-   heurdata = SCIPheurGetData(heur);
-   assert(heurdata != NULL);
-
    /* Do not call heuristic if deactivation check is enabled and percentage of found solutions in relation
     * to number of calls falls below heurdata->stoppercentage */
    if( heurdata->stopziround && SCIPheurGetNCalls(heur) >= heurdata->minstopncalls
@@ -875,40 +877,42 @@ SCIP_DECL_HEUREXEC(heurExecGcgzirounding)
  */
 
 /** creates the GCG zirounding primal heuristic and includes it in SCIP */
-SCIP_RETCODE SCIPincludeHeurGcgzirounding(
-   SCIP*                 scip                /**< SCIP data structure */
+SCIP_RETCODE GCGincludeHeurGcgzirounding(
+   GCG*                  gcg                 /**< GCG data structure */
    )
 {
    SCIP_HEURDATA* heurdata;
    SCIP_HEUR* heur;
+   SCIP* origprob = GCGgetOrigprob(gcg);
 
    /* create GCG zirounding primal heuristic data */
-   SCIP_CALL( SCIPallocBlockMemory(scip, &heurdata) );
+   SCIP_CALL( SCIPallocBlockMemory(origprob, &heurdata) );
+   heurdata->gcg = gcg;
 
    /* include primal heuristic */
-   SCIP_CALL( SCIPincludeHeurBasic(scip, &heur,
+   SCIP_CALL( SCIPincludeHeurBasic(origprob, &heur,
          HEUR_NAME, HEUR_DESC, HEUR_DISPCHAR, HEUR_PRIORITY, HEUR_FREQ, HEUR_FREQOFS,
          HEUR_MAXDEPTH, HEUR_TIMING, HEUR_USESSUBSCIP, heurExecGcgzirounding, heurdata) );
 
    assert(heur != NULL);
 
    /* set non-NULL pointers to callback methods */
-   SCIP_CALL( SCIPsetHeurFree(scip, heur, heurFreeGcgzirounding) );
-   SCIP_CALL( SCIPsetHeurInit(scip, heur, heurInitGcgzirounding) );
-   SCIP_CALL( SCIPsetHeurExit(scip, heur, heurExitGcgzirounding) );
-   SCIP_CALL( SCIPsetHeurInitsol(scip, heur, heurInitsolGcgzirounding) );
+   SCIP_CALL( SCIPsetHeurFree(origprob, heur, heurFreeGcgzirounding) );
+   SCIP_CALL( SCIPsetHeurInit(origprob, heur, heurInitGcgzirounding) );
+   SCIP_CALL( SCIPsetHeurExit(origprob, heur, heurExitGcgzirounding) );
+   SCIP_CALL( SCIPsetHeurInitsol(origprob, heur, heurInitsolGcgzirounding) );
 
    /* add GCG zirounding primal heuristic parameters */
-   SCIP_CALL( SCIPaddIntParam(scip, "heuristics/gcgzirounding/maxroundingloops",
+   SCIP_CALL( SCIPaddIntParam(origprob, "heuristics/gcgzirounding/maxroundingloops",
          "determines maximum number of rounding loops",
          &heurdata->maxroundingloops, TRUE, DEFAULT_MAXROUNDINGLOOPS, 0, INT_MAX, NULL, NULL) );
-   SCIP_CALL( SCIPaddBoolParam(scip, "heuristics/gcgzirounding/stopziround",
+   SCIP_CALL( SCIPaddBoolParam(origprob, "heuristics/gcgzirounding/stopziround",
          "flag to determine if Zirounding is deactivated after a certain percentage of unsuccessful calls",
          &heurdata->stopziround, TRUE, DEFAULT_STOPZIROUND, NULL, NULL) );
-   SCIP_CALL( SCIPaddRealParam(scip,"heuristics/gcgzirounding/stoppercentage",
+   SCIP_CALL( SCIPaddRealParam(origprob,"heuristics/gcgzirounding/stoppercentage",
          "if percentage of found solutions falls below this parameter, Zirounding will be deactivated",
          &heurdata->stoppercentage, TRUE, DEFAULT_STOPPERCENTAGE, 0.0, 1.0, NULL, NULL) );
-   SCIP_CALL( SCIPaddIntParam(scip, "heuristics/gcgzirounding/minstopncalls",
+   SCIP_CALL( SCIPaddIntParam(origprob, "heuristics/gcgzirounding/minstopncalls",
          "determines the minimum number of calls before percentage-based deactivation of"
          " Zirounding is applied", &heurdata->minstopncalls, TRUE, DEFAULT_MINSTOPNCALLS, 1, INT_MAX, NULL, NULL) );
 
