@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -36,12 +37,13 @@
 #define GCG_OBJPRICER_GCG_H_
 
 #include "objscip/objscip.h"
-#include "class_pricingtype.h"
-#include "class_pricingcontroller.h"
-#include "class_stabilization.h"
-#include "pub_gcgcol.h"
-#include "pub_colpool.h"
-#include "pricestore_gcg.h"
+#include "gcg/class_pricingtype.h"
+#include "gcg/class_pricingcontroller.h"
+#include "gcg/class_stabilization.h"
+#include "gcg/pub_gcgcol.h"
+#include "gcg/pub_colpool.h"
+#include "gcg/pricestore_gcg.h"
+#include "gcg/type_branchgcg.h"
 
 using gcg::Pricingcontroller;
 using gcg::Stabilization;
@@ -55,22 +57,21 @@ class ObjPricerGcg : public scip::ObjPricer
 public:
    /*lint --e{1540}*/
 
+   GCG*                   gcg;                /**< GCG data structure */
    SCIP*                  origprob;           /**< the original program */
    SCIP_PRICERDATA*       pricerdata;         /**< pricerdata data structure */
    GCG_COLPOOL*           colpool;            /**< column pool */
    GCG_PRICESTORE*        pricestore;         /**< price storage */
-   static int             threads;
 
    /** default constructor */
    ObjPricerGcg(
-         SCIP* scip, /**< SCIP data structure */
-         SCIP*              origscip,           /**< SCIP data structure of original problem */
-         const char* name, /**< name of variable pricer */
-         const char* desc, /**< description of variable pricer */
-         int priority, /**< priority of the variable pricer */
-         unsigned int delay, /**< should the pricer be delayed until no other pricers or already existing*/
-         SCIP_PRICERDATA *pricerdata /**< pricerdata data structure */
-   );
+      GCG* gcgstruct, /**< GCG data structure */
+      const char* name, /**< name of variable pricer */
+      const char* desc, /**< description of variable pricer */
+      int priority, /**< priority of the variable pricer */
+      unsigned int delay, /**< should the pricer be delayed until no other pricers or already existing*/
+      SCIP_PRICERDATA* p_pricerdata /**< pricerdata data structure */
+      );
    /** destructor */
    virtual ~ObjPricerGcg()
    {}
@@ -89,7 +90,7 @@ public:
    virtual SCIP_DECL_PRICERREDCOST(scip_redcost);
    /** farkas pricing method of variable pricer for infeasible LPs */
    virtual SCIP_DECL_PRICERFARKAS(scip_farkas);
-   inline SCIP_PRICERDATA *getPricerdata()
+   inline SCIP_PRICERDATA* getPricerdata()
    {
       return pricerdata;
    }
@@ -104,7 +105,7 @@ public:
    SCIP_RETCODE setPricingObjs(
       PricingType*          pricetype,          /**< Farkas or Reduced cost pricing */
       SCIP_Bool             stabilize           /**< do we use stabilization ? */
-   );
+      );
 
    /** update reduced cost of columns in column pool */
    void updateRedcostColumnPool(
@@ -121,11 +122,10 @@ public:
       PricingType*       pricetype,          /**< type of the pricing */
       SCIP_RESULT*       result,             /**< result pointer */
       double*            lowerbound          /**< lower bound of pricingproblems */
-   );
+      );
 
    /** creates a new master variable corresponding to the given solution and problem */
    SCIP_RETCODE createNewMasterVar(
-      SCIP*              scip,               /**< SCIP data structure */
       PricingType*       pricetype,          /**< type of the pricing */
       SCIP_SOL*          sol,                /**< solution to compute reduced cost for */
       SCIP_VAR**         solvars,            /**< array of variables with non-zero value in the solution of the pricing problem */
@@ -136,18 +136,17 @@ public:
       unsigned int       force,              /**< should the given variable be added also if it has non-negative reduced cost? */
       unsigned int*      added,              /**< pointer to store whether the variable was successfully added */
       SCIP_VAR**         addedvar            /**< pointer to store the created variable */
-   );
+      );
 
    /** creates a new master variable corresponding to the given gcg column */
    SCIP_RETCODE createNewMasterVarFromGcgCol(
-      SCIP*                 scip,               /**< SCIP data structure */
       PricingType*          pricetype,          /**< type of pricing */
       GCG_COL*              gcgcol,             /**< GCG column data structure */
       SCIP_Bool             force,              /**< should the given variable be added also if it has non-negative reduced cost? */
       SCIP_Bool*            added,              /**< pointer to store whether the variable was successfully added */
       SCIP_VAR**            addedvar,           /**< pointer to store the created variable */
       SCIP_Real             score               /**< score of column (or -1.0 if not specified) */
-   );
+      );
 
    /* Compute difference of two dual solutions */
    SCIP_RETCODE computeDualDiff(
@@ -156,7 +155,7 @@ public:
       SCIP_Real**          dualvals2,           /**< array of dual values for each pricing problem */
       SCIP_Real*           dualconv2,           /**< array of dual solutions for the convexity constraints  */
       SCIP_Real*           dualdiff             /**< pointer to store difference of duals solutions */
-   );
+      );
 
    /** the pricing loop: solve the pricing problems */
    SCIP_RETCODE pricingLoop(
@@ -165,7 +164,7 @@ public:
       int*           nfoundvars,         /**< pointer to store number of found variables */
       SCIP_Real*     lowerbound,         /**< pointer to store lowerbound obtained due to lagrange bound */
       SCIP_Bool*     bestredcostvalid    /**< pointer to store if bestredcost are valid (pp solvedoptimal) */
-   );
+      );
 
    const FarkasPricing* getFarkasPricing() const
    {
@@ -187,11 +186,16 @@ public:
       return reducedcostpricing;
    }
 
-   SCIP* getOrigprob()
+   SCIP* getOrigprob() const
    {
       return origprob;
    }
 
+   GCG* getGcg() const
+   {
+      return gcg;
+   }
+
    /** get the number of columns to be added to the master LP in the current pricing round */
    int getMaxColsRound() const;
 
@@ -201,9 +205,6 @@ public:
    /** add artificial vars */
    SCIP_RETCODE addArtificialVars();
 
-   /** add trivial sols */
-   SCIP_RETCODE addTrivialsols();
-
    /** create the pointers for the pricing types */
    SCIP_RETCODE createPricingTypes();
 
@@ -219,19 +220,12 @@ public:
    /** create the pointers for the pricestore */
    SCIP_RETCODE createPricestore();
 
-   /** for given columns, (re-)compute and update their reduced costs */
-   void updateRedcosts(
-      PricingType*          pricetype,          /**< type of pricing */
-      GCG_COL**             cols,               /**< columns to compute reduced costs for */
-      int                   ncols,              /**< number of columns */
-      int*                  nimpcols            /**< pointer to store number of improving columns */
-      );
-
-   /** add a new column to the pricing storage */
+   /** add a new column to the pricer's column buffer that will be added to the pricing storage */
    SCIP_RETCODE addColToPricestore(
-      GCG_COL*              col                 /**< priced col */
+      GCG_COL*              col,                /**< priced col */
+      SCIP_Bool*            added               /**< pointer to var that indicates whether the col was added */
       );
-
+   
    /** for each pricing problem, get the best found column from the pricing storage */
    void getBestCols(
       GCG_COL**             pricingprobcols     /**< array to be filled with best column per pricing problem */
@@ -260,11 +254,35 @@ public:
       GCG_COL*              gcgcol              /**< GCG column data structure */
       );
 
-   /** compute master cut coefficients of column */
-   SCIP_RETCODE computeColMastercuts(
+   /** compute original separator master cut coefficients of column */
+   SCIP_RETCODE computeColOriginalSepaCuts(
       GCG_COL*              gcgcol              /**< GCG column data structure */
       );
 
+   /** compute mastersepacut coefficients of column */
+   SCIP_RETCODE computeColMastersepacutCoeffs(
+      GCG_COL*              gcgcol              /**< GCG column data structure */
+      );
+
+   /** add variable with computed coefficients to the original separator cuts */
+   SCIP_RETCODE addVariableToMastersepacutsFromGCGCol(
+      SCIP_VAR*             newvar,             /**< The new variable to add */
+      GCG_COL*              gcgcol              /**< GCG column data structure */
+      );
+
+   /** add variable with computed coefficients to the separator master cuts */
+   SCIP_RETCODE addVariableToMastersepacuts(
+      SCIP_VAR*             newvar,             /**< The new variable to add */
+      int                   prob,               /**< number of the pricing problem the solution belongs to */
+      SCIP_VAR**            solvars,            /**< array of variables with non-zero value in the solution of the pricing problem */
+      SCIP_Real*            solvals,            /**< array of values in the solution of the pricing problem for variables in array solvars*/
+      int                   nsolvars            /**< number of variables in array solvars */
+      );
+
+   GCG_SOLVER** getSolvers() const;
+
+   int getNumSolvers() const;
+
 private:
    ReducedCostPricing*    reducedcostpricing;
    FarkasPricing*         farkaspricing;
@@ -281,7 +299,7 @@ private:
       SCIP_Bool             solisray,           /**< is the solution a ray? */
       int                   prob,               /**< number of the pricing problem the solution belongs to */
       SCIP_Real*            objvalptr           /**< pointer to store the computed objective value */
-   ) const;
+      ) const;
 
    /** for given columns, (re-)compute and update their reduced costs */
    void updateRedcosts(
@@ -296,23 +314,23 @@ private:
    /** ensures size of pricedvars array */
    SCIP_RETCODE ensureSizePricedvars(
       int                   size                /**< needed size */
-   );
+      );
 
    /** adds new variable to the end of the priced variables array */
    SCIP_RETCODE addVariableToPricedvars(
       SCIP_VAR*             newvar              /**< variable to add */
-   );
+      );
 
    /** ensures size of root bounds arrays */
    SCIP_RETCODE ensureSizeRootBounds(
       int                   size                /**< needed size */
-   );
+      );
 
    /** adds new bounds to the bound arrays as well as some additional information on dual variables and root lp solution */
    SCIP_RETCODE addRootBounds(
       SCIP_Real             primalbound,        /**< new primal bound for the root master LP */
       SCIP_Real             dualbound           /**< new dual bound for the root master LP */
-   );
+      );
 
    /** add master variable to all constraints */
    SCIP_RETCODE addVariableToMasterconstraints(
@@ -321,7 +339,7 @@ private:
       SCIP_VAR**            solvars,            /**< array of variables with non-zero value in the solution of the pricing problem */
       SCIP_Real*            solvals,            /**< array of values in the solution of the pricing problem for variables in array solvars*/
       int                   nsolvars            /**< number of variables in array solvars */
-   );
+      );
 
    /** add master variable to all constraints */
    SCIP_RETCODE addVariableToMasterconstraintsFromGCGCol(
@@ -329,8 +347,8 @@ private:
       GCG_COL*              gcgcol              /**< GCG column data structure */
       );
 
-   /** add variable with computed coefficients to the master cuts */
-   SCIP_RETCODE addVariableToMastercuts(
+   /** add variable with computed coefficients to the original separator master cuts */
+   SCIP_RETCODE addVariableToOriginalSepaCuts(
       SCIP_VAR*             newvar,             /**< The new variable to add */
       int                   prob,               /**< number of the pricing problem the solution belongs to */
       SCIP_VAR**            solvars,            /**< array of variables with non-zero value in the solution of the pricing problem */
@@ -338,12 +356,17 @@ private:
       int                   nsolvars            /**< number of variables in array solvars */
       );
 
-   /** add variable with computed coefficients to the master cuts */
-   SCIP_RETCODE addVariableToMastercutsFromGCGCol(
+   /** add variable with computed coefficients to the original separator master cuts */
+   SCIP_RETCODE addVariableToOriginalSepaCutsFromGCGCol(
       SCIP_VAR*             newvar,             /**< The new variable to add */
       GCG_COL*              gcgcol              /**< GCG column data structure */
       );
 
+   /** add variable to the extended master conss */
+   SCIP_RETCODE addVariableToExtendedmastercons(
+      SCIP_VAR*             newvar              /**< The new variable to add */
+      );
+
    /**
     * check whether pricing can be aborted:
     * if objective value is always integral and the current node's current
@@ -359,12 +382,12 @@ private:
    /** returns the gegeneracy of the masterproblem */
    SCIP_RETCODE computeCurrentDegeneracy(
       double*               degeneracy          /**< pointer to store degeneracy */
-   );
+      );
 
    /** set subproblem memory limit */
    SCIP_RETCODE setPricingProblemMemorylimit(
       SCIP*                 pricingscip         /**< SCIP of the pricingproblem */
-   );
+      );
 
 #ifdef SCIP_DISABLED_CODE
    /** generic method to generate feasible columns from the pricing problem
@@ -412,16 +435,33 @@ private:
    /** calls the exitsol method of all solvers */
    SCIP_RETCODE solversExitsol();
 
-   /** computes the stack of masterbranch constraints up to the last generic branching node
-    * @note This method has to be threadsafe!
-    */
-   SCIP_RETCODE computeGenericBranchingconssStack(
-      PricingType*          pricetype,          /**< type of pricing: reduced cost or Farkas */
-      int                   prob,               /**< index of pricing problem */
-      SCIP_CONS***          consstack,          /**< stack of branching constraints */
-      int*                  nconsstack,         /**< size of the stack */
-      SCIP_Real**           consduals           /**< dual values of the masterbranch solutions, or NULL */
-   ) const;
+   /** frees all pricing callback plugins */
+   SCIP_RETCODE pricingcbsFree();
+
+   /** calls the init method on all pricing callback plugins */
+   SCIP_RETCODE pricingcbsInit();
+
+   /** calls the exit method on all pricing callback plugins */
+   SCIP_RETCODE pricingcbsExit();
+
+   /** calls the initsol method on all pricing callback plugins */
+   SCIP_RETCODE pricingcbsInitsol();
+
+   /** calls the exitsol method of all pricing callback plugins */
+   SCIP_RETCODE pricingcbsExitsol();
+
+   /** calls pre-pricing method of the pricing callback plugins */
+   SCIP_RETCODE pricingcbsPrepricing(
+      SCIP_PRICER*          pricer,             /**< the pointer to the calling pricer */
+      GCG_PRICETYPE         type,               /**< the type of pricing, either redcost or farkas */
+      SCIP_Bool*            abort               /**< flag to set whether the pricing should be aborted */
+      );
+
+   /** calls post-pricing method of the pricing callback plugins */
+   SCIP_RETCODE pricingcbsPostpricing(
+      SCIP_PRICER*          pricer,             /**< the pointer to the calling pricer */
+      GCG_PRICETYPE         type                /**< the type of pricing, either redcost or farkas */
+      );
 
    /** add bounds change from constraint from the pricing problem at this node
     * @note This method has to be threadsafe!
@@ -429,14 +469,14 @@ private:
    SCIP_RETCODE addBranchingBoundChangesToPricing(
       int                   prob,               /**< index of pricing problem */
       SCIP_CONS*            branchcons          /**< branching constraints from which bound should applied */
-   ) const;
+      ) const;
 
    SCIP_RETCODE checkBranchingBoundChanges(
       int                   prob,               /**< index of pricing problem */
       SCIP_SOL*             sol,                /**< solution to check */
       SCIP_CONS*            branchcons,         /**< branching constraints from which bound should applied */
       SCIP_Bool*            feasible            /**< check whether the solution is feasible */
-   ) const;
+      ) const;
 
    /** check bounds change from constraint from the pricing problem at this node
     * @note This method has to be threadsafe!
@@ -445,11 +485,17 @@ private:
       GCG_COL*              gcgcol,             /**< gcg column to check */
       SCIP_CONS*            branchcons,         /**< branching constraints from which bound should applied */
       SCIP_Bool*            feasible            /**< check whether the solution is feasible */
-   ) const;
+      ) const;
 
    SCIP_RETCODE ensureSizeArtificialvars(
       int                   size                /**< needed size */
-   );
+      );
+
+   /** activates all pricing modifications */
+   SCIP_RETCODE activatePricingModifications(
+      GCG_EXTENDEDMASTERCONSDATA*      extendedmasterconsdata,       /**< extended master cons data */
+      SCIP_Bool                        apply                         /**< should the modifications actually be activated */
+      );
 
 };
 
