@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -37,17 +38,13 @@
 #include <stdlib.h>
 #include <assert.h>
 #include <string.h>
-#if defined(_WIN32) || defined(_WIN64)
-#else
-#include <strings.h> /*lint --e{766}*/ /* needed for strcasecmp() */
-#endif
 #include <ctype.h>
 
-#include "reader_ref.h"
-#include "gcg.h"
+#include "gcg/reader_ref.h"
+#include "gcg/gcg.h"
 #include "scip/cons_linear.h"
-#include "cons_decomp.h"
-#include "relax_gcg.h"
+#include "gcg/cons_decomp.h"
+#include "gcg/relax_gcg.h"
 
 #define READER_NAME             "refreader"
 #define READER_DESC             "file reader for blocks corresponding to a mip in lpb format"
@@ -99,6 +96,11 @@ struct RefInput
 };
 typedef struct RefInput REFINPUT;
 
+struct SCIP_ReaderData
+{
+   GCG*                  gcg;                /**< GCG data structure */
+};
+
 static const char delimchars[] = " \f\n\r\t\v";
 static const char tokenchars[] = "-+:<>=";
 static const char commentchars[] = "\\";
@@ -374,7 +376,7 @@ SCIP_Bool isInt(
 
    assert(refinput != NULL);
    assert(value != NULL);
-   assert(!(strcasecmp(refinput->token, "INFINITY") == 0) && !(strcasecmp(refinput->token, "INF") == 0));
+   assert(!(SCIPstrcasecmp(refinput->token, "INFINITY") == 0) && !(SCIPstrcasecmp(refinput->token, "INF") == 0));
 
    val = strtol(refinput->token, &endptr, 0);
    if( endptr != refinput->token && *endptr == '\0' )
@@ -564,12 +566,13 @@ SCIP_RETCODE readREFFile(
    const char*           filename            /**< name of the input file */
    )
 {
-
+   SCIP_READERDATA* readerdata;
    assert(scip != NULL);
    assert(reader != NULL);
    assert(refinput != NULL);
    assert(filename != NULL);
 
+   readerdata = SCIPreaderGetData(reader);
    if( SCIPgetStage(scip) < SCIP_STAGE_TRANSFORMED )
       SCIP_CALL( SCIPtransformProb(scip) );
 
@@ -622,7 +625,7 @@ SCIP_RETCODE readREFFile(
    SCIPfclose(refinput->file);
 
    /* copy information to decomp */
-   SCIP_CALL_QUIET( GCGfilloutDecompFromHashmaps(scip, decomp, refinput->vartoblock, refinput->constoblock, refinput->nblocks, FALSE) );
+   SCIP_CALL_QUIET( GCGfilloutDecompFromHashmaps(readerdata->gcg, decomp, refinput->vartoblock, refinput->constoblock, refinput->nblocks, FALSE) );
 
    GCGdecompSetPresolved(decomp, FALSE);
    GCGdecompSetDetector(decomp, NULL);
@@ -641,7 +644,7 @@ SCIP_RETCODE writeREFFile(
 {
    SCIP_HASHMAP *cons2origindex;
    GCG_DECOMP* decomp;
-
+   SCIP_READERDATA* readerdata;
    SCIP_CONS** conss;
    int nconss;
    SCIP_CONS*** subscipconss;
@@ -653,11 +656,13 @@ SCIP_RETCODE writeREFFile(
    assert(reader != NULL);
    assert(file != NULL);
 
-   decomp = GCGgetBestDecomp(scip, TRUE);
+   readerdata = SCIPreaderGetData(reader);
+   assert(readerdata != NULL);
+   decomp = GCGgetBestDecomp(readerdata->gcg, TRUE);
 
    if( decomp == NULL )
    {
-      decomp = GCGgetStructDecomp(scip);
+      decomp = GCGgetStructDecomp(readerdata->gcg);
    }
 
    if( decomp == NULL )
@@ -717,65 +722,14 @@ SCIP_RETCODE writeREFFile(
    }
    SCIPhashmapFree(&cons2origindex);
 
-   GCGdecompFree(scip, &decomp);
+   GCGdecompFree(readerdata->gcg, &decomp);
 
    return SCIP_OKAY;
 }
 
-
-/*
- * Callback methods of reader
- */
-
-/** destructor of reader to free user data (called when SCIP is exiting) */
-#define readerFreeRef NULL
-
-/** problem reading method of reader */
-static
-SCIP_DECL_READERREAD(readerReadRef)
-{
-   if( SCIPgetStage(scip) == SCIP_STAGE_INIT || SCIPgetNVars(scip) == 0 || SCIPgetNConss(scip) == 0 )
-   {
-      SCIPverbMessage(scip, SCIP_VERBLEVEL_DIALOG, NULL, "No problem exists, will not detect structure!\n");
-      return SCIP_OKAY;
-   }
-
-   SCIP_CALL( SCIPreadRef(scip, reader, filename, result) );
-
-   return SCIP_OKAY;
-}
-
-
-/** problem writing method of reader */
-static
-SCIP_DECL_READERWRITE(readerWriteRef)
-{
-   /*lint --e{715}*/
-   SCIP_CALL( writeREFFile(scip, reader, file) );
-   *result = SCIP_SUCCESS;
-   return SCIP_OKAY;
-}
-
-/*
- * reader specific interface methods
- */
-
-/** includes the ref file reader in SCIP */
-SCIP_RETCODE SCIPincludeReaderRef(
-   SCIP*                 scip                /**< SCIP data structure */
-   )
-{
-   assert(scip != NULL);
-   /* include lp reader */
-   SCIP_CALL( SCIPincludeReader(scip, READER_NAME, READER_DESC, READER_EXTENSION,
-         NULL, readerFreeRef, readerReadRef, readerWriteRef, NULL) );
-
-   return SCIP_OKAY;
-}
-
-
 /** reads problem from file */
-SCIP_RETCODE SCIPreadRef(
+static
+SCIP_RETCODE readRef(
    SCIP*                 scip,               /**< SCIP data structure */
    SCIP_READER*          reader,             /**< the file reader itself */
    const char*           filename,           /**< full path and name of file to read, or NULL if stdin should be used */
@@ -786,11 +740,15 @@ SCIP_RETCODE SCIPreadRef(
    REFINPUT refinput;
    GCG_DECOMP* decomp;
    int i;
+   SCIP_READERDATA* readerdata;
 #ifdef SCIP_DEBUG
    SCIP_VAR** vars;
    int nvars;
 #endif
 
+   readerdata = SCIPreaderGetData(reader);
+   assert(readerdata != NULL);
+
    /* initialize REF input data */
    refinput.file = NULL;
    refinput.linebuf[0] = '\0';
@@ -819,13 +777,13 @@ SCIP_RETCODE SCIPreadRef(
    SCIP_CALL( SCIPhashmapCreate(&refinput.constoblock, SCIPblkmem(scip), SCIPgetNConss(scip)) );
 
    /* read the file */
-   SCIP_CALL( GCGdecompCreate(scip, &decomp) );
+   SCIP_CALL( GCGdecompCreate(readerdata->gcg, &decomp) );
 
    retcode = readREFFile(scip, reader, &refinput, decomp, filename);
 
    if( retcode == SCIP_OKAY )
    {
-      SCIP_CALL( GCGconshdlrDecompAddPreexistingDecomp(scip, decomp) );
+      SCIP_CALL( GCGconshdlrDecompAddPreexistingDecomp(readerdata->gcg, decomp) );
       SCIPdebugMessage("Read %d/%d conss in ref-file\n", refinput.totalreadconss, refinput.totalconss);
       SCIPdebugMessage("Assigned %d variables to %d blocks.\n", refinput.nassignedvars, refinput.nblocks);
 #ifdef SCIP_DEBUG
@@ -841,7 +799,7 @@ SCIP_RETCODE SCIPreadRef(
 #endif
    }
 
-   SCIP_CALL( GCGdecompFree(scip, &decomp) );
+   SCIP_CALL( GCGdecompFree(readerdata->gcg, &decomp) );
 
    /* free dynamically allocated memory */
    SCIPfreeMemoryArray(scip, &refinput.token);
@@ -863,3 +821,70 @@ SCIP_RETCODE SCIPreadRef(
 
    return retcode;
 }
+
+
+/*
+ * Callback methods of reader
+ */
+
+/** destructor of reader to free user data (called when SCIP is exiting) */
+static
+SCIP_DECL_READERFREE(readerFreeRef)
+{
+   SCIP_READERDATA* readerdata;
+   
+   assert(scip != NULL);
+   assert(reader != NULL);
+   readerdata = SCIPreaderGetData(reader);
+   SCIPfreeMemory(scip, &readerdata);
+   return SCIP_OKAY;
+}
+
+/** problem reading method of reader */
+static
+SCIP_DECL_READERREAD(readerReadRef)
+{
+   if( SCIPgetStage(scip) == SCIP_STAGE_INIT || SCIPgetNVars(scip) == 0 || SCIPgetNConss(scip) == 0 )
+   {
+      SCIPverbMessage(scip, SCIP_VERBLEVEL_DIALOG, NULL, "No problem exists, will not detect structure!\n");
+      return SCIP_OKAY;
+   }
+
+   SCIP_CALL( readRef(scip, reader, filename, result) );
+
+   return SCIP_OKAY;
+}
+
+
+/** problem writing method of reader */
+static
+SCIP_DECL_READERWRITE(readerWriteRef)
+{
+   /*lint --e{715}*/
+   SCIP_CALL( writeREFFile(scip, reader, file) );
+   *result = SCIP_SUCCESS;
+   return SCIP_OKAY;
+}
+
+/*
+ * reader specific interface methods
+ */
+
+/** includes the ref file reader in SCIP */
+SCIP_RETCODE GCGincludeReaderRef(
+   GCG*                  gcg                 /**< GCG data structure */
+   )
+{
+   SCIP_READERDATA* readerdata;
+   SCIP* origprob = GCGgetOrigprob(gcg);
+   assert(origprob != NULL);
+   
+   SCIP_CALL( SCIPallocMemory(origprob, &readerdata) );
+   readerdata->gcg = gcg;
+   
+   /* include reader */
+   SCIP_CALL( SCIPincludeReader(origprob, READER_NAME, READER_DESC, READER_EXTENSION,
+         NULL, readerFreeRef, readerReadRef, readerWriteRef, readerdata) );
+
+   return SCIP_OKAY;
+}
