@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -33,15 +34,19 @@
 
 /*---+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8----+----9----+----0----+----1----+----2*/
 
-#include "pub_solver.h"
-#include "solver.h"
-#include "struct_solver.h"
+#include "gcg/pub_solver.h"
+#include "gcg/solver.h"
+#include "gcg/struct_solver.h"
 
-#include "gcg.h"
-#include "pricer_gcg.h"
+#include "gcg/gcg.h"
+#include "gcg/pricer_gcg.h"
 
 #include <string.h>
 
+#ifdef _OPENMP
+#include <omp.h>
+#endif
+
 
 /** compares two solvers w. r. t. their priorities */
 SCIP_DECL_SORTPTRCOMP(GCGsolverComp)
@@ -57,7 +62,7 @@ SCIP_DECL_SORTPTRCOMP(GCGsolverComp)
 
 /** creates a GCG pricing solver */
 SCIP_RETCODE GCGsolverCreate(
-   SCIP*                 scip,               /**< SCIP data structure (master problem) */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_SOLVER**          solver,             /**< pointer to pricing solver data structure */
    const char*           name,               /**< name of solver */
    const char*           desc,               /**< description of solver */
@@ -77,17 +82,20 @@ SCIP_RETCODE GCGsolverCreate(
 {
    char paramname[SCIP_MAXSTRLEN];
    char paramdesc[SCIP_MAXSTRLEN];
+   SCIP* masterprob;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
+   masterprob = GCGgetMasterprob(gcg);
+
    if( solveheur == NULL && solversolve == NULL )
    {
-      SCIPwarningMessage(scip, "Solver <%s> has neither heuristic nor exact solving method and will not be included.\n", name);
+      SCIPwarningMessage(masterprob, "Solver <%s> has neither heuristic nor exact solving method and will not be included.\n", name);
       return SCIP_OKAY;
    }
 
-   SCIP_CALL( SCIPallocMemory(scip, solver) ); /*lint !e866*/
+   SCIP_CALL( SCIPallocMemory(masterprob, solver) ); /*lint !e866*/
 
    SCIP_ALLOC( BMSduplicateMemoryArray(&(*solver)->name, name, strlen(name)+1) );
    SCIP_ALLOC( BMSduplicateMemoryArray(&(*solver)->desc, desc, strlen(desc)+1) );
@@ -102,10 +110,10 @@ SCIP_RETCODE GCGsolverCreate(
    (*solver)->solverexitsol = solverexitsol;
    (*solver)->solverdata = solverdata;
 
-   SCIP_CALL( SCIPcreateCPUClock(scip, &((*solver)->optfarkasclock)) );
-   SCIP_CALL( SCIPcreateCPUClock(scip, &((*solver)->optredcostclock)) );
-   SCIP_CALL( SCIPcreateCPUClock(scip, &((*solver)->heurfarkasclock)) );
-   SCIP_CALL( SCIPcreateCPUClock(scip, &((*solver)->heurredcostclock)) );
+   SCIP_CALL( SCIPcreateCPUClock(masterprob, &((*solver)->optfarkasclock)) );
+   SCIP_CALL( SCIPcreateCPUClock(masterprob, &((*solver)->optredcostclock)) );
+   SCIP_CALL( SCIPcreateCPUClock(masterprob, &((*solver)->heurfarkasclock)) );
+   SCIP_CALL( SCIPcreateCPUClock(masterprob, &((*solver)->heurredcostclock)) );
 
    (*solver)->optfarkascalls = 0;
    (*solver)->optredcostcalls = 0;
@@ -116,7 +124,7 @@ SCIP_RETCODE GCGsolverCreate(
    {
       (void) SCIPsnprintf(paramname, SCIP_MAXSTRLEN, "pricingsolver/%s/heurenabled", name);
       (void) SCIPsnprintf(paramdesc, SCIP_MAXSTRLEN, "flag to indicate whether heuristic solving method of solver <%s> is enabled", name);
-      SCIP_CALL( SCIPaddBoolParam(GCGmasterGetOrigprob(scip), paramname, paramdesc,
+      SCIP_CALL( SCIPaddBoolParam(GCGgetOrigprob(gcg), paramname, paramdesc,
                      &((*solver)->heurenabled), FALSE, heurenabled, NULL, NULL));
    }
    else
@@ -126,7 +134,7 @@ SCIP_RETCODE GCGsolverCreate(
    {
       (void) SCIPsnprintf(paramname, SCIP_MAXSTRLEN, "pricingsolver/%s/exactenabled", name);
       (void) SCIPsnprintf(paramdesc, SCIP_MAXSTRLEN, "flag to indicate whether exact solving method of solver <%s> is enabled", name);
-      SCIP_CALL( SCIPaddBoolParam(GCGmasterGetOrigprob(scip), paramname, paramdesc,
+      SCIP_CALL( SCIPaddBoolParam(GCGgetOrigprob(gcg), paramname, paramdesc,
                      &((*solver)->exactenabled), FALSE, exactenabled, NULL, NULL));
    }
    else
@@ -134,7 +142,7 @@ SCIP_RETCODE GCGsolverCreate(
 
    (void) SCIPsnprintf(paramname, SCIP_MAXSTRLEN, "pricingsolver/%s/priority", name);
    (void) SCIPsnprintf(paramdesc, SCIP_MAXSTRLEN, "priority of solver <%s>", name);
-   SCIP_CALL( SCIPaddIntParam(GCGmasterGetOrigprob(scip), paramname, paramdesc,
+   SCIP_CALL( SCIPaddIntParam(GCGgetOrigprob(gcg), paramname, paramdesc,
                   &((*solver)->priority), FALSE, priority, INT_MIN/4, INT_MAX/4, NULL, NULL));
 
    return SCIP_OKAY;
@@ -142,51 +150,55 @@ SCIP_RETCODE GCGsolverCreate(
 
 /** calls destructor and frees memory of GCG pricing solver */
 SCIP_RETCODE GCGsolverFree(
-   SCIP*                 scip,               /**< SCIP data structure (master problem) */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_SOLVER**          solver              /**< pointer to pricing solver data structure */
    )
 {
-   assert(scip != NULL);
+   SCIP* masterprob;
+   assert(gcg != NULL);
    assert(solver != NULL);
    assert(*solver != NULL);
 
+   masterprob = GCGgetMasterprob(gcg);
+
    if( (*solver)->solverfree != NULL )
    {
-      SCIP_CALL( (*solver)->solverfree(scip, *solver) );
+      SCIP_CALL( (*solver)->solverfree(gcg, *solver) );
    }
 
    BMSfreeMemoryArray(&(*solver)->name);
    BMSfreeMemoryArray(&(*solver)->desc);
 
-   SCIP_CALL( SCIPfreeClock(scip, &((*solver)->optfarkasclock)) );
-   SCIP_CALL( SCIPfreeClock(scip, &((*solver)->optredcostclock)) );
-   SCIP_CALL( SCIPfreeClock(scip, &((*solver)->heurfarkasclock)) );
-   SCIP_CALL( SCIPfreeClock(scip, &((*solver)->heurredcostclock)) );
+   SCIP_CALL( SCIPfreeClock(masterprob, &((*solver)->optfarkasclock)) );
+   SCIP_CALL( SCIPfreeClock(masterprob, &((*solver)->optredcostclock)) );
+   SCIP_CALL( SCIPfreeClock(masterprob, &((*solver)->heurfarkasclock)) );
+   SCIP_CALL( SCIPfreeClock(masterprob, &((*solver)->heurredcostclock)) );
 
-   SCIPfreeMemory(scip, solver);
+   SCIPfreeMemory(masterprob, solver);
 
    return SCIP_OKAY;
 }
 
 /** initializes GCG pricing solver */
 SCIP_RETCODE GCGsolverInit(
-   SCIP*                 scip,               /**< SCIP data structure (master problem) */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_SOLVER*           solver              /**< pricing solver */
    )
 {
    SCIP_Bool resetstat;
+   SCIP* masterprob = GCGgetMasterprob(gcg);
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
-   SCIP_CALL( SCIPgetBoolParam(scip, "misc/resetstat", &resetstat) );
+   SCIP_CALL( SCIPgetBoolParam(masterprob, "misc/resetstat", &resetstat) );
 
    if( resetstat )
    {
-      SCIP_CALL( SCIPresetClock(scip, solver->optfarkasclock) );
-      SCIP_CALL( SCIPresetClock(scip, solver->optredcostclock) );
-      SCIP_CALL( SCIPresetClock(scip, solver->heurfarkasclock) );
-      SCIP_CALL( SCIPresetClock(scip, solver->heurredcostclock) );
+      SCIP_CALL( SCIPresetClock(masterprob, solver->optfarkasclock) );
+      SCIP_CALL( SCIPresetClock(masterprob, solver->optredcostclock) );
+      SCIP_CALL( SCIPresetClock(masterprob, solver->heurfarkasclock) );
+      SCIP_CALL( SCIPresetClock(masterprob, solver->heurredcostclock) );
 
       solver->optfarkascalls = 0;
       solver->optredcostcalls = 0;
@@ -196,7 +208,7 @@ SCIP_RETCODE GCGsolverInit(
 
    if( solver->solverinit != NULL )
    {
-      SCIP_CALL( solver->solverinit(scip, solver) );
+      SCIP_CALL( solver->solverinit(gcg, solver) );
    }
 
    return SCIP_OKAY;
@@ -204,16 +216,16 @@ SCIP_RETCODE GCGsolverInit(
 
 /** calls exit method of GCG pricing solver */
 SCIP_RETCODE GCGsolverExit(
-   SCIP*                 scip,               /**< SCIP data structure (master problem) */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_SOLVER*           solver              /**< pricing solver */
    )
 {
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
    if( solver->solverexit != NULL )
    {
-      SCIP_CALL( solver->solverexit(scip, solver) );
+      SCIP_CALL( solver->solverexit(gcg, solver) );
    }
 
    return SCIP_OKAY;
@@ -221,16 +233,16 @@ SCIP_RETCODE GCGsolverExit(
 
 /** calls solving process initialization method of GCG pricing solver */
 SCIP_RETCODE GCGsolverInitsol(
-   SCIP*                 scip,               /**< SCIP data structure (master problem) */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_SOLVER*           solver              /**< pricing solver */
    )
 {
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
    if( solver->solverinitsol != NULL )
    {
-      SCIP_CALL( solver->solverinitsol(scip, solver) );
+      SCIP_CALL( solver->solverinitsol(gcg, solver) );
    }
 
    return SCIP_OKAY;
@@ -238,16 +250,16 @@ SCIP_RETCODE GCGsolverInitsol(
 
 /** calls solving process deinitialization method of GCG pricing solver */
 SCIP_RETCODE GCGsolverExitsol(
-   SCIP*                 scip,               /**< SCIP data structure (master problem) */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_SOLVER*           solver              /**< pricing solver */
    )
 {
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
    if( solver->solverexitsol != NULL )
    {
-      SCIP_CALL( solver->solverexitsol(scip, solver) );
+      SCIP_CALL( solver->solverexitsol(gcg, solver) );
    }
 
    return SCIP_OKAY;
@@ -255,6 +267,7 @@ SCIP_RETCODE GCGsolverExitsol(
 
 /** calls update method of GCG pricing solver */
 SCIP_RETCODE GCGsolverUpdate(
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP*                 pricingprob,        /**< the pricing problem that should be solved */
    GCG_SOLVER*           solver,             /**< pricing solver */
    int                   probnr,             /**< number of the pricing problem */
@@ -266,9 +279,9 @@ SCIP_RETCODE GCGsolverUpdate(
    assert(pricingprob != NULL);
    assert(solver != NULL);
 
-   if( solver->solverupdate != NULL )
+   if( solver->solverupdate != NULL && (varobjschanged || varbndschanged || consschanged) )
    {
-      SCIP_CALL( solver->solverupdate(pricingprob, solver, probnr, varobjschanged, varbndschanged, consschanged) );
+      SCIP_CALL( solver->solverupdate(gcg, pricingprob, solver, probnr, varobjschanged, varbndschanged, consschanged) );
    }
 
    return SCIP_OKAY;
@@ -278,7 +291,7 @@ SCIP_RETCODE GCGsolverUpdate(
  * @note This method has to be threadsafe!
  */
 SCIP_RETCODE GCGsolverSolve(
-   SCIP*                 scip,               /**< SCIP data structure (master problem) */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP*                 pricingprob,        /**< the pricing problem that should be solved */
    GCG_SOLVER*           solver,             /**< pricing solver */
    SCIP_Bool             redcost,            /**< is reduced cost (TRUE) or Farkas (FALSE) pricing performed? */
@@ -291,13 +304,16 @@ SCIP_RETCODE GCGsolverSolve(
    )
 {
    SCIP_CLOCK* clock;
+   SCIP* masterprob;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(pricingprob != NULL);
    assert(solver != NULL);
    assert(lowerbound != NULL);
    assert(status != NULL);
 
+   masterprob = GCGgetMasterprob(gcg);
+
    *solved = FALSE;
 
    if( heuristic )
@@ -311,18 +327,22 @@ SCIP_RETCODE GCGsolverSolve(
          else
             clock = solver->heurfarkasclock;
 
-         #pragma omp critical (clock)
-         {
-            SCIP_CALL_ABORT( SCIPstartClock(scip, clock) );
-         }
+#ifdef _OPENMP
+         if( omp_get_num_threads() == 1 )
+            SCIP_CALL_ABORT( SCIPstartClock(masterprob, clock) );
+#else
+         SCIP_CALL_ABORT( SCIPstartClock(masterprob, clock) );
+#endif
 
-         SCIP_CALL( solver->solversolveheur(scip, pricingprob, solver, probnr, dualsolconv, lowerbound, status) );
+         SCIP_CALL( solver->solversolveheur(gcg, pricingprob, solver, probnr, dualsolconv, lowerbound, status) );
          *solved = TRUE;
 
-         #pragma omp critical (clock)
-         {
-            SCIP_CALL_ABORT( SCIPstopClock(scip, clock) );
-         }
+#ifdef _OPENMP
+         if( omp_get_num_threads() == 1 )
+            SCIP_CALL_ABORT( SCIPstopClock(masterprob, clock) );
+#else
+         SCIP_CALL_ABORT( SCIPstopClock(masterprob, clock) );
+#endif
       }
    }
    else
@@ -336,19 +356,22 @@ SCIP_RETCODE GCGsolverSolve(
          else
             clock = solver->optfarkasclock;
 
-         #pragma omp critical (clock)
-         {
-            SCIP_CALL_ABORT( SCIPstartClock(scip, clock) );
-         }
+#ifdef _OPENMP
+         if( omp_get_num_threads() == 1 )
+            SCIP_CALL_ABORT( SCIPstartClock(masterprob, clock) );
+#else
+         SCIP_CALL_ABORT( SCIPstartClock(masterprob, clock) );
+#endif
 
-         SCIP_CALL( solver->solversolve(scip, pricingprob, solver, probnr, dualsolconv, lowerbound, status) );
+         SCIP_CALL( solver->solversolve(gcg, pricingprob, solver, probnr, dualsolconv, lowerbound, status) );
          *solved = TRUE;
 
-         #pragma omp critical (clock)
-         {
-            SCIP_CALL_ABORT( SCIPstopClock(scip, clock) );
-         }
-
+#ifdef _OPENMP
+         if( omp_get_num_threads() == 1 )
+            SCIP_CALL_ABORT( SCIPstopClock(masterprob, clock) );
+#else
+          SCIP_CALL_ABORT( SCIPstopClock(masterprob, clock) );
+#endif
       }
    }
 
@@ -356,17 +379,17 @@ SCIP_RETCODE GCGsolverSolve(
    {
       if( redcost )
          if( heuristic )
-            #pragma omp atomic
+            #pragma omp atomic update
             ++solver->heurredcostcalls;
          else
-            #pragma omp atomic
+            #pragma omp atomic update
             ++solver->optredcostcalls;
       else
          if( heuristic )
-            #pragma omp atomic
+            #pragma omp atomic update
             ++solver->heurfarkascalls;
          else
-            #pragma omp atomic
+            #pragma omp atomic update
             ++solver->optfarkascalls;
    }
 
@@ -486,48 +509,48 @@ int GCGsolverGetHeurRedcostCalls(
 
 /** gets exact Farkas pricing time of pricing solver */
 SCIP_Real GCGsolverGetOptFarkasTime(
-   SCIP*                 scip,               /**< SCIP data structure (master problem) */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_SOLVER*           solver              /**< pricing solver */
    )
 {
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
-   return SCIPgetClockTime(scip, solver->optfarkasclock);
+   return SCIPgetClockTime(GCGgetMasterprob(gcg), solver->optfarkasclock);
 }
 
 /** gets exact reduced cost pricing time of pricing solver */
 SCIP_Real GCGsolverGetOptRedcostTime(
-   SCIP*                 scip,               /**< SCIP data structure (master problem) */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_SOLVER*           solver              /**< pricing solver */
    )
 {
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
-   return SCIPgetClockTime(scip, solver->optredcostclock);
+   return SCIPgetClockTime(GCGgetMasterprob(gcg), solver->optredcostclock);
 }
 
 /** gets heuristic Farkas pricing time of pricing solver */
 SCIP_Real GCGsolverGetHeurFarkasTime(
-   SCIP*                 scip,               /**< SCIP data structure (master problem) */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_SOLVER*           solver              /**< pricing solver */
    )
 {
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
-   return SCIPgetClockTime(scip, solver->heurfarkasclock);
+   return SCIPgetClockTime(GCGgetMasterprob(gcg), solver->heurfarkasclock);
 }
 
 /** gets heuristic reduced cost pricing time of pricing solver */
 SCIP_Real GCGsolverGetHeurRedcostTime(
-   SCIP*                 scip,               /**< SCIP data structure (master problem) */
+   GCG*                  gcg,                /**< GCG data structure */
    GCG_SOLVER*           solver              /**< pricing solver */
    )
 {
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
-   return SCIPgetClockTime(scip, solver->heurredcostclock);
+   return SCIPgetClockTime(GCGgetMasterprob(gcg), solver->heurredcostclock);
 }
