@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -37,13 +38,13 @@
 #include <assert.h>
 #include <string.h>
 
-#include "solver_knapsack.h"
+#include "gcg/solver_knapsack.h"
 #include "scip/cons_linear.h"
 #include "scip/cons_knapsack.h"
-#include "pricer_gcg.h"
-#include "pub_solver.h"
-#include "relax_gcg.h"
-#include "pub_gcgcol.h"
+#include "gcg/pricer_gcg.h"
+#include "gcg/pub_solver.h"
+#include "gcg/relax_gcg.h"
+#include "gcg/pub_gcgcol.h"
 
 #define SOLVER_NAME          "knapsack"
 #define SOLVER_DESC          "knapsack solver for pricing problems"
@@ -64,7 +65,7 @@
 static
 SCIP_RETCODE solveKnapsack(
    SCIP_Bool             exactly,            /**< should the pricing problem be solved to optimality or heuristically? */
-   SCIP*                 scip,               /**< master problem SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP*                 pricingprob,        /**< pricing problem SCIP data structure */
    GCG_SOLVER*           solver,             /**< solver data structure */
    int                   probnr,             /**< problem number */
@@ -72,18 +73,56 @@ SCIP_RETCODE solveKnapsack(
    GCG_PRICINGSTATUS*    status              /**< pointer to store pricing problem status */
    )
 { /*lint -e715 */
+   int nsolvars;
+   GCG_COL* col;
+   SCIP_Real solval;
+   SCIP_VAR** solvars = NULL;
+   SCIP_Real* solvals = NULL;
+
+   assert(pricingprob != NULL);
+
+   SCIP_CALL( GCGsolverKnapsackSolveKnapsack(exactly, pricingprob, &solval, status, &solvars, &solvals, &nsolvars) );
+
+   if( solvars != NULL )
+   {
+      assert(solvals != NULL);
+
+      if( *status == GCG_PRICINGSTATUS_OPTIMAL )
+      {
+         *lowerbound = exactly ? solval : -SCIPinfinity(pricingprob);
+
+         SCIP_CALL( GCGcreateGcgCol(gcg, pricingprob, &col, probnr, solvars, solvals, nsolvars, FALSE, SCIPinfinity(pricingprob)) );
+         SCIP_CALL( GCGpricerAddCol(gcg, col) );
+      }
+
+      SCIPfreeBufferArray(pricingprob, &solvals);
+      SCIPfreeBufferArray(pricingprob, &solvars);
+   }
+
+   return SCIP_OKAY;
+}
+
+
+GCG_EXPORT
+SCIP_RETCODE GCGsolverKnapsackSolveKnapsack(
+   SCIP_Bool             exactly,            /**< should the pricing problem be solved to optimality or heuristically? */
+   SCIP*                 pricingprob,        /**< pricing problem SCIP data structure */
+   SCIP_Real*            solval,             /**< pointer to store the solution objective value */
+   GCG_PRICINGSTATUS*    status,             /**< pointer to store pricing problem status */
+   SCIP_VAR***           solvars,            /**< pointer to store solution vars (will be allocated as buffer array and must be freed afterwards) */
+   SCIP_Real**           solvals,            /**< pointer to store solution vals (will be allocated as buffer array and must be freed afterwards) */
+   int*                  nsolvars            /**< pointer to store the number of solution vars */
+   )
+{
    SCIP_CONS* cons;
    SCIP_CONSHDLR* conshdlr;
    SCIP_VAR** consvars;
    SCIP_Longint* consvals;
    int nconsvars;
-   SCIP_VAR** solvars;
-   SCIP_Real* solvals;
-   int nsolvars;
+
    SCIP_VAR** pricingprobvars;
    int npricingprobvars;
    int nconss;
-
    int                   nitems;
    SCIP_Longint*         weights;
    SCIP_Real*            profits;
@@ -95,19 +134,14 @@ SCIP_RETCODE solveKnapsack(
    int                   nsolitems;
    int*                  nonsolitems;
    int                   nnonsolitems;
-   SCIP_Real             solval;
    SCIP_Bool             success;
-   GCG_COL*              col;
    SCIP_Bool             inferbounds;
    int i;
    int j;
    int k;
 
    /* check preconditions */
-   assert(scip != NULL);
    assert(pricingprob != NULL);
-   assert(solver != NULL);
-   assert(lowerbound != NULL);
    assert(status != NULL);
 
    assert(SCIPgetObjsense(pricingprob) == SCIP_OBJSENSE_MINIMIZE);
@@ -123,6 +157,12 @@ SCIP_RETCODE solveKnapsack(
     * - all variables are nonnegative integer variables
     * - there is only one constraint, which has infinite lhs and integer rhs
     */
+   nconss = SCIPgetNConss(pricingprob);
+   if( nconss != 1 )
+   {
+      SCIPdebugMessage("  -> pricing problem has more than one constraint\n");
+      return SCIP_OKAY;
+   }
    if( SCIPgetNBinVars(pricingprob) + SCIPgetNIntVars(pricingprob) < npricingprobvars )
    {
       SCIPdebugMessage("  -> pricing problem has continuous variables\n");
@@ -137,13 +177,6 @@ SCIP_RETCODE solveKnapsack(
       }
    }
 
-   nconss = SCIPgetNConss(pricingprob);
-   if( nconss != 1 )
-   {
-      SCIPdebugMessage("  -> pricing problem has more than one constraint\n");
-      return SCIP_OKAY;
-   }
-
    cons = SCIPgetConss(pricingprob)[0];
    assert(cons != NULL);
 
@@ -279,9 +312,8 @@ SCIP_RETCODE solveKnapsack(
    }
    SCIPdebugMessage("-> %d items\n", nitems);
 
-   SCIP_CALL( SCIPallocBufferArray(pricingprob, &solvars, npricingprobvars) );
-   SCIP_CALL( SCIPallocBufferArray(pricingprob, &solvals, npricingprobvars) );
-
+   SCIP_CALL( SCIPallocBufferArray(pricingprob, solvars, npricingprobvars) );
+   SCIP_CALL( SCIPallocBufferArray(pricingprob, solvals, npricingprobvars) );
    SCIP_CALL( SCIPallocBufferArray(pricingprob, &items, nitems) );
    SCIP_CALL( SCIPallocBufferArray(pricingprob, &weights, nitems) );
    SCIP_CALL( SCIPallocBufferArray(pricingprob, &profits, nitems) );
@@ -371,12 +403,12 @@ SCIP_RETCODE solveKnapsack(
       if( exactly )
       {
          SCIP_CALL( SCIPsolveKnapsackExactly(pricingprob, nitems, weights, profits, capacity, items, solitems,
-            nonsolitems, &nsolitems, &nnonsolitems, &solval, &success ));
+            nonsolitems, &nsolitems, &nnonsolitems, solval, &success ));
       }
       else
       {
          SCIP_CALL( SCIPsolveKnapsackApproximately(pricingprob, nitems, weights, profits, capacity, items, solitems,
-            nonsolitems, &nsolitems, &nnonsolitems, &solval ));
+            nonsolitems, &nsolitems, &nnonsolitems, solval ));
       }
 
       if( !success )
@@ -387,10 +419,10 @@ SCIP_RETCODE solveKnapsack(
       else if( exactly )
          *status = GCG_PRICINGSTATUS_OPTIMAL;
 
-      SCIPdebugMessage("Knapsack solved, solval = %g\n", solval);
+      SCIPdebugMessage("Knapsack solved, solval = %g\n", *solval);
    }
 
-   nsolvars = 0;
+   *nsolvars = 0;
 
    for( i = 0; i < nsolitems; i++ )
    {
@@ -398,18 +430,18 @@ SCIP_RETCODE solveKnapsack(
 
       if( consvals[solitems[i]] >= 0 && !SCIPvarIsNegated(consvars[solitems[i]]) )
       {
-         for( j = 0; j < nsolvars; ++j )
-            if( solvars[j] == consvars[solitems[i]] )
+         for( j = 0; j < *nsolvars; ++j )
+            if( (*solvars)[j] == consvars[solitems[i]] )
                break;
 
-         if( j == nsolvars )
+         if( j == *nsolvars )
          {
-            solvars[j] = consvars[solitems[i]];
-            solvals[j] = 1.0;
-            ++nsolvars;
+            (*solvars)[j] = consvars[solitems[i]];
+            (*solvals)[j] = 1.0;
+            ++(*nsolvars);
          }
          else
-            solvals[j] += 1.0;
+            (*solvals)[j] += 1.0;
       }
    }
 
@@ -421,18 +453,18 @@ SCIP_RETCODE solveKnapsack(
       {
          SCIP_VAR* solvar = SCIPvarIsNegated(consvars[nonsolitems[i]]) ? SCIPvarGetNegatedVar(consvars[nonsolitems[i]]) : consvars[nonsolitems[i]];
 
-         for( j = 0; j < nsolvars; ++j )
-            if( solvars[j] == solvar )
+         for( j = 0; j < *nsolvars; ++j )
+            if( (*solvars)[j] == solvar )
                break;
 
-         if( j == nsolvars )
+         if( j == *nsolvars )
          {
-            solvars[j] = solvar;
-            solvals[j] = 1.0;
-            ++nsolvars;
+            (*solvars)[j] = solvar;
+            (*solvals)[j] = 1.0;
+            ++(*nsolvars);
          }
          else
-            solvals[j] += 1.0;
+            (*solvals)[j] += 1.0;
       }
    }
 
@@ -440,30 +472,25 @@ SCIP_RETCODE solveKnapsack(
    {
       if( SCIPisGE(pricingprob, SCIPvarGetLbLocal(pricingprobvars[i]), 1.0) )
       {
-         for( j = 0; j < nsolvars; ++j )
-            if( solvars[j] == pricingprobvars[i] )
+         for( j = 0; j < *nsolvars; ++j )
+            if( (*solvars)[j] == pricingprobvars[i] )
                break;
 
-         if( j == nsolvars )
+         if( j == *nsolvars )
          {
-            solvars[j] = pricingprobvars[i];
-            solvals[j] = SCIPfloor(pricingprob, SCIPvarGetLbLocal(pricingprobvars[i]));
-            ++nsolvars;
+            (*solvars)[j] = pricingprobvars[i];
+            (*solvals)[j] = SCIPfloor(pricingprob, SCIPvarGetLbLocal(pricingprobvars[i]));
+            ++(*nsolvars);
          }
          else
-            solvals[j] += SCIPfloor(pricingprob, SCIPvarGetLbLocal(pricingprobvars[i]));
+            (*solvals)[j] += SCIPfloor(pricingprob, SCIPvarGetLbLocal(pricingprobvars[i]));
       }
    }
 
-   SCIP_CALL( GCGcreateGcgCol(pricingprob, &col, probnr, solvars, solvals, nsolvars, FALSE, SCIPinfinity(pricingprob)) );
-   SCIP_CALL( GCGpricerAddCol(scip, col) );
+   *solval = 0.0;
 
-   solval = 0.0;
-
-   for( i = 0; i < nsolvars; ++i )
-      solval += solvals[i] * SCIPvarGetObj(solvars[i]);
-
-   *lowerbound = exactly ? solval : -SCIPinfinity(pricingprob);
+   for( i = 0; i < *nsolvars; ++i )
+      (*solval) += (*solvals)[i] * SCIPvarGetObj((*solvars)[i]);
 
  TERMINATE:
    SCIPfreeBufferArray(pricingprob, &nonsolitems);
@@ -471,14 +498,13 @@ SCIP_RETCODE solveKnapsack(
    SCIPfreeBufferArray(pricingprob, &profits);
    SCIPfreeBufferArray(pricingprob, &weights);
    SCIPfreeBufferArray(pricingprob, &items);
-   SCIPfreeBufferArray(pricingprob, &solvals);
-   SCIPfreeBufferArray(pricingprob, &solvars);
    SCIPfreeBufferArray(pricingprob, &ubs);
    SCIPfreeBufferArray(pricingprob, &consvals);
 
    return SCIP_OKAY;
 }
 
+
 /*
  * Callback methods for pricing problem solver
  */
@@ -496,7 +522,7 @@ GCG_DECL_SOLVERSOLVE(solverSolveKnapsack)
 {  /*lint --e{715}*/
 
    /* solve the knapsack problem exactly */
-   SCIP_CALL( solveKnapsack(TRUE, scip, pricingprob, solver, probnr, lowerbound, status) );
+   SCIP_CALL( solveKnapsack(TRUE, gcg, pricingprob, solver, probnr, lowerbound, status) );
 
    return SCIP_OKAY;
 }
@@ -508,7 +534,7 @@ GCG_DECL_SOLVERSOLVEHEUR(solverSolveHeurKnapsack)
 {  /*lint --e{715}*/
 
    /* solve the knapsack problem approximately */
-   SCIP_CALL( solveKnapsack(FALSE, scip, pricingprob, solver, probnr, lowerbound, status) );
+   SCIP_CALL( solveKnapsack(FALSE, gcg, pricingprob, solver, probnr, lowerbound, status) );
 
    return SCIP_OKAY;
 }
@@ -516,10 +542,10 @@ GCG_DECL_SOLVERSOLVEHEUR(solverSolveHeurKnapsack)
 
 /** creates the knapsack solver for pricing problems and includes it in GCG */
 SCIP_RETCODE GCGincludeSolverKnapsack(
-   SCIP*                 scip                /**< SCIP data structure */
+   GCG*                  gcg                 /**< GCG data structure */
    )
 {
-   SCIP_CALL( GCGpricerIncludeSolver(scip, SOLVER_NAME, SOLVER_DESC, SOLVER_PRIORITY,
+   SCIP_CALL( GCGpricerIncludeSolver(gcg, SOLVER_NAME, SOLVER_DESC, SOLVER_PRIORITY,
          SOLVER_HEURENABLED, SOLVER_EXACTENABLED,
          solverUpdateKnapsack, solverSolveKnapsack, solverSolveHeurKnapsack,
          solverFreeKnapsack, solverInitKnapsack, solverExitKnapsack,
