@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -39,20 +40,20 @@
 #include <assert.h>
 #include <string.h>
 
-#include "solver_mip.h"
+#include "gcg/solver_mip.h"
 #include "scip/cons_linear.h"
 #include "scip/cons_knapsack.h"
-#include "gcg.h"
-#include "pricer_gcg.h"
-#include "pub_solver.h"
+#include "gcg/gcg.h"
+#include "gcg/pricer_gcg.h"
+#include "gcg/pub_solver.h"
 #include "scip/scipdefplugins.h"
-#include "pub_gcgcol.h"
+#include "gcg/pub_gcgcol.h"
 
-#define SOLVER_NAME          "mip"
-#define SOLVER_DESC          "pricing solver solving the pricing problem as a sub-MIP, using SCIP"
-#define SOLVER_PRIORITY      0
-#define SOLVER_HEURENABLED   TRUE            /**< indicates whether the heuristic solving method of the solver should be enabled */
-#define SOLVER_EXACTENABLED  TRUE            /**< indicates whether the exact solving method of the solver should be enabled */
+#define SOLVER_NAME                  "mip"
+#define SOLVER_DESC                  "pricing solver solving the pricing problem as a sub-MIP, using SCIP"
+#define SOLVER_PRIORITY              0
+#define SOLVER_HEURENABLED           TRUE    /**< indicates whether the heuristic solving method of the solver should be enabled */
+#define SOLVER_EXACTENABLED          TRUE    /**< indicates whether the exact solving method of the solver should be enabled */
 
 #define DEFAULT_CHECKSOLS            TRUE    /**< should solutions be checked extensively */
 #define DEFAULT_STARTNODELIMIT       1000LL  /**< start node limit for heuristic pricing */
@@ -64,6 +65,7 @@
 #define DEFAULT_GAPLIMITFAC          0.8     /**< factor by which to decrease gap limit for heuristic pricing (1.0: subtract start limit) */
 #define DEFAULT_SOLLIMITFAC          1.0     /**< factor by which to increase solution limit for heuristic pricing (1.0: add start limit) */
 #define DEFAULT_SETTINGSFILE         "-"     /**< settings file to be applied in pricing problems */
+#define DEFAULT_ENABLE_WARNINGS      FALSE   /**< should warnings (of pricing problems) be enabled by default */
 
 
 
@@ -81,6 +83,7 @@ struct GCG_SolverData
    SCIP_Real             gaplimitfac;         /**< factor by which to decrease gap limit for heuristic pricing (1.0: subtract start limit) */
    SCIP_Real             sollimitfac;         /**< factor by which to increase solution limit for heuristic pricing (1.0: add start limit) */
    char*                 settingsfile;        /**< settings file to be applied in pricing problems */
+   SCIP_Bool             enablewarnings;      /**< enable warnings of pricing problems */
 
    /* solver data */
    SCIP_Longint*         curnodelimit;        /**< current node limit per pricing problem */
@@ -94,63 +97,6 @@ struct GCG_SolverData
  * Local methods
  */
 
-/** extracts ray from pricing problem */
-static
-SCIP_RETCODE createColumnFromRay(
-   SCIP*                 pricingprob,        /**< pricing problem SCIP data structure */
-   int                   probnr,             /**< problem number */
-   GCG_COL**             newcol              /**< column pointer to store new column */
-)
-{
-   SCIP_VAR** probvars;
-   SCIP_VAR** solvars;
-   SCIP_Real* solvals;
-   int nprobvars;
-   int nsolvars;
-   int i;
-
-   assert(pricingprob != NULL);
-   assert(newcol != NULL);
-   assert(SCIPhasPrimalRay(pricingprob));
-
-   probvars = SCIPgetOrigVars(pricingprob);
-   nprobvars = SCIPgetNOrigVars(pricingprob);
-   nsolvars = 0;
-   SCIP_CALL( SCIPallocBufferArray(pricingprob, &solvars, nprobvars) );
-   SCIP_CALL( SCIPallocBufferArray(pricingprob, &solvals, nprobvars) );
-
-   /* store the primal ray values */
-   for ( i = 0; i < nprobvars; i++ )
-   {
-      if ( SCIPisZero(pricingprob, SCIPgetPrimalRayVal(pricingprob, probvars[i])) )
-         continue;
-
-      assert(!SCIPisInfinity(pricingprob, SCIPgetPrimalRayVal(pricingprob, probvars[i])));
-      assert(!SCIPisInfinity(pricingprob, -SCIPgetPrimalRayVal(pricingprob, probvars[i])));
-
-      solvars[nsolvars] = probvars[i];
-      solvals[nsolvars] = SCIPgetPrimalRayVal(pricingprob, probvars[i]);
-      assert(!SCIPisInfinity(pricingprob, solvals[nsolvars]));
-      /* todo: check if/ensure that this value is integral! */
-      nsolvars++;
-
-      SCIPdebugMessage("%s: %g (obj = %g)\n", SCIPvarGetName(probvars[i]), SCIPgetPrimalRayVal(pricingprob, probvars[i]), SCIPvarGetObj(probvars[i]));
-   }
-
-   SCIP_CALL( SCIPfreeSolve(pricingprob, TRUE) );
-   SCIP_CALL( SCIPtransformProb(pricingprob) );
-
-   /* todo: is it okay to write pricingprob into column structure? */
-   SCIP_CALL( GCGcreateGcgCol(pricingprob, newcol, probnr, solvars, solvals, nsolvars, TRUE, SCIPinfinity(pricingprob)) );
-
-   SCIPfreeBufferArray(pricingprob, &solvals);
-   SCIPfreeBufferArray(pricingprob, &solvars);
-
-   SCIPdebugMessage("pricingproblem has an unbounded ray!\n");
-
-   return SCIP_OKAY;
-}
-
 /** solves the pricing problem again without presolving */
 static
 SCIP_RETCODE resolvePricingWithoutPresolving(
@@ -227,68 +173,6 @@ SCIP_RETCODE checkSolNew(
    return SCIP_OKAY;
 }
 
-/** get the status of the pricing problem */
-static
-GCG_PRICINGSTATUS getPricingstatus(
-  SCIP*                 pricingprob         /**< pricing problem SCIP data structure */
-  )
-{
-   /* all SCIP statuses handled so far; these are currently:
-    *    SCIP_STATUS_USERINTERRUPT
-    *    SCIP_STATUS_NODELIMIT
-    *    SCIP_STATUS_TOTALNODELIMIT
-    *    SCIP_STATUS_STALLNODELIMIT
-    *    SCIP_STATUS_TIMELIMIT
-    *    SCIP_STATUS_MEMLIMIT
-    *    SCIP_STATUS_GAPLIMIT
-    *    SCIP_STATUS_SOLLIMIT
-    *    SCIP_STATUS_BESTSOLLIMIT
-    *    SCIP_STATUS_OPTIMAL
-    *    SCIP_STATUS_INFEASIBLE
-    *    SCIP_STATUS_UNBOUNDED
-    *    SCIP_STATUS_INFORUNBD
-    */
-   /* @todo: can SCIP_STATUS_UNKNOWN happen, too? */
-   assert((SCIPgetStatus(pricingprob) >= SCIP_STATUS_USERINTERRUPT && SCIPgetStatus(pricingprob) <= SCIP_STATUS_BESTSOLLIMIT)
-          || SCIPgetStatus(pricingprob) == SCIP_STATUS_OPTIMAL
-          || SCIPgetStatus(pricingprob) == SCIP_STATUS_INFEASIBLE
-          || SCIPgetStatus(pricingprob) == SCIP_STATUS_UNBOUNDED
-          || SCIPgetStatus(pricingprob) == SCIP_STATUS_INFORUNBD);
-
-   /* translate SCIP solution status to GCG pricing status */
-   switch( SCIPgetStatus(pricingprob) ) 
-   {
-   case SCIP_STATUS_USERINTERRUPT:
-     SCIPdebugMessage("  -> interrupted, %d solutions found\n", SCIPgetNSols(pricingprob)); /*lint -fallthrough*/
-   case SCIP_STATUS_UNKNOWN:
-   case SCIP_STATUS_TOTALNODELIMIT:
-   case SCIP_STATUS_TIMELIMIT:
-   case SCIP_STATUS_MEMLIMIT:
-   case SCIP_STATUS_BESTSOLLIMIT:
-     return GCG_PRICINGSTATUS_UNKNOWN;
-
-   case SCIP_STATUS_NODELIMIT:
-   case SCIP_STATUS_STALLNODELIMIT:
-   case SCIP_STATUS_GAPLIMIT:
-   case SCIP_STATUS_SOLLIMIT:
-     return GCG_PRICINGSTATUS_SOLVERLIMIT;
-
-   case SCIP_STATUS_OPTIMAL:
-     return GCG_PRICINGSTATUS_OPTIMAL;
-
-   case SCIP_STATUS_INFEASIBLE:
-     return GCG_PRICINGSTATUS_INFEASIBLE;
-
-   case SCIP_STATUS_UNBOUNDED:
-   case SCIP_STATUS_INFORUNBD:
-     return GCG_PRICINGSTATUS_UNBOUNDED;
-
-   default:
-     SCIPerrorMessage("invalid SCIP status of pricing problem: %d\n", SCIPgetStatus(pricingprob));
-     return GCG_PRICINGSTATUS_UNKNOWN;
-   }
-}
-
 /** check whether a column contains an infinite solution value */
 static
 SCIP_Bool solutionHasInfiniteValue(
@@ -311,81 +195,10 @@ SCIP_Bool solutionHasInfiniteValue(
    return FALSE;
 }
 
-/** transforms feasible solutions of the pricing problem into columns */
-static
-SCIP_RETCODE getColumnsFromPricingprob(
-   SCIP*                 scip,               /**< master problem SCIP data structure */
-   SCIP*                 pricingprob,        /**< pricing problem SCIP data structure */
-   int                   probnr,             /**< problem number */
-   SCIP_Bool             checksols           /**< should solutions be checked extensively */
-)
-{
-   SCIP_SOL** probsols;
-   int nprobsols;
-
-   int s;
-
-   probsols = SCIPgetSols(pricingprob);
-   nprobsols = SCIPgetNSols(pricingprob);
-
-   for( s = 0; s < nprobsols; s++ )
-   {
-      GCG_COL* col;
-      SCIP_Bool feasible;
-      assert(probsols[s] != NULL);
-      SCIP_CALL( SCIPcheckSolOrig(pricingprob, probsols[s], &feasible, FALSE, FALSE) );
-
-      if( !feasible )
-      {
-         SCIPwarningMessage(pricingprob, "solution %d of pricing problem %d not feasible:\n", s, probnr);
-         SCIP_CALL( SCIPcheckSolOrig(pricingprob, probsols[s], &feasible, TRUE, TRUE) );
-         continue;
-      }
-
-      /* check whether the solution is equal to one of the previous solutions */
-      if( checksols )
-      {
-         SCIP_Bool isnew;
-
-         SCIP_CALL( checkSolNew(pricingprob, probsols, s, &isnew) );
-
-         if( !isnew )
-            continue;
-      }
-
-      /* Check whether the pricing problem solution has infinite values; if not, transform it to a column */
-      if( !solutionHasInfiniteValue(pricingprob, probsols[s]) )
-      {
-         SCIP_CALL( GCGcreateGcgColFromSol(pricingprob, &col, probnr, probsols[s], FALSE, SCIPinfinity(pricingprob)) );
-         SCIP_CALL( GCGpricerAddCol(scip, col) );
-      }
-      /* If the best solution has infinite values, try to repair it */
-      else if( s == 0 )
-      {
-         SCIP_SOL* newsol;
-         SCIP_Bool success;
-
-         newsol = NULL;
-         success = FALSE;
-
-         SCIPdebugMessage("solution has infinite values, create a copy with finite values\n");
-
-         SCIP_CALL( SCIPcreateFiniteSolCopy(pricingprob, &newsol, probsols[0], &success) );
-         assert(success);
-         assert(newsol != NULL);
-
-         SCIP_CALL( GCGcreateGcgColFromSol(pricingprob, &col, probnr, newsol, FALSE, SCIPinfinity(pricingprob)) );
-         SCIP_CALL( GCGpricerAddCol(scip, col) );
-      }
-   }
-
-   return SCIP_OKAY;
-}
-
 /** solves the given pricing problem as a sub-SCIP */
 static
 SCIP_RETCODE solveProblem(
-   SCIP*                 scip,               /**< master problem SCIP data structure */
+   GCG*                  gcg,                /**< GCG data structure */
    SCIP*                 pricingprob,        /**< pricing problem SCIP data structure */
    int                   probnr,             /**< problem number */
    GCG_SOLVERDATA*       solverdata,         /**< solver data structure */
@@ -394,23 +207,45 @@ SCIP_RETCODE solveProblem(
    )
 {
    GCG_COL* col;
-   SCIP_RETCODE retcode;
+   SCIP_RETCODE retcode = SCIP_OKAY;
 #ifdef SCIP_STATISTIC
    SCIP_Longint oldnnodes;
 #endif
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(pricingprob != NULL);
    assert(probnr >= 0);
    assert(solverdata != NULL);
    assert(lowerbound != NULL);
    assert(status != NULL);
 
+#ifndef DEBUG_PRICING_ALL_OUTPUT
+   if( !solverdata->enablewarnings && !SCIPmessagehdlrIsQuiet(SCIPgetMessagehdlr(pricingprob)) )
+   {
+      SCIPsetMessagehdlrQuiet(pricingprob, TRUE);
+   }
+#endif
+
 #ifdef SCIP_STATISTIC
    oldnnodes = SCIPgetNNodes(pricingprob);
 #endif
-   /* solve the pricing SCIP */
-   retcode = SCIPsolve(pricingprob);
+
+   if( SCIPgetStage(pricingprob) == SCIP_STAGE_PROBLEM )
+   {
+      retcode = SCIPpresolve(pricingprob);
+      if( SCIPgetNContVars(pricingprob) > 0 )
+      {
+         /* disable presolving restarts because of numerical instability caused by these (workaround) */
+         SCIP_CALL( SCIPsetIntParam(pricingprob, "presolving/maxrestarts", 0) );
+      }
+   }
+
+   if( retcode == SCIP_OKAY )
+   {
+      /* solve the pricing SCIP */
+      retcode = SCIPsolve(pricingprob);
+   }
+
    if( retcode != SCIP_OKAY )
    {
       SCIPwarningMessage(pricingprob, "Pricing problem %d terminated with retcode = %d, ignoring\n", probnr, retcode);
@@ -444,8 +279,8 @@ SCIP_RETCODE solveProblem(
       }
 
       SCIPdebugMessage("  -> unbounded, creating column from ray\n");
-      SCIP_CALL( createColumnFromRay(pricingprob, probnr, &col) );
-      SCIP_CALL( GCGpricerAddCol(scip, col) );
+      SCIP_CALL( createColumnFromRay(gcg, pricingprob, NULL, NULL, probnr, &col) );
+      SCIP_CALL( GCGpricerAddCol(gcg, col) );
 
       break;
 
@@ -459,7 +294,7 @@ SCIP_RETCODE solveProblem(
           && SCIPgetStatus(pricingprob) != SCIP_STATUS_SOLLIMIT));
 
       /* Transform at most maxcols many solutions from the pricing problem into columns */
-      SCIP_CALL( getColumnsFromPricingprob(scip, pricingprob, probnr, solverdata->checksols) );
+      SCIP_CALL( getColumnsFromPricingprob(gcg, pricingprob, NULL, NULL, probnr, solverdata->checksols) );
 
       *lowerbound = SCIPgetDualbound(pricingprob);
 
@@ -484,14 +319,16 @@ static
 GCG_DECL_SOLVERFREE(solverFreeMip)
 {
    GCG_SOLVERDATA* solverdata;
+   SCIP* scip;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
+   scip = GCGgetDwMasterprob(gcg);
    solverdata = GCGsolverGetData(solver);
    assert(solverdata != NULL);
 
-   SCIPfreeMemory(scip, &solverdata);
+   SCIPfreeBlockMemory(scip, &solverdata);
 
    GCGsolverSetData(solver, NULL);
 
@@ -511,19 +348,22 @@ GCG_DECL_SOLVERINITSOL(solverInitsolMip)
    GCG_SOLVERDATA* solverdata;
    int npricingprobs;
    int i;
+   SCIP* masterprob;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
+   masterprob = GCGgetMasterprob(gcg);
+
    solverdata = GCGsolverGetData(solver);
    assert(solverdata != NULL);
 
-   npricingprobs = GCGgetNPricingprobs(GCGmasterGetOrigprob(scip));
+   npricingprobs = GCGgetNPricingprobs(gcg);
 
-   SCIP_CALL( SCIPallocBlockMemoryArray(scip, &solverdata->curnodelimit, npricingprobs) );
-   SCIP_CALL( SCIPallocBlockMemoryArray(scip, &solverdata->curstallnodelimit, npricingprobs) );
-   SCIP_CALL( SCIPallocBlockMemoryArray(scip, &solverdata->curgaplimit, npricingprobs) );
-   SCIP_CALL( SCIPallocBlockMemoryArray(scip, &solverdata->cursollimit, npricingprobs) );
+   SCIP_CALL( SCIPallocBlockMemoryArray(masterprob, &solverdata->curnodelimit, npricingprobs) );
+   SCIP_CALL( SCIPallocBlockMemoryArray(masterprob, &solverdata->curstallnodelimit, npricingprobs) );
+   SCIP_CALL( SCIPallocBlockMemoryArray(masterprob, &solverdata->curgaplimit, npricingprobs) );
+   SCIP_CALL( SCIPallocBlockMemoryArray(masterprob, &solverdata->cursollimit, npricingprobs) );
 
    for( i = 0; i < npricingprobs; ++i )
    {
@@ -542,19 +382,22 @@ GCG_DECL_SOLVEREXITSOL(solverExitsolMip)
 {
    GCG_SOLVERDATA* solverdata;
    int npricingprobs;
+   SCIP* masterprob;
 
-   assert(scip != NULL);
+   assert(gcg != NULL);
    assert(solver != NULL);
 
+   masterprob = GCGgetMasterprob(gcg);
+
    solverdata = GCGsolverGetData(solver);
    assert(solverdata != NULL);
 
-   npricingprobs = GCGgetNPricingprobs(GCGmasterGetOrigprob(scip));
+   npricingprobs = GCGgetNPricingprobs(gcg);
 
-   SCIPfreeBlockMemoryArray(scip, &solverdata->cursollimit, npricingprobs);
-   SCIPfreeBlockMemoryArray(scip, &solverdata->curgaplimit, npricingprobs);
-   SCIPfreeBlockMemoryArray(scip, &solverdata->curstallnodelimit, npricingprobs);
-   SCIPfreeBlockMemoryArray(scip, &solverdata->curnodelimit, npricingprobs);
+   SCIPfreeBlockMemoryArray(masterprob, &solverdata->cursollimit, npricingprobs);
+   SCIPfreeBlockMemoryArray(masterprob, &solverdata->curgaplimit, npricingprobs);
+   SCIPfreeBlockMemoryArray(masterprob, &solverdata->curstallnodelimit, npricingprobs);
+   SCIPfreeBlockMemoryArray(masterprob, &solverdata->curnodelimit, npricingprobs);
 
    return SCIP_OKAY;
 }
@@ -588,10 +431,10 @@ GCG_DECL_SOLVERSOLVE(solverSolveMip)
    *lowerbound = -SCIPinfinity(pricingprob);
 
    SCIPdebugMessage("Solving pricing %d (pointer: %p)\n", probnr, (void*)pricingprob);
-   SCIP_CALL( solveProblem(scip, pricingprob, probnr, solverdata, lowerbound, status) );
+   SCIP_CALL( solveProblem(gcg, pricingprob, probnr, solverdata, lowerbound, status) );
 
 #ifdef DEBUG_PRICING_ALL_OUTPUT
-   SCIP_CALL( SCIPsetIntParam(pricingprob, "display/verblevel", 0) );
+   SCIP_CALL( SCIPsetIntParam(pricingprob, "display/verblevel", SCIP_VERBLEVEL_NONE) );
    SCIP_CALL( SCIPprintStatistics(pricingprob, NULL) );
 #endif
 
@@ -664,30 +507,241 @@ GCG_DECL_SOLVERSOLVEHEUR(solverSolveHeurMip)
 
    /* solve the pricing problem */
    SCIPdebugMessage("Solving pricing %d heuristically (pointer: %p)\n", probnr, (void*)pricingprob);
-   SCIP_CALL( solveProblem(scip, pricingprob, probnr, solverdata, lowerbound, status) );
+   SCIP_CALL( solveProblem(gcg, pricingprob, probnr, solverdata, lowerbound, status) );
 
 #ifdef DEBUG_PRICING_ALL_OUTPUT
-   SCIP_CALL( SCIPsetIntParam(pricingprob, "display/verblevel", 0) );
+   SCIP_CALL( SCIPsetIntParam(pricingprob, "display/verblevel", SCIP_VERBLEVEL_NONE) );
    SCIP_CALL( SCIPprintStatistics(pricingprob, NULL) );
 #endif
 
    return SCIP_OKAY;
 }
 
+/** extracts ray from a subproblem used to solve a pricing problem pricing problem (or directly from the pricing problem if no subproblem is specified) */
+SCIP_RETCODE createColumnFromRay(
+   GCG*                  gcg,                /**< GCG data structure */
+   SCIP*                 pricingprob,        /**< pricing problem SCIP data structure */
+   SCIP*                 subproblem,         /**< SCIP data structure that contains the actual solution (if NULL pricingprob will be used) */
+   SCIP_HASHMAP*         varmap,             /**< mapping of pricingprob vars to subproblem vars (can be NULL if subproblem is NULL) */
+   int                   probnr,             /**< problem number */
+   GCG_COL**             newcol              /**< column pointer to store new column */
+)
+{
+   SCIP_VAR** probvars;
+   SCIP_VAR** solvars;
+   SCIP_Real* solvals;
+   int nprobvars;
+   int nsolvars;
+   int i;
+   SCIP* solprob;
+
+   if (subproblem == NULL)
+      solprob = pricingprob;
+   else
+      solprob = subproblem;
+
+   assert(pricingprob != NULL);
+   assert(newcol != NULL);
+   assert(SCIPhasPrimalRay(solprob));
+
+   probvars = SCIPgetOrigVars(pricingprob);
+   nprobvars = SCIPgetNOrigVars(pricingprob);
+   nsolvars = 0;
+   SCIP_CALL( SCIPallocBufferArray(pricingprob, &solvars, nprobvars) );
+   SCIP_CALL( SCIPallocBufferArray(pricingprob, &solvals, nprobvars) );
+
+   /* store the primal ray values */
+   for ( i = 0; i < nprobvars; i++ )
+   {
+      double solval;
+
+      if (varmap == NULL)
+         solval = SCIPgetPrimalRayVal(pricingprob, probvars[i]);
+      else
+         solval = SCIPgetPrimalRayVal(subproblem, SCIPhashmapGetImage(varmap, probvars[i]));
+
+      if ( SCIPisZero(solprob, solval) )
+         continue;
+
+      assert(!SCIPisInfinity(solprob, solval));
+      assert(!SCIPisInfinity(solprob, -solval));
+
+      solvars[nsolvars] = probvars[i];
+      solvals[nsolvars] = solval;
+      assert(!SCIPisInfinity(solprob, solvals[nsolvars]));
+      /* todo: check if/ensure that this value is integral! */
+      nsolvars++;
+
+      SCIPdebugMessage("%s: %g (obj = %g)\n", SCIPvarGetName(probvars[i]), solval, SCIPvarGetObj(probvars[i]));
+   }
+
+   SCIP_CALL( SCIPfreeSolve(solprob, TRUE) );
+   SCIP_CALL( SCIPtransformProb(solprob) );
+
+   /* todo: is it okay to write pricingprob into column structure? */
+   SCIP_CALL( GCGcreateGcgCol(gcg, pricingprob, newcol, probnr, solvars, solvals, nsolvars, TRUE, SCIPinfinity(pricingprob)) );
+
+   SCIPfreeBufferArray(pricingprob, &solvals);
+   SCIPfreeBufferArray(pricingprob, &solvars);
+
+   SCIPdebugMessage("pricingproblem has an unbounded ray!\n");
+
+   return SCIP_OKAY;
+}
+
+GCG_PRICINGSTATUS getPricingstatus(
+  SCIP*                 pricingprob         /**< pricing problem SCIP data structure */
+  )
+{
+   /* all SCIP statuses handled so far; these are currently:
+    *    SCIP_STATUS_USERINTERRUPT
+    *    SCIP_STATUS_NODELIMIT
+    *    SCIP_STATUS_TOTALNODELIMIT
+    *    SCIP_STATUS_STALLNODELIMIT
+    *    SCIP_STATUS_TIMELIMIT
+    *    SCIP_STATUS_MEMLIMIT
+    *    SCIP_STATUS_GAPLIMIT
+    *    SCIP_STATUS_SOLLIMIT
+    *    SCIP_STATUS_BESTSOLLIMIT
+    *    SCIP_STATUS_OPTIMAL
+    *    SCIP_STATUS_INFEASIBLE
+    *    SCIP_STATUS_UNBOUNDED
+    *    SCIP_STATUS_INFORUNBD
+    */
+   /* @todo: can SCIP_STATUS_UNKNOWN happen, too? */
+   assert((SCIPgetStatus(pricingprob) >= SCIP_STATUS_USERINTERRUPT && SCIPgetStatus(pricingprob) <= SCIP_STATUS_BESTSOLLIMIT)
+          || SCIPgetStatus(pricingprob) == SCIP_STATUS_OPTIMAL
+          || SCIPgetStatus(pricingprob) == SCIP_STATUS_INFEASIBLE
+          || SCIPgetStatus(pricingprob) == SCIP_STATUS_UNBOUNDED
+          || SCIPgetStatus(pricingprob) == SCIP_STATUS_INFORUNBD);
+
+   /* translate SCIP solution status to GCG pricing status */
+   switch( SCIPgetStatus(pricingprob) )
+   {
+   case SCIP_STATUS_USERINTERRUPT:
+     SCIPdebugMessage("  -> interrupted, %d solutions found\n", SCIPgetNSols(pricingprob)); /*lint -fallthrough*/
+   case SCIP_STATUS_UNKNOWN:
+   case SCIP_STATUS_TOTALNODELIMIT:
+   case SCIP_STATUS_TIMELIMIT:
+   case SCIP_STATUS_MEMLIMIT:
+   case SCIP_STATUS_BESTSOLLIMIT:
+     return GCG_PRICINGSTATUS_UNKNOWN;
+
+   case SCIP_STATUS_NODELIMIT:
+   case SCIP_STATUS_STALLNODELIMIT:
+   case SCIP_STATUS_GAPLIMIT:
+   case SCIP_STATUS_SOLLIMIT:
+     return GCG_PRICINGSTATUS_SOLVERLIMIT;
+
+   case SCIP_STATUS_OPTIMAL:
+     return GCG_PRICINGSTATUS_OPTIMAL;
+
+   case SCIP_STATUS_INFEASIBLE:
+     return GCG_PRICINGSTATUS_INFEASIBLE;
+
+   case SCIP_STATUS_UNBOUNDED:
+   case SCIP_STATUS_INFORUNBD:
+     return GCG_PRICINGSTATUS_UNBOUNDED;
+
+   default:
+     SCIPerrorMessage("invalid SCIP status of pricing problem: %d\n", SCIPgetStatus(pricingprob));
+     return GCG_PRICINGSTATUS_UNKNOWN;
+   }
+}
+
+SCIP_RETCODE getColumnsFromPricingprob(
+   GCG*                  gcg,                /**< GCG data structure */
+   SCIP*                 pricingprob,        /**< pricing problem SCIP data structure */
+   SCIP*                 subproblem,         /**< SCIP data structure that contains the actual solution (if NULL pricingprob will be used) */
+   SCIP_HASHMAP*         varmap,             /**< mapping of pricingprob vars to subproblem vars (can be NULL if subproblem is NULL) */
+   int                   probnr,             /**< problem number */
+   SCIP_Bool             checksols           /**< should solutions be checked extensively */
+)
+{
+   SCIP_SOL** probsols;
+   int nprobsols;
+   SCIP* solprob;
+
+   int s;
+
+   if (subproblem == NULL)
+      solprob = pricingprob;
+   else
+      solprob = subproblem;
+
+   probsols = SCIPgetSols(solprob);
+   nprobsols = SCIPgetNSols(solprob);
+
+   for( s = 0; s < nprobsols; s++ )
+   {
+      GCG_COL* col;
+      SCIP_Bool feasible;
+      assert(probsols[s] != NULL);
+      SCIP_CALL( SCIPcheckSolOrig(solprob, probsols[s], &feasible, FALSE, FALSE) );
+
+      if( !feasible )
+      {
+         SCIPwarningMessage(solprob, "solution %d of pricing problem %d (%s) not feasible:\n", s, probnr, SCIPgetProbName(solprob));
+         SCIP_CALL( SCIPcheckSolOrig(solprob, probsols[s], &feasible, TRUE, TRUE) );
+         continue;
+      }
+
+      /* check whether the solution is equal to one of the previous solutions */
+      if( checksols )
+      {
+         SCIP_Bool isnew;
+
+         SCIP_CALL( checkSolNew(solprob, probsols, s, &isnew) );
+
+         if( !isnew )
+            continue;
+      }
+
+      /* Check whether the pricing problem solution has infinite values; if not, transform it to a column */
+      if( !solutionHasInfiniteValue(solprob, probsols[s]) )
+      {
+         SCIP_CALL( GCGcreateGcgColFromSol(gcg, pricingprob, subproblem, varmap, &col, probnr, probsols[s], FALSE, SCIPinfinity(solprob)) );
+         SCIP_CALL( GCGpricerAddCol(gcg, col) );
+      }
+      /* If the best solution has infinite values, try to repair it */
+      else if( s == 0 )
+      {
+         SCIP_SOL* newsol;
+         SCIP_Bool success;
+
+         newsol = NULL;
+         success = FALSE;
+
+         SCIPdebugMessage("solution has infinite values, create a copy with finite values\n");
+
+         SCIP_CALL( SCIPcreateFiniteSolCopy(solprob, &newsol, probsols[0], &success) );
+         assert(success);
+         assert(newsol != NULL);
+
+         SCIP_CALL( GCGcreateGcgColFromSol(gcg, pricingprob, subproblem, varmap, &col, probnr, newsol, FALSE, SCIPinfinity(solprob)) );
+         SCIP_CALL( GCGpricerAddCol(gcg, col) );
+      }
+   }
+
+   return SCIP_OKAY;
+}
+
 /** creates the mip solver for pricing problems and includes it in GCG */
 SCIP_RETCODE GCGincludeSolverMip(
-   SCIP*                 scip                /**< SCIP data structure */
+   GCG*                  gcg                 /**< GCG data structure */
    )
 {
    SCIP* origprob;
+   SCIP* masterprob;
    GCG_SOLVERDATA* solverdata;
 
-   origprob = GCGmasterGetOrigprob(scip);
+   origprob = GCGgetOrigprob(gcg);
+   masterprob = GCGgetDwMasterprob(gcg);
 
-   SCIP_CALL( SCIPallocMemory(scip, &solverdata) );
+   SCIP_CALL( SCIPallocBlockMemory(masterprob, &solverdata) );
    solverdata->settingsfile = NULL;
 
-   SCIP_CALL( GCGpricerIncludeSolver(scip, SOLVER_NAME, SOLVER_DESC, SOLVER_PRIORITY,
+   SCIP_CALL( GCGpricerIncludeSolver(gcg, SOLVER_NAME, SOLVER_DESC, SOLVER_PRIORITY,
          SOLVER_HEURENABLED, SOLVER_EXACTENABLED,
          solverUpdateMip, solverSolveMip, solverSolveHeurMip, solverFreeMip, solverInitMip, solverExitMip,
          solverInitsolMip, solverExitsolMip, solverdata) );
@@ -732,6 +786,10 @@ SCIP_RETCODE GCGincludeSolverMip(
          "settings file for pricing problems",
          &solverdata->settingsfile, TRUE, DEFAULT_SETTINGSFILE, NULL, NULL) );
 
+   SCIP_CALL( SCIPaddBoolParam(origprob, "pricingsolver/mip/enablewarnings",
+         "should warnings of pricing problems be printed",
+         &solverdata->enablewarnings, FALSE, DEFAULT_ENABLE_WARNINGS, NULL, NULL) );
+
 
    return SCIP_OKAY;
 }
