@@ -1,27 +1,28 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                           */
-/*                  This file is part of the program                         */
+/*                  This file is part of the program and library             */
 /*          GCG --- Generic Column Generation                                */
 /*                  a Dantzig-Wolfe decomposition based extension            */
 /*                  of the branch-cut-and-price framework                    */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/* Copyright (C) 2010-2025 Operations Research, RWTH Aachen University       */
+/* Copyright (C) 2010-2026 Operations Research, RWTH Aachen University       */
 /*                         Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software; you can redistribute it and/or             */
-/* modify it under the terms of the GNU Lesser General Public License        */
-/* as published by the Free Software Foundation; either version 3            */
-/* of the License, or (at your option) any later version.                    */
+/*  Licensed under the Apache License, Version 2.0 (the "License");          */
+/*  you may not use this file except in compliance with the License.         */
+/*  You may obtain a copy of the License at                                  */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*      http://www.apache.org/licenses/LICENSE-2.0                           */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program; if not, write to the Free Software               */
-/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.*/
+/*  Unless required by applicable law or agreed to in writing, software      */
+/*  distributed under the License is distributed on an "AS IS" BASIS,        */
+/*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. */
+/*  See the License for the specific language governing permissions and      */
+/*  limitations under the License.                                           */
+/*                                                                           */
+/*  You should have received a copy of the Apache-2.0 license                */
+/*  along with GCG; see the file LICENSE. If not visit gcg.or.rwth-aachen.de.*/
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -37,10 +38,10 @@
 #ifndef GCG_ROWGRAPH_WEIGHTED_DEF_H_
 #define GCG_ROWGRAPH_WEIGHTED_DEF_H_
 
-#include "rowgraph_weighted.h"
-#include "graph_gcg.h"
-#include "graphalgorithms.h"
-#include "priority_graph.h"
+#include "graph/rowgraph_weighted.h"
+#include "graph/graph_gcg.h"
+#include "graph/graphalgorithms.h"
+#include "graph/priority_graph.h"
 #include <algorithm>
 #include <iostream>
 #include <vector>
@@ -55,9 +56,9 @@ namespace gcg {
 
 template <class T>
 RowGraphWeighted<T>::RowGraphWeighted(
-   SCIP*                 scip,              /**< SCIP data structure */
+   GCG*                  gcgstruct,         /**< GCG data structure */
    Weights               w                  /**< weights for the given graph */
-   ) : RowGraph<T>(scip,w)
+   ) : RowGraph<T>(gcgstruct,w)
 {
    this->name = string("rowgraph_weighted");
    n_blocks = -1;
@@ -86,6 +87,7 @@ SCIP_RETCODE RowGraphWeighted<T>::createFromMatrix(
    int k;
    int l;
    SCIP_Bool success;
+   SCIP* scip = GCGgetOrigprob(this->gcg);
 
    assert(conss != NULL);
    assert(vars != NULL);
@@ -103,7 +105,7 @@ SCIP_RETCODE RowGraphWeighted<T>::createFromMatrix(
       SCIP_VAR** curvars1 = NULL;
 
       int ncurvars1;
-      SCIP_CALL( SCIPgetConsNVars(this->scip_, conss[i], &ncurvars1, &success) );
+      SCIP_CALL( SCIPgetConsNVars(scip, conss[i], &ncurvars1, &success) );
       assert(success);
       if( ncurvars1 == 0 )
          continue;
@@ -112,8 +114,8 @@ SCIP_RETCODE RowGraphWeighted<T>::createFromMatrix(
        * may work as is, as we are copying the constraint later regardless
        * if there are variables in it or not
        */
-      SCIP_CALL( SCIPallocBufferArray(this->scip_, &curvars1, ncurvars1) );
-      SCIP_CALL( SCIPgetConsVars(this->scip_, conss[i], curvars1, ncurvars1, &success) );
+      SCIP_CALL( SCIPallocBufferArray(scip, &curvars1, ncurvars1) );
+      SCIP_CALL( SCIPgetConsVars(scip, conss[i], curvars1, ncurvars1, &success) );
       assert(success);
 
       /* go through all constraints again */
@@ -121,7 +123,7 @@ SCIP_RETCODE RowGraphWeighted<T>::createFromMatrix(
       {
          SCIP_VAR** curvars2 = NULL;
          int ncurvars2;
-         SCIP_CALL( SCIPgetConsNVars(this->scip_, conss[j], &ncurvars2, &success) );
+         SCIP_CALL( SCIPgetConsNVars(scip, conss[j], &ncurvars2, &success) );
          assert(success);
          if( ncurvars2 == 0 )
             continue;
@@ -131,8 +133,8 @@ SCIP_RETCODE RowGraphWeighted<T>::createFromMatrix(
           * may work as is, as we are copying the constraint later regardless
           * if there are variables in it or not
           */
-         SCIP_CALL( SCIPallocBufferArray(this->scip_, &curvars2, ncurvars2) );
-         SCIP_CALL( SCIPgetConsVars(this->scip_, conss[j], curvars2, ncurvars2, &success) );
+         SCIP_CALL( SCIPallocBufferArray(scip, &curvars2, ncurvars2) );
+         SCIP_CALL( SCIPgetConsVars(scip, conss[j], curvars2, ncurvars2, &success) );
          assert(success);
 
 
@@ -150,7 +152,7 @@ SCIP_RETCODE RowGraphWeighted<T>::createFromMatrix(
             if( !GCGisVarRelevant(curvars1[k]) )
                continue;
 
-            if( SCIPgetStage(this->scip_) >= SCIP_STAGE_TRANSFORMED)
+            if( SCIPgetStage(scip) >= SCIP_STAGE_TRANSFORMED)
                var1 = SCIPvarGetProbvar(curvars1[k]);
             else
                var1 = curvars1[k];
@@ -168,7 +170,7 @@ SCIP_RETCODE RowGraphWeighted<T>::createFromMatrix(
                if( !GCGisVarRelevant(curvars2[l]) )
                   continue;
 
-               if( SCIPgetStage(this->scip_) >= SCIP_STAGE_TRANSFORMED)
+               if( SCIPgetStage(scip) >= SCIP_STAGE_TRANSFORMED)
                   var2 = SCIPvarGetProbvar(curvars2[l]);
                else
                   var2 = curvars2[l];
@@ -195,9 +197,9 @@ SCIP_RETCODE RowGraphWeighted<T>::createFromMatrix(
             this->graph.addEdge(i, j, edge_weight);
          }
 
-         SCIPfreeBufferArray(this->scip_, &curvars2);
+         SCIPfreeBufferArray(scip, &curvars2);
       }
-      SCIPfreeBufferArray(this->scip_, &curvars1);
+      SCIPfreeBufferArray(scip, &curvars1);
    }
 
    if(dist == INTERSECTION)
@@ -420,6 +422,7 @@ double RowGraphWeighted<T>::calculateSimilarity(int _a, int _b, int _c, DISTANCE
 template <>
 SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcess(vector<int>& labels, bool enabled)
 {
+   SCIP* scip = GCGgetOrigprob(this->gcg);
    assert((int)labels.size() == graph.getNNodes());
    set<int> diff_blocks_beginning;
    for(auto curr_int = labels.begin(), end = labels.end(); curr_int != end; ++curr_int)
@@ -445,7 +448,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcess(vector<int>& labels, bool e
       // item i saves number which appears most often in the all_labels_in_col[i]
       vector<int> col_labels(this->nvars, -1);
 
-      SCIP_CONS** conss = SCIPgetConss(this->scip_);
+      SCIP_CONS** conss = SCIPgetConss(scip);
       SCIP_Bool success;
 
       // For each var save the labels of all the constraints where this var appears.
@@ -453,7 +456,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcess(vector<int>& labels, bool e
       {
          SCIP_VAR** curvars = NULL;
          int ncurvars;
-         SCIP_CALL( SCIPgetConsNVars(this->scip_, conss[i], &ncurvars, &success) );
+         SCIP_CALL( SCIPgetConsNVars(scip, conss[i], &ncurvars, &success) );
          assert(success);
          if( ncurvars == 0 )
             continue;
@@ -462,8 +465,8 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcess(vector<int>& labels, bool e
           * may work as is, as we are copying the constraint later regardless
           * if there are variables in it or not
           */
-         SCIP_CALL( SCIPallocBufferArray(this->scip_, &curvars, ncurvars) );
-         SCIP_CALL( SCIPgetConsVars(this->scip_, conss[i], curvars, ncurvars, &success) );
+         SCIP_CALL( SCIPallocBufferArray(scip, &curvars, ncurvars) );
+         SCIP_CALL( SCIPgetConsVars(scip, conss[i], curvars, ncurvars, &success) );
          assert(success);
 
          for(auto j = 0; j < ncurvars; j++)
@@ -473,7 +476,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcess(vector<int>& labels, bool e
             if( !GCGisVarRelevant(curvars[j]) )
                continue;
 
-            if( SCIPgetStage(this->scip_) >= SCIP_STAGE_TRANSFORMED)
+            if( SCIPgetStage(scip) >= SCIP_STAGE_TRANSFORMED)
                var = SCIPvarGetProbvar(curvars[j]);
             else
                var = curvars[j];
@@ -487,7 +490,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcess(vector<int>& labels, bool e
             all_labels_in_col[varIndex].push_back(labels[i]);
             all_label_occ_in_col[varIndex][labels[i]]++;
          }
-         SCIPfreeBufferArray(this->scip_, &curvars);
+         SCIPfreeBufferArray(scip, &curvars);
       }
 
       // fill the col_labels
@@ -509,7 +512,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcess(vector<int>& labels, bool e
       {
          SCIP_VAR** curvars1 = NULL;
          int ncurvars1;
-         SCIP_CALL( SCIPgetConsNVars(this->scip_, conss[i], &ncurvars1, &success) );
+         SCIP_CALL( SCIPgetConsNVars(scip, conss[i], &ncurvars1, &success) );
          assert(success);
          if( ncurvars1 == 0 )
             continue;
@@ -518,8 +521,8 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcess(vector<int>& labels, bool e
           * may work as is, as we are copying the constraint later regardless
           * if there are variables in it or not
           */
-         SCIP_CALL( SCIPallocBufferArray(this->scip_, &curvars1, ncurvars1) );
-         SCIP_CALL( SCIPgetConsVars(this->scip_, conss[i], curvars1, ncurvars1, &success) );
+         SCIP_CALL( SCIPallocBufferArray(scip, &curvars1, ncurvars1) );
+         SCIP_CALL( SCIPgetConsVars(scip, conss[i], curvars1, ncurvars1, &success) );
          assert(success);
 
          for(auto j = 0; j < ncurvars1; j++)
@@ -529,7 +532,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcess(vector<int>& labels, bool e
             if( !GCGisVarRelevant(curvars1[j]) )
                continue;
 
-            if( SCIPgetStage(this->scip_) >= SCIP_STAGE_TRANSFORMED)
+            if( SCIPgetStage(scip) >= SCIP_STAGE_TRANSFORMED)
                var1 = SCIPvarGetProbvar(curvars1[j]);
             else
                var1 = curvars1[j];
@@ -558,7 +561,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcess(vector<int>& labels, bool e
                col_labels[varIndex] = pr->first;
             }
          }
-         SCIPfreeBufferArray(this->scip_, &curvars1);
+         SCIPfreeBufferArray(scip, &curvars1);
 
       }
    }
@@ -821,6 +824,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcessForPartialGraph(gcg::DETPROB
 template <>
 SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcessStableSet(vector<int>& labels, bool enabled)
 {
+   SCIP* scip = GCGgetOrigprob(this->gcg);
    assert((int)labels.size() == graph.getNNodes());
    set<int> diff_blocks_beginning;
    for(auto curr_int = labels.begin(), end = labels.end(); curr_int != end; ++curr_int)
@@ -844,7 +848,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcessStableSet(vector<int>& label
       vector< vector<int> > all_ind_in_col(this->nvars);
 
 
-      SCIP_CONS** conss = SCIPgetConss(this->scip_);
+      SCIP_CONS** conss = SCIPgetConss(scip);
       SCIP_Bool success;
 
       /* go through all constraints */
@@ -853,7 +857,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcessStableSet(vector<int>& label
          SCIP_VAR** curvars1 = NULL;
 
          int ncurvars1;
-         SCIP_CALL( SCIPgetConsNVars(this->scip_, conss[i], &ncurvars1, &success) );
+         SCIP_CALL( SCIPgetConsNVars(scip, conss[i], &ncurvars1, &success) );
          assert(success);
          if( ncurvars1 == 0 )
             continue;
@@ -862,8 +866,8 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcessStableSet(vector<int>& label
           * may work as is, as we are copying the constraint later regardless
           * if there are variables in it or not
           */
-         SCIP_CALL( SCIPallocBufferArray(this->scip_, &curvars1, ncurvars1) );
-         SCIP_CALL( SCIPgetConsVars(this->scip_, conss[i], curvars1, ncurvars1, &success) );
+         SCIP_CALL( SCIPallocBufferArray(scip, &curvars1, ncurvars1) );
+         SCIP_CALL( SCIPgetConsVars(scip, conss[i], curvars1, ncurvars1, &success) );
          assert(success);
 
          /* go through all constraints */
@@ -873,7 +877,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcessStableSet(vector<int>& label
             SCIP_VAR** curvars2 = NULL;
             SCIP_Bool continueloop;
             int ncurvars2;
-            SCIP_CALL( SCIPgetConsNVars(this->scip_, conss[j], &ncurvars2, &success) );
+            SCIP_CALL( SCIPgetConsNVars(scip, conss[j], &ncurvars2, &success) );
             assert(success);
             if( ncurvars2 == 0 )
                continue;
@@ -883,8 +887,8 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcessStableSet(vector<int>& label
              * may work as is, as we are copying the constraint later regardless
              * if there are variables in it or not
              */
-            SCIP_CALL( SCIPallocBufferArray(this->scip_, &curvars2, ncurvars2) );
-            SCIP_CALL( SCIPgetConsVars(this->scip_, conss[j], curvars2, ncurvars2, &success) );
+            SCIP_CALL( SCIPallocBufferArray(scip, &curvars2, ncurvars2) );
+            SCIP_CALL( SCIPgetConsVars(scip, conss[j], curvars2, ncurvars2, &success) );
             assert(success);
 
 
@@ -899,7 +903,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcessStableSet(vector<int>& label
                if( !GCGisVarRelevant(curvars1[k]) )
                   continue;
 
-               if( SCIPgetStage(this->scip_) >= SCIP_STAGE_TRANSFORMED)
+               if( SCIPgetStage(scip) >= SCIP_STAGE_TRANSFORMED)
                   var1 = SCIPvarGetProbvar(curvars1[k]);
                else
                   var1 = curvars1[k];
@@ -917,7 +921,7 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcessStableSet(vector<int>& label
                   if( !GCGisVarRelevant(curvars2[l]) )
                      continue;
 
-                  if( SCIPgetStage(this->scip_) >= SCIP_STAGE_TRANSFORMED)
+                  if( SCIPgetStage(scip) >= SCIP_STAGE_TRANSFORMED)
                      var2 = SCIPvarGetProbvar(curvars2[l]);
                   else
                      var2 = curvars2[l];
@@ -944,9 +948,9 @@ SCIP_RETCODE RowGraphWeighted<GraphGCG>::postProcessStableSet(vector<int>& label
                if(continueloop)
                   break;
             }
-            SCIPfreeBufferArray(this->scip_, &curvars2);
+            SCIPfreeBufferArray(scip, &curvars2);
          }
-         SCIPfreeBufferArray(this->scip_, &curvars1);
+         SCIPfreeBufferArray(scip, &curvars1);
       }
 
       // run greedy heuristic for stable set
