@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2025 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2026 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -493,7 +493,7 @@ char* SCIPdecompPrintStats(
       decomp->statscomplete ? "" :
                "(approximately: graph construction hit size limit)");
 
-   return strbuf;
+   return strbuf;  /* cppcheck-suppress uninitvar */
 }
 
 /** creates a decomposition storage */
