@@ -4,7 +4,7 @@
 /*                    UG --- Ubquity Generator Framework                     */
 /*                                                                           */
 /*  Copyright Written by Yuji Shinano <shinano@zib.de>,                      */
-/*            Copyright (C) 2021-2024 by Zuse Institute Berlin,              */
+/*            Copyright (C) 2021-2026 by Zuse Institute Berlin,              */
 /*            licensed under LGPL version 3 or later.                        */
 /*            Commercial licenses are available through <licenses@zib.de>    */
 /*                                                                           */
@@ -66,7 +66,7 @@ outputCommandLineMessages(
       )
 {
    std::cout << std::endl;
-   std::cout << "syntax: " << argv[0] << " #MPI_processes(#solvers + 1) parascip_param_file problem_file_name "
+   std::cout << "syntax: mpirun(command provided by MPI) #MPI_processes(#solvers + 1) " << argv[0] << " parascip_param_file problem_file_name "
              << "[-l <logfile>] [-q] [-sl <settings>] [-s <settings>] [-sr <root_settings>] [-w <prefix_warm>] [-sth <number>] [-fsol <solution_file>] [-isol <initial solution file]" << std::endl;
    std::cout << "  -l <logfile>        : copy output into log file" << std::endl;
    std::cout << "  -q                  : suppress screen messages" << std::endl;
@@ -77,6 +77,10 @@ outputCommandLineMessages(
    std::cout << "  -fsol <solution file> : specify output solution file" << std::endl;
    std::cout << "  -isol <intial solution file> : specify initial solution file" << std::endl;
    std::cout << "File names need to be specified by full path form." << std::endl;
+
+//   std::cout << "TagKeepRacing == " << TagKeepRacing << std::endl;
+//   std::cout << "TagTerminateSolvingToRestart == " << TagTerminateSolvingToRestart << std::endl;
+//   std::cout << "TagSelfSplitFinished == " << TagSelfSplitFinished << std::endl;
 }
 
 void
