@@ -4,7 +4,7 @@
 /*                    UG --- Ubquity Generator Framework                     */
 /*                                                                           */
 /*  Copyright Written by Yuji Shinano <shinano@zib.de>,                      */
-/*            Copyright (C) 2021-2024 by Zuse Institute Berlin,              */
+/*            Copyright (C) 2021-2026 by Zuse Institute Berlin,              */
 /*            licensed under LGPL version 3 or later.                        */
 /*            Commercial licenses are available through <licenses@zib.de>    */
 /*                                                                           */
@@ -620,7 +620,11 @@ ScipParaInstance::getActiveVariables(
 
    if( transformed )
    {
+#if SCIP_APIVERSION >= 128
+      SCIP_CALL_ABORT( SCIPgetProbvarLinearSum(scip, vars, scalars, nvars, *nvars, constant, &requiredsize) );
+#else
       SCIP_CALL_ABORT( SCIPgetProbvarLinearSum(scip, vars, scalars, nvars, *nvars, constant, &requiredsize, TRUE) );
+#endif
 
       if( requiredsize > *nvars )
       {
@@ -628,7 +632,11 @@ ScipParaInstance::getActiveVariables(
          SCIP_CALL_ABORT( SCIPreallocBufferArray(scip, &vars, *nvars ) );
          SCIP_CALL_ABORT( SCIPreallocBufferArray(scip, &scalars, *nvars ) );
 
+#if SCIP_APIVERSION >= 128
+         SCIP_CALL_ABORT( SCIPgetProbvarLinearSum(scip, vars, scalars, nvars, *nvars, constant, &requiredsize) );
+#else
          SCIP_CALL_ABORT( SCIPgetProbvarLinearSum(scip, vars, scalars, nvars, *nvars, constant, &requiredsize, TRUE) );
+#endif
          assert( requiredsize <= *nvars );
       }
    }
@@ -1540,26 +1548,22 @@ ScipParaInstance::createProblem(
          // includeUserPlugins(scip);
 
          /* copy all plugins and settings */
-   #if SCIP_VERSION == 211 && SCIP_SUBVERSION == 0
+#if SCIP_APIVERSION >= 131
          SCIP_CALL_ABORT( SCIPcopyPlugins(tempScip, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-               TRUE, TRUE, TRUE, TRUE, &success) );
-   #else
-      #if SCIP_APIVERSION >= 100
-         #if SCIP_APIVERSION >= 101
-             SCIP_CALL_ABORT( SCIPcopyPlugins(tempScip, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-                   TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-         #else
-             SCIP_CALL_ABORT( SCIPcopyPlugins(tempScip, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-                   TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-         #endif
-      #elif SCIP_APIVERSION >= 17
+               TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 101
+         SCIP_CALL_ABORT( SCIPcopyPlugins(tempScip, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+               TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 100
+         SCIP_CALL_ABORT( SCIPcopyPlugins(tempScip, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+               TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 17
          SCIP_CALL_ABORT( SCIPcopyPlugins(tempScip, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
                TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-      #else
+#else
          SCIP_CALL_ABORT( SCIPcopyPlugins(tempScip, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
                TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-      #endif
-   #endif
+#endif
          SCIP_CALL_ABORT( SCIPcopyParamSettings(tempScip, scip) );
 
          /* create the variable mapping hash map */
