@@ -4,7 +4,7 @@
 /*                    UG --- Ubquity Generator Framework                     */
 /*                                                                           */
 /*  Copyright Written by Yuji Shinano <shinano@zib.de>,                      */
-/*            Copyright (C) 2021-2024 by Zuse Institute Berlin,              */
+/*            Copyright (C) 2021-2026 by Zuse Institute Berlin,              */
 /*            licensed under LGPL version 3 or later.                        */
 /*            Commercial licenses are available through <licenses@zib.de>    */
 /*                                                                           */
@@ -65,25 +65,21 @@ ScipParaInstance::copyScipEnvironment(
    SCIP_Bool success = TRUE;
 
    /* copy all plugins and settings */
-#if SCIP_VERSION == 211 && SCIP_SUBVERSION == 0
+#if SCIP_APIVERSION >= 131
    SCIP_CALL_ABORT( SCIPcopyPlugins(scip, *targetscip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-         TRUE, TRUE, TRUE, TRUE, &success) );
+         TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 101
+   SCIP_CALL_ABORT( SCIPcopyPlugins(scip, *targetscip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+         TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 100
+   SCIP_CALL_ABORT( SCIPcopyPlugins(scip, *targetscip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+         TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 17
+   SCIP_CALL_ABORT( SCIPcopyPlugins(scip, *targetscip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+         TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
 #else
-   #if SCIP_APIVERSION >= 100
-      #if SCIP_APIVERSION >= 101
-          SCIP_CALL_ABORT( SCIPcopyPlugins(scip, *targetscip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-                TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-      #else
-          SCIP_CALL_ABORT( SCIPcopyPlugins(scip, *targetscip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-                TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-      #endif
-   #elif SCIP_APIVERSION >= 17
-      SCIP_CALL_ABORT( SCIPcopyPlugins(scip, *targetscip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-            TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-   #else
-      SCIP_CALL_ABORT( SCIPcopyPlugins(scip, *targetscip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-            TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-   #endif
+   SCIP_CALL_ABORT( SCIPcopyPlugins(scip, *targetscip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+         TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
 #endif
    SCIP_CALL_ABORT( SCIPcopyParamSettings(scip, *targetscip) );
 
@@ -202,25 +198,21 @@ ScipParaInstanceTh::bcast(
       SCIP_CALL_ABORT( SCIPcreate(&tempScip) );
 
       /* copy all plugins and settings */
-#if SCIP_VERSION == 211 && SCIP_SUBVERSION == 0
+#if SCIP_APIVERSION >= 131
       SCIP_CALL_ABORT( SCIPcopyPlugins(scip, tempScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-            TRUE, TRUE, TRUE, TRUE, &success) );
-#else
-   #if SCIP_APIVERSION >= 100
-      #if SCIP_APIVERSION >= 101
-          SCIP_CALL_ABORT( SCIPcopyPlugins(scip, tempScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-                TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-      #else
-          SCIP_CALL_ABORT( SCIPcopyPlugins(scip, tempScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-                TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-      #endif
-   #elif SCIP_APIVERSION >= 17
+            TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 101
+      SCIP_CALL_ABORT( SCIPcopyPlugins(scip, tempScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+            TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 100
+      SCIP_CALL_ABORT( SCIPcopyPlugins(scip, tempScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+            TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 17
       SCIP_CALL_ABORT( SCIPcopyPlugins(scip, tempScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
             TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-   #else
+#else
       SCIP_CALL_ABORT( SCIPcopyPlugins(scip, tempScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
             TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-   #endif
 #endif
       SCIP_CALL_ABORT( SCIPcopyParamSettings(scip, tempScip) );
 
@@ -294,14 +286,23 @@ ScipParaInstanceTh::bcast(
             SCIPhashmapFree(&varmap);
          }
          SCIPfree(&tempScip);
-         std::cout << "** ParaScipInstance copy does not increase the number of variables. **" << std::endl;
+         SCIPinfoMessage(scip, NULL, "** ParaScipInstance copy does not increase the number of variables. **\n");
       }
       else
       {
          assert(n > nVars);
-         std::cout << "** ParaScipInstance copy increased the number of variables. **" << std::endl;
+         SCIPinfoMessage(scip, NULL, "** ParaScipInstance copy increased the number of variables. **\n");
          varIndexRange = SCIPgetNTotalVars(tempScip);
          copyIncreasedVariables = true;
+         if( SCIPgetNConss(scip) > 0 )
+         {
+            SCIPhashmapFree(&conssmap);
+         }
+         if( SCIPgetNVars(scip) > 0 )
+         {
+            SCIPhashmapFree(&varmap);
+         }
+         SCIPfree(&tempScip);
          /**
          // mapToOriginalIndecies = new int[n];
          mapToOriginalIndecies = new int[SCIPgetNTotalVars(tempScip)];   // need to allocate enough for SCIPvarGetIndex(copyvar)
@@ -385,30 +386,26 @@ ScipParaInstanceTh::bcast(
          SCIP_Bool success = TRUE;
 
          /* copy all plugins and settings */
-#if SCIP_VERSION == 211 && SCIP_SUBVERSION == 0
+#if SCIP_APIVERSION >= 131
          SCIP_CALL_ABORT( SCIPcopyPlugins(received, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-               TRUE, TRUE, TRUE, TRUE, &success) );
+               TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 101
+         SCIP_CALL_ABORT( SCIPcopyPlugins(received, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+               TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 100
+         SCIP_CALL_ABORT( SCIPcopyPlugins(received, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+               TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
+#elif SCIP_APIVERSION >= 17
+         SCIP_CALL_ABORT( SCIPcopyPlugins(received, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+               TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
 #else
-   #if SCIP_APIVERSION >= 100
-      #if SCIP_APIVERSION >= 101
-          SCIP_CALL_ABORT( SCIPcopyPlugins(received, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-                TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-      #else
-          SCIP_CALL_ABORT( SCIPcopyPlugins(received, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-                TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-      #endif
-   #elif SCIP_APIVERSION >= 17
-      SCIP_CALL_ABORT( SCIPcopyPlugins(received, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-            TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-   #else
-      SCIP_CALL_ABORT( SCIPcopyPlugins(received, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-            TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-   #endif
+         SCIP_CALL_ABORT( SCIPcopyPlugins(received, scip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+               TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
 #endif
          // std::cout <<  "copy plugins : success = " << success << std::endl;
          if( !success )
          {
-             std::cout << "Error in SCIPcopyPlugins" << std::endl;
+             SCIPerrorMessage("Error in SCIPcopyPlugins\n");
              abort();
          }
          SCIP_CALL_ABORT( SCIPcopyParamSettings(received, scip) );
@@ -624,29 +621,25 @@ ScipParaInstance::createProblem(
       /****************************************/
       /* the following code need to be tested */
       /****************************************/
-      std::cout << "* If you use check mechanism, you should check the following codes. *" << std::endl;
+      SCIPinfoMessage(scip, NULL, "* If you use check mechanism, you should check the following codes. *\n");
       SCIP_Bool success = TRUE;
       char probname[SCIP_MAXSTRLEN];
       /* copy all plugins and settings */
-#if SCIP_VERSION == 211 && SCIP_SUBVERSION == 0
+#if SCIP_APIVERSION >= 131
       SCIP_CALL_ABORT( SCIPcopyPlugins(scip, inScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-          TRUE, TRUE, TRUE, TRUE, &success) );
+            TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, &success) );
+#elif SCIP_APIVERSION >= 101
+      SCIP_CALL_ABORT( SCIPcopyPlugins(scip, inScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+            TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, &success) );
+#elif SCIP_APIVERSION >= 100
+      SCIP_CALL_ABORT( SCIPcopyPlugins(scip, inScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+           TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, &success) );
+#elif SCIP_APIVERSION >= 17
+      SCIP_CALL_ABORT( SCIPcopyPlugins(scip, inScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
+            TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, &success) );
 #else
-   #if SCIP_APIVERSION >= 100
-      #if SCIP_APIVERSION >= 101
-          SCIP_CALL_ABORT( SCIPcopyPlugins(scip, inScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-                TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-      #else
-          SCIP_CALL_ABORT( SCIPcopyPlugins(scip, inScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-                TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-      #endif
-   #elif SCIP_APIVERSION >= 17
       SCIP_CALL_ABORT( SCIPcopyPlugins(scip, inScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-            TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-   #else
-      SCIP_CALL_ABORT( SCIPcopyPlugins(scip, inScip, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
-           TRUE, TRUE, TRUE, TRUE, FALSE, &success) );
-   #endif
+            TRUE, TRUE, TRUE, TRUE, TRUE, &success) );
 #endif
       SCIP_CALL_ABORT( SCIPcopyParamSettings(scip, inScip) );
 
@@ -719,7 +712,7 @@ ScipParaInstance::createProblem(
    }
    if( method == 2 )
    {
-      std::cout << "You should use instance transfer method 0 or 1!" << std::endl;
+      SCIPinfoMessage(scip, NULL, "You should use instance transfer method 0 or 1!\n");
       exit(0);
    }
 }
