@@ -4,7 +4,7 @@
 /*                    UG --- Ubquity Generator Framework                     */
 /*                                                                           */
 /*  Copyright Written by Yuji Shinano <shinano@zib.de>,                      */
-/*            Copyright (C) 2021-2024 by Zuse Institute Berlin,              */
+/*            Copyright (C) 2021-2026 by Zuse Institute Berlin,              */
 /*            licensed under LGPL version 3 or later.                        */
 /*            Commercial licenses are available through <licenses@zib.de>    */
 /*                                                                           */
@@ -410,18 +410,23 @@ ScipParaObjCommPointHdlr::scip_exec(
                   {
                      orgIndex = scipParaSolver->getOriginalIndex(orgIndex);
                   }
-                  PARA_COMM_CALL(
-                        paraComm->send((void *)&orgIndex, 1, UG::ParaINT, 0, UG::TagLbBoundTightenedIndex )
-                        );
-                  PARA_COMM_CALL(
-                        paraComm->send((void *)&newbound, 1, UG::ParaDOUBLE, 0, UG::TagLbBoundTightenedBound )
-                        );
-                  /*
-                  std::cout << "Rank " << paraComm->getRank()
-                        << ": send tightened lower bond. idx = " << index
-                        << ", bound = " << newbound
-                        << ", var status = " << SCIPvarGetStatus(SCIPeventGetVar(event))
-                        << std::endl;  */
+                  if( (!scipParaSolver->isCopyIncreasedVariables()) ||
+                      (scipParaSolver->isCopyIncreasedVariables() &&
+                      orgIndex >= 0 && orgIndex < scipParaSolver->getNOrgVars() ) )
+                  {
+                     PARA_COMM_CALL(
+                           paraComm->send((void *)&orgIndex, 1, UG::ParaINT, 0, UG::TagLbBoundTightenedIndex )
+                           );
+                     PARA_COMM_CALL(
+                           paraComm->send((void *)&newbound, 1, UG::ParaDOUBLE, 0, UG::TagLbBoundTightenedBound )
+                           );
+                     /*
+                     std::cout << "Rank " << paraComm->getRank()
+                           << ": send tightened lower bond. idx = " << index
+                           << ", bound = " << newbound
+                           << ", var status = " << SCIPvarGetStatus(SCIPeventGetVar(event))
+                           << std::endl;  */
+                  }
                }
             }
             else
