@@ -7,7 +7,7 @@
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*
- * Copyright (C) 2001-2024 by Thorsten Koch <koch@zib.de>
+ * Copyright (C) 2001-2025 by Thorsten Koch <koch@zib.de>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -233,6 +233,8 @@ extern CodeNode* i_set_new_tuple(CodeNode* self) expects_NONNULL returns_NONNULL
 extern CodeNode* i_set_new_elem(CodeNode* self) expects_NONNULL returns_NONNULL;
 //lint -sem(     i_set_proj, inout(1), 1p == 1, @P == 1P) 
 extern CodeNode* i_set_proj(CodeNode* self) expects_NONNULL returns_NONNULL;
+//lint -sem(     i_set_permute, inout(1), 1p == 1, @P == 1P) 
+extern CodeNode* i_set_permute(CodeNode* self) expects_NONNULL returns_NONNULL;
 //lint -sem(     i_set_pseudo, inout(1), 1p == 1, @P == 1P) 
 extern CodeNode* i_set_pseudo(CodeNode* self) expects_NONNULL returns_NONNULL;
 //lint -sem(     i_set_range, inout(1), 1p == 1, @P == 1P) 
